# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 28.4 `sparseig` Chapter.

``sparseig`` - Large Scale Eigenproblems

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module provides functions for computing **some** eigenvalues and
eigenvectors of large-scale (sparse) standard and generalized eigenvalue
problems.
It provides functions for:

    solution of symmetric eigenvalue problems;

    solution of nonsymmetric eigenvalue problems;

    solution of generalized symmetric-definite eigenvalue problems;

    solution of generalized nonsymmetric eigenvalue problems;

    solution of polynomial eigenvalue problems;

    partial singular value decomposition.

Functions are provided for both `real` and `complex` data.

ARPACK Functions
~~~~~~~~~~~~~~~~
The functions in this module whose names do not contain substring
'_feast_' have all been derived from the ARPACK software suite (see
Lehoucq `et al.` (1998)), a collection of Fortran 77 functions designed
to solve large scale eigenvalue problems.
The interfaces provided in this module have been chosen to combine ease
of use with the flexibility of the original ARPACK software.
The underlying iterative methods and algorithms remain essentially the
same as those in ARPACK and are described fully in Lehoucq `et al.`
(1998).

The algorithms used in the ARPACK functions are based upon an
algorithmic variant of the Arnoldi process called the Implicitly
Restarted Arnoldi Method.
For symmetric matrices, this reduces to a variant of the Lanczos process
called the Implicitly Restarted Lanczos Method.
These variants may be viewed as a synthesis of the Arnoldi/Lanczos
process with the Implicitly Shifted QR technique that is suitable for
large scale problems.
For many standard problems, a matrix factorization is not required.
Only the action of the matrix on a vector is needed.

The ARPACK functions can be used to find the eigenvalues with the
largest and/or smallest magnitudes, real part or imaginary part.

FEAST Functions
~~~~~~~~~~~~~~~
The functions in this module whose names contain substring '_feast_'
have been derived from the FEAST software suite (see Polizzi (2009)).
FEAST is a general purpose eigensolver for standard, generalized and
polynomial eigenvalue problems.
It is suitable for both sparse and dense matrices, and functions are
available for real, complex, symmetric, Hermitian and non-Hermitian
eigenvalue problems.
The FEAST algorithm requires you to specify a particular region of
interest in the complex plane within which eigenvalues are sought.
The algorithm then performs a numerical quadrature computation,
involving solving linear systems along a complex contour around the
region of interest.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12intro.html
"""

# NAG Copyright 2017-2022.

import collections as _collections
import logging as _logging
import sys as _sys

from ..base import utils

def real_init(n, nev, ncv):
    u"""
    Initialization function for (``real_iter``) computing selected
    eigenvalues and, optionally, eigenvectors of a real nonsymmetric
    sparse (standard or generalized) eigenproblem.

    ``real_init`` is a setup function in a suite of functions consisting
    of ``real_init``, ``real_iter``, ``real_proc``, ``real_option`` and
    ``real_monit``.
    It is used to find some of the eigenvalues (and optionally the
    corresponding eigenvectors) of a standard or generalized eigenvalue
    problem defined by real nonsymmetric matrices.

    The suite of functions is suitable for the solution of large sparse,
    standard or generalized, nonsymmetric eigenproblems where only a few
    eigenvalues from a selected range of the spectrum are required.

    For full information please refer to the NAG Library document for
    f12aa

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12aaf.html

    Parameters
    ----------
    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    nev : int
        The number of eigenvalues to be computed.

    ncv : int
        The number of Arnoldi basis vectors to use during the
        computation.

        At present there is no `a priori` analysis to guide the
        selection of `ncv` relative to `nev`.

        However, it is recommended that `ncv` >= 2*`nev`+1.

        If many problems of the same type are to be solved, you should
        experiment with increasing `ncv` while keeping `nev` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the work and storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to CPU time is problem
        dependent and must be determined empirically.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nev` = *<value>*.

            Constraint: `nev` > 0.

        (`errno` 3)
            On entry, `ncv` = *<value>*, `nev` = *<value>* and `n` =
            *<value>*.

            Constraint: `ncv` > `nev`+1 and `ncv` <= `n`.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, (and optionally the corresponding eigenvectors,
    x) of a standard eigenvalue problem Ax = lambda x, or of a
    generalized eigenvalue problem Ax = lambda Bx of order n, where n is
    large and the coefficient matrices A and B are sparse, real and
    nonsymmetric.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale dense, real and nonsymmetric problems.

    ``real_init`` is a setup function which must be called before
    ``real_iter``, the reverse communication iterative solver, and
    before ``real_option``, the options setting function. ``real_proc``
    is a post-processing function that must be called following a
    successful final exit from ``real_iter``, while ``real_monit`` can
    be used to return additional monitoring information during the
    computation.

    This setup function initializes the communication arrays, sets (to
    their default values) all options that can be set by you via the
    option setting function ``real_option``, and checks that the lengths
    of the communication arrays as passed by you are of sufficient
    length.
    For details of the options available and how to set them see [Other
    Parameters for real_option].

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_init'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nev.
        utils._BYREF_INT, # Input, ncv.
        utils._EngineIntArrayType.c_type, # Output, icomm[max(1, licomm)].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineFloat64ArrayType.c_type, # Output, comm[max(1, lcomm)].
        utils._BYREF_INT, # Input, lcomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nev = utils._EngineIntScalarType(
        nev, {'fun_name': fun_name, 'entity': 'nev'},
    ).to_ctypes()
    _ncv = utils._EngineIntScalarType(
        ncv, {'fun_name': fun_name, 'entity': 'ncv'},
    ).to_ctypes()
    _licomm = utils._EngineIntScalarType.c_type(140)
    _licomm_py = utils._EngineIntScalarType.to_py(_licomm)
    _icomm = utils._EngineIntArrayType.empty_ctypes(max(1, _licomm_py))
    _comm_py = {}
    _comm_py['icomm'] = _icomm
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ncv_py = utils._EngineIntScalarType.to_py(_ncv)
    _lcomm = utils._EngineIntScalarType(
        3*_n_py+3*_ncv_py*_ncv_py+6*_ncv_py+60, {'fun_name': fun_name, 'entity': '_lcomm'},
    ).to_ctypes()
    _lcomm_py = utils._EngineIntScalarType.to_py(_lcomm)
    _comm = utils._EngineFloat64ArrayType.empty_ctypes(max(1, _lcomm_py))
    _comm_py['comm'] = _comm
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _nev,
        _ncv,
        _icomm,
        _licomm,
        _comm,
        _lcomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nev > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncv = {:s}, nev = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: ncv > nev+1 and ncv <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = _comm_py
    _err_class.manage()
    return return_data

def real_iter(irevcm, resid, v, x, mx, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a real
    nonsymmetric sparse eigenproblem, reverse communication.

    ``real_iter`` is an iterative solver in a suite of functions
    consisting of ``real_init``, ``real_iter``, ``real_proc``,
    ``real_option`` and ``real_monit``.
    It is used to find some of the eigenvalues (and optionally the
    corresponding eigenvectors) of a standard or generalized eigenvalue
    problem defined by real nonsymmetric matrices.

    For full information please refer to the NAG Library document for
    f12ab

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12abf.html

    Parameters
    ----------
    irevcm : int
        `On initial entry`: `irevcm` = 0, otherwise an error condition
        will be raised.

        `On intermediate entry`: must be unchanged from its previous
        exit value. Changing `irevcm` to any other value between calls
        will result in an error.

    resid : float, shape (n), modified in place
        `On initial entry`: need not be set unless the option 'Initial
        Residual' has been set in a prior call to ``real_option`` in
        which case `resid` should contain an initial residual vector,
        possibly from a previous run.

        `On intermediate entry`: must be unchanged from its previous
        exit. Changing `resid` to any other value between calls may
        result in an error exit.

        `On intermediate exit`: contains the current residual vector.

        `On final exit`: contains the final residual vector.

    v : float, shape (n, ncv), modified in place
        `On initial entry`: need not be set.

        `On intermediate entry`: must be unchanged from its previous
        exit.

        `On intermediate exit`: contains the current set of Arnoldi
        basis vectors.

        `On final exit`: contains the final set of Arnoldi basis
        vectors.

    x : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `comm`['icomm'][(int(`comm`['comm'][0])+24)-1]=0:
        `comm`['icomm'][(int(`comm`['comm'][0])+1)-1]; otherwise: 1.

        `On initial entry`: need not be set, it is used as a convenient
        mechanism for accessing elements of `comm`['comm'].

        `On intermediate entry`: if 'Pointers' = YES, `x` need not be
        set.

        If 'Pointers' = NO, `x` must contain the result of y = op(x)
        when `irevcm` returns the value -1 or +1.

        It must return the real parts of the computed shifts when
        `irevcm` returns the value 3.

        `On intermediate exit`: if 'Pointers' = YES, `x` is not
        referenced.

        If 'Pointers' = NO, `x` contains the vector x when `irevcm`
        returns the value -1 or +1.

        `On final exit`: does not contain useful data.

    mx : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `comm`['icomm'][(int(`comm`['comm'][0])+24)-1]=0:
        `comm`['icomm'][(int(`comm`['comm'][0])+1)-1]; otherwise: 1.

        `On initial entry`: need not be set, it is used as a convenient
        mechanism for accessing elements of `comm`['comm'].

        `On intermediate entry`: if 'Pointers' = YES, `mx` need not be
        set.

        If 'Pointers' = NO, `mx` must contain the result of y = Bx when
        `irevcm` returns the value 2.

        It must return the imaginary parts of the computed shifts when
        `irevcm` returns the value 3.

        `On intermediate exit`: if 'Pointers' = YES, `mx` is not
        referenced.

        If 'Pointers' = NO, `mx` contains the vector Bx when `irevcm`
        returns the value +1.

        `On final exit`: does not contain any useful data.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    irevcm : int
        `On intermediate exit`: has the following meanings.

        `irevcm` = -1

            The calling program must compute the matrix-vector product y
            = op(x), where x is stored in `x` (by default) or in the
            array `comm`['comm'] (starting from the location given by
            the first element of `comm`['icomm']) when the option
            'Pointers' = 'YES' is set in a prior call to
            ``real_option``. The result y is returned in `x` (by
            default) or in the array `comm`['comm'] (starting from the
            location given by the second element of `comm`['icomm'])
            when the option 'Pointers' = 'YES' is set. If B is not
            symmetric semidefinite then the precomputed values in `mx`
            should not be used (see the explanation under `irevcm` = 2).

        `irevcm` = 1

            The calling program must compute the matrix-vector product y
            = op(x). This is similar to the case `irevcm` = -1 except
            that the result of the matrix-vector product Bx (as required
            in some computational modes) has already been computed and
            is available in `mx` (by default) or in the array
            `comm`['comm'] (starting from the location given by the
            third element of `comm`['icomm']) when the option 'Pointers'
            = 'YES' is set.

        `irevcm` = 2

            The calling program must compute the matrix-vector product y
            = Bx, where x is stored as described in the case `irevcm` =
            -1 and y is returned in the location described by the case
            `irevcm` = 1. This computation is requested when solving the
            'Generalized' problem using either 'Shifted Inverse
            Imaginary', 'Shifted Inverse Real'; in these cases B is used
            as an inner-product space and requires that B be symmetric
            semidefinite. If neither A nor B is symmetric semidefinite
            then the problem should be reformulated in a 'Standard'
            form.

        `irevcm` = 3

            Compute the `nshift` real and imaginary parts of the shifts
            where the real parts are to be returned in the first
            `nshift` locations of the array `x` and the imaginary parts
            are to be returned in the first `nshift` locations of the
            array `mx`. Only complex conjugate pairs of shifts may be
            applied and the pairs must be placed in consecutive
            locations. This value of `irevcm` will only arise if the
            option 'Supplied Shifts' is set in a prior call to
            ``real_option`` which is intended for experienced users
            only; the default and recommended option is to use exact
            shifts (see Lehoucq `et al.` (1998) for details).

        `irevcm` = 4

            Monitoring step: a call to ``real_monit`` can now be made to
            return the number of Arnoldi iterations, the number of
            converged Ritz values, their real and imaginary parts, and
            the corresponding Ritz estimates.

        `On final exit`: `irevcm` = 5: ``real_iter`` has completed its
        tasks. The value of errno determines whether the iteration has
        been successfully completed, or whether errors have been
        detected. On successful completion ``real_proc`` must be called
        to return the requested eigenvalues and eigenvectors (and/or
        Schur vectors).

    nshift : int
        `On intermediate exit`: if the option 'Supplied Shifts' is set
        and `irevcm` returns a value of 3, `nshift` returns the number
        of complex shifts required.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The maximum number of iterations <= 0, the option 'Iteration
            Limit' has been set to *<value>*.

        (`errno` 2)
            The options 'Generalized' and 'Regular' are incompatible.

        (`errno` 5)
            No shifts could be applied during a cycle of the implicitly
            restarted Arnoldi iteration.

        (`errno` 6)
            Could not build an Arnoldi factorization. The size of the
            current Arnoldi factorization = *<value>*.

        (`errno` 7)
            Error in internal call to compute eigenvalues and
            corresponding error bounds of the current upper Hessenberg
            matrix. Please contact NAG.

        (`errno` 8)
            Either the initialization function has not been called prior
            to the first call of this function or a communication array
            has become corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            The option 'Initial Residual' was selected but the starting
            vector held in `resid` is zero.

        (`errno` 4)
            The maximum number of iterations has been reached. The
            maximum number of iterations = *<value>*. The number of
            converged eigenvalues = *<value>*. See the function document
            for further details.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, (and optionally the corresponding eigenvectors,
    x) of a standard eigenvalue problem Ax = lambda x, or of a
    generalized eigenvalue problem Ax = lambda Bx of order n, where n is
    large and the coefficient matrices A and B are sparse, real and
    nonsymmetric.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale dense, real and nonsymmetric problems.

    ``real_iter`` is a **reverse communication** function, based on the
    ARPACK routine **dnaupd**, using the Implicitly Restarted Arnoldi
    iteration method.
    The method is described in Lehoucq and Sorensen (1996) and Lehoucq
    (2001) while its use within the ARPACK software is described in
    great detail in Lehoucq `et al.` (1998).
    An evaluation of software for computing eigenvalues of sparse
    nonsymmetric matrices is provided in Lehoucq and Scott (1996).
    This suite of functions offers the same functionality as the ARPACK
    software for real nonsymmetric problems, but the interface design is
    quite different in order to make the option setting clearer and to
    simplify the interface of ``real_iter``.

    The setup function ``real_init`` must be called before
    ``real_iter``, the reverse communication iterative solver.
    Options may be set for ``real_iter`` by prior calls to the option
    setting function ``real_option`` and a post-processing function
    ``real_proc`` must be called following a successful final exit from
    ``real_iter``. ``real_monit``, may be called following certain
    flagged, intermediate exits from ``real_iter`` to provide additional
    monitoring information about the computation.

    ``real_iter`` uses **reverse communication**, i.e., it returns
    repeatedly to the calling program with the argument `irevcm` (see
    [Parameters]) set to specified values which require the calling
    program to carry out one of the following tasks:

    -   compute the matrix-vector product y = op(x), where op is defined
        by the computational mode;

    -   compute the matrix-vector product y = Bx;

    -   notify the completion of the computation;

    -   allow the calling program to monitor the solution.

    The problem type to be solved (standard or generalized), the
    spectrum of eigenvalues of interest, the mode used (regular, regular
    inverse, shifted inverse, shifted real or shifted imaginary) and
    other options can all be set using the option setting function
    ``real_option`` (see [Other Parameters for real_option] for details
    on setting options and of the default settings).

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_iter'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12abft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, resid[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, mx[*].
        utils._BYREF_INT, # Output, nshift.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[*].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((v, 'v'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_n_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)),), validate=False)
    else:
        _vl_f12aaf_n_py = 0
    _resid_cls = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vl_f12aaf_n_py,),
    )
    _resid = _resid_cls.to_ctypes()
    _ldv_py = utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+2,), validate=False)
    else:
        _vl_f12aaf_ncv_py = 0
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_vl_f12aaf_n_py, _vl_f12aaf_ncv_py),
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12aaf_lcomm_py = 0
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12aaf_licomm_py = 0
    _comm = comm['comm']
    _icomm = comm['icomm']
    if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+24)-1,), validate=False) == 0:
        _x_dim1 = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+1)-1,), validate=False)
    else:
        _x_dim1 = 1
    exp_shape_x = (_x_dim1,)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=exp_shape_x,
    )
    _x = _x_cls.to_ctypes()
    if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+24)-1,), validate=False) == 0:
        _mx_dim1 = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+1)-1,), validate=False)
    else:
        _mx_dim1 = 1
    exp_shape_mx = (_mx_dim1,)
    _mx_cls = utils._EngineFloat64ArrayType(
        mx, {'fun_name': fun_name, 'entity': 'mx'},
        exp_shape=exp_shape_mx,
    )
    _mx = _mx_cls.to_ctypes()
    _nshift = utils._EngineIntScalarType.empty_ctypes()
    _routine_name = utils._EngineRoutineName(*b'f12ab ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _irevcm,
        _resid,
        _v,
        _ldv,
        _x,
        _mx,
        _nshift,
        _comm,
        _icomm,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 5 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations <= 0, the option',
                    '\'Iteration Limit\' has been set to {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The options \'Generalized\' and \'Regular\' are incompatible.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Initial Residual\' was selected but the',
                    'starting vector held in resid is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations has been reached.',
                    'The maximum number of iterations = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The number of converged eigenvalues = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'See the function document for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No shifts could be applied during a cycle of the',
                    'implicitly restarted Arnoldi iteration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Could not build an Arnoldi factorization. The size',
                    'of the current Arnoldi factorization = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error in internal call to compute eigenvalues and corresponding',
                    'error bounds of the current upper Hessenberg matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called prior',
                    'to the first call of this function or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _resid_cls.refresh_py(_resid)
        _v_cls.refresh_py(_v)
        _x_cls.refresh_py(_x)
        _mx_cls.refresh_py(_mx)
        _nshift_py = utils._EngineIntScalarType.to_py(_nshift)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['irevcm', 'nshift'],
        )(
            _irevcm_py,
            _nshift_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_proc(dr, di, z, sigmar, sigmai, resid, v, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a real
    nonsymmetric sparse eigenproblem, postprocessing for ``real_iter``.

    ``real_proc`` is a post-processing function in a suite of functions
    consisting of ``real_init``, ``real_option`` and ``real_monit``.
    It must be called following a final exit from ``real_iter``.

    For full information please refer to the NAG Library document for
    f12ac

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12acf.html

    Parameters
    ----------
    dr : float, shape (nev+1), modified in place
        `On exit`: the first `nconv` locations of the array `dr` contain
        the real parts of the converged approximate eigenvalues.

    di : float, shape (nev+1), modified in place
        `On exit`: the first `nconv` locations of the array `di` contain
        the imaginary parts of the converged approximate eigenvalues.

    z : float, shape (n, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if int(Re(`comm`['comm'][0]))>=1 and
        int(Re(`comm`['comm'][0]))<=1000: if
        `comm`['icomm'][(int(`comm`['comm'][0])+7)-1]=1: nev+1;
        otherwise: 1; otherwise: 0.

        `On exit`: if the default option 'Vectors' = 'RITZ' (see
        ``real_option``) has been selected then `z` contains the final
        set of eigenvectors corresponding to the eigenvalues held in
        `dr` and `di`. The complex eigenvector associated with the
        eigenvalue with positive imaginary part is stored in two
        consecutive columns. The first column holds the real part of the
        eigenvector and the second column holds the imaginary part. The
        eigenvector associated with the eigenvalue with negative
        imaginary part is simply the complex conjugate of the
        eigenvector associated with the positive imaginary part.

    sigmar : float
        If one of the 'Shifted Inverse Real' modes have been selected
        then `sigmar` contains the real part of the shift used;
        otherwise `sigmar` is not referenced.

    sigmai : float
        If one of the 'Shifted Inverse Real' modes have been selected
        then `sigmai` contains the imaginary part of the shift used;
        otherwise `sigmai` is not referenced.

    resid : float, shape (n)
        Must not be modified following a call to ``real_iter`` since it
        contains data required by ``real_proc``.

    v : float, shape (n, ncv), modified in place
        `On entry`: the ncv columns of `v` contain the Arnoldi basis
        vectors for op as constructed by ``real_iter``.

        `On exit`: if the option 'Vectors' = 'SCHUR' has been set, or
        the option 'Vectors' = 'RITZ' has been set and a separate array
        `z` has been passed (i.e., `z` does not equal `v`), then the
        first `nconv` columns of `v` will contain approximate Schur
        vectors that span the desired invariant subspace.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged eigenvalues as found by ``real_iter``.

    Raises
    ------
    NagValueError
        (`errno` 3)
            The number of eigenvalues found to sufficient accuracy, as
            communicated through the argument `comm`['icomm'], is zero.
            See the function document for further details.

        (`errno` 4)
            Got a different count of the number of converged Ritz values
            than the value passed to it through the argument
            `comm`['icomm']: number counted = *<value>*, number expected
            = *<value>*.

        (`errno` 5)
            During calculation of a real Schur form, there was a failure
            to compute *<value>* eigenvalues in a total of *<value>*
            iterations.

        (`errno` 6)
            The computed Schur form could not be reordered by an
            internal call. This function returned with errno =
            *<value>*. Please contact NAG.

        (`errno` 7)
            In calculating eigenvectors, an internal call returned with
            an error. Please contact NAG.

        (`errno` 8)
            Either the solver function has not been called prior to the
            call of this function or a communication array has become
            corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            On entry, 'Vectors' = 'SELECT', but this is not yet
            implemented.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, (and optionally the corresponding eigenvectors,
    x) of a standard eigenvalue problem Ax = lambda x, or of a
    generalized eigenvalue problem Ax = lambda Bx of order n, where n is
    large and the coefficient matrices A and B are sparse, real and
    nonsymmetric.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale dense, real and nonsymmetric problems.

    Following a call to ``real_iter``, ``real_proc`` returns the
    converged approximations to eigenvalues and (optionally) the
    corresponding approximate eigenvectors and/or an orthonormal basis
    for the associated approximate invariant subspace.
    The eigenvalues (and eigenvectors) are selected from those of a
    standard or generalized eigenvalue problem defined by real
    nonsymmetric matrices.
    There is negligible additional cost to obtain eigenvectors; an
    orthonormal basis is always computed, but there is an additional
    storage cost if both are requested.

    ``real_proc`` is based on the function **dneupd** from the ARPACK
    package, which uses the Implicitly Restarted Arnoldi iteration
    method.
    The method is described in Lehoucq and Sorensen (1996) and Lehoucq
    (2001) while its use within the ARPACK software is described in
    great detail in Lehoucq `et al.` (1998).
    An evaluation of software for computing eigenvalues of sparse
    nonsymmetric matrices is provided in Lehoucq and Scott (1996).
    This suite of functions offers the same functionality as the ARPACK
    software for real nonsymmetric problems, but the interface design is
    quite different in order to make the option setting clearer and to
    simplify some of the interfaces.

    ``real_proc`` is a post-processing function that must be called
    following a successful final exit from ``real_iter``. ``real_proc``
    uses data returned from ``real_iter`` and options, set either by
    default or explicitly by calling ``real_option``, to return the
    converged approximations to selected eigenvalues and (optionally):

    -   the corresponding approximate eigenvectors;

    -   an orthonormal basis for the associated approximate invariant
        subspace;

    -   both.

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_proc'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12acft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Output, nconv.
        utils._EngineFloat64ArrayType.c_type, # Output, dr[*].
        utils._EngineFloat64ArrayType.c_type, # Output, di[*].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._BYREF_FLOAT64, # Input, sigmar.
        utils._BYREF_FLOAT64, # Input, sigmai.
        utils._EngineFloat64ArrayType.c_type, # Input, resid[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[*].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'), (v, 'v')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_nev_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+1,), validate=False)
    else:
        _vl_f12aaf_nev_py = 0
    _dr_cls = utils._EngineFloat64ArrayType(
        dr, {'fun_name': fun_name, 'entity': 'dr'},
        exp_shape=(_vl_f12aaf_nev_py+1,),
        check_contents=False,
    )
    _dr = _dr_cls.to_ctypes(can_reference_contents=False)
    _di_cls = utils._EngineFloat64ArrayType(
        di, {'fun_name': fun_name, 'entity': 'di'},
        exp_shape=(_vl_f12aaf_nev_py+1,),
        check_contents=False,
    )
    _di = _di_cls.to_ctypes(can_reference_contents=False)
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12aaf_lcomm_py = 0
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12aaf_licomm_py = 0
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _comm = comm['comm']
    _icomm = comm['icomm']
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_n_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)),), validate=False)
    else:
        _vl_f12aaf_n_py = 0
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+7)-1,), validate=False) == 1:
            _z_dim2 = _vl_f12aaf_nev_py+1
        else:
            _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_vl_f12aaf_n_py, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _sigmar = utils._EngineFloat64ScalarType(
        sigmar, {'fun_name': fun_name, 'entity': 'sigmar'},
    ).to_ctypes()
    _sigmai = utils._EngineFloat64ScalarType(
        sigmai, {'fun_name': fun_name, 'entity': 'sigmai'},
    ).to_ctypes()
    _resid = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vl_f12aaf_n_py,),
    ).to_ctypes()
    _ldv_py = utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+2,), validate=False)
    else:
        _vl_f12aaf_ncv_py = 0
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_vl_f12aaf_n_py, _vl_f12aaf_ncv_py),
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    _routine_name = utils._EngineRoutineName(*b'f12ac ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _nconv,
        _dr,
        _di,
        _z,
        _ldz,
        _sigmar,
        _sigmai,
        _resid,
        _v,
        _ldv,
        _comm,
        _icomm,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, \'Vectors\' = \'SELECT\', but this is not yet implemented.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of eigenvalues found to sufficient accuracy,',
                    'as communicated through the argument comm[\'icomm\'], is zero.',
                    'See the function document for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Got a different count of the number of converged Ritz values',
                    'than the value passed to it through the argument comm[\'icomm\']:',
                    'number counted = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'number expected = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'During calculation of a real Schur form, there was a failure',
                    'to compute {:s} eigenvalues in a total of'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The computed Schur form could not be reordered by an internal call.',
                    'This function returned with errno = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In calculating eigenvectors, an internal call returned with an error.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the solver function has not been called prior',
                    'to the call of this function or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _dr_cls.refresh_py(_dr)
        _di_cls.refresh_py(_di)
        _z_cls.refresh_py(_z)
        _v_cls.refresh_py(_v)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _nconv_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['nconv'],
            )(
                _nconv_py,
            )
    _err_class.manage()
    return return_data

def real_option(optstr, comm, io_manager=None):
    u"""
    Set a single option from a string
    (``real_iter``/``real_proc``/``real_band_solve``).

    Note: this function uses optional algorithmic parameters, see also:
    ``real_iter``, ``real_proc``, ``real_band_solve``, ``real_init``,
    ``real_band_init``.

    ``real_option`` is an option setting function in a suite of
    functions consisting of ``real_init``, ``real_iter``, ``real_proc``,
    ``real_option`` and ``real_monit``; it supplies individual options
    to ``real_iter`` and ``real_proc``. ``real_option`` is also an
    option setting function in a suite of functions consisting of
    ``real_option``, ``real_band_init`` and ``real_band_solve``; it
    supplies individual options to ``real_band_solve``.

    The initialization function for the appropriate suite, ``real_init``
    or ``real_band_init``, **must** have been called prior to calling
    ``real_option``.

    For full information please refer to the NAG Library document for
    f12ad

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12adf.html

    Parameters
    ----------
    optstr : str
        A single valid option string (as described in [Notes] and [Other
        Parameters]).

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_init`` or ``real_band_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Other Parameters
    ----------------
    'Advisory' : int
        Default = advisory message unit number

        The destination for advisory messages.

    'Defaults' : valueless
        This special keyword may be used to reset all options to their
        default values.

    'Exact Shifts' : valueless
        Default

        During the Arnoldi iterative process, shifts are applied
        internally as part of the implicit restarting scheme.
        The shift strategy used by default and selected by the 'Exact
        Shifts' is strongly recommended over the alternative 'Supplied
        Shifts' (see Lehoucq `et al.` (1998) for details of shift
        strategies).

        If 'Exact Shifts' are used then these are computed internally by
        the algorithm in the implicit restarting scheme.

        If 'Supplied Shifts' are used then, during the Arnoldi iterative
        process, you must supply shifts through array arguments of
        ``real_iter`` when ``real_iter`` returns with irevcm = 3; the
        real and imaginary parts of the shifts are supplied in x and mx
        respectively (or in comm when the option 'Pointers' = 'YES' is
        set). This option should only be used if you are an experienced
        user since this requires some algorithmic knowledge and because
        more operations are usually required than for the implicit shift
        scheme.
        Details on the use of explicit shifts and further references on
        shift strategies are available in Lehoucq `et al.` (1998).

    'Supplied Shifts' : valueless
        During the Arnoldi iterative process, shifts are applied
        internally as part of the implicit restarting scheme.
        The shift strategy used by default and selected by the 'Exact
        Shifts' is strongly recommended over the alternative 'Supplied
        Shifts' (see Lehoucq `et al.` (1998) for details of shift
        strategies).

        If 'Exact Shifts' are used then these are computed internally by
        the algorithm in the implicit restarting scheme.

        If 'Supplied Shifts' are used then, during the Arnoldi iterative
        process, you must supply shifts through array arguments of
        ``real_iter`` when ``real_iter`` returns with irevcm = 3; the
        real and imaginary parts of the shifts are supplied in x and mx
        respectively (or in comm when the option 'Pointers' = 'YES' is
        set). This option should only be used if you are an experienced
        user since this requires some algorithmic knowledge and because
        more operations are usually required than for the implicit shift
        scheme.
        Details on the use of explicit shifts and further references on
        shift strategies are available in Lehoucq `et al.` (1998).

    'Iteration Limit' : int
        Default = 300

        The limit on the number of Arnoldi iterations that can be
        performed before ``real_iter`` or ``real_band_solve`` exits.
        If not all requested eigenvalues have converged to within
        'Tolerance' and the number of Arnoldi iterations has reached
        this limit then ``real_iter`` or ``real_band_solve`` exits with
        an error; ``real_band_solve`` returns the number of converged
        eigenvalues, the converged eigenvalues and, if requested, the
        corresponding eigenvectors, while ``real_proc`` can be called
        subsequent to ``real_iter`` to do the same.

    'Largest Magnitude' : valueless
        Default

        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_iter`` or ``real_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of op (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Largest Imaginary' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_iter`` or ``real_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of op (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Largest Real' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_iter`` or ``real_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of op (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Imaginary' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_iter`` or ``real_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of op (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Magnitude' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_iter`` or ``real_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of op (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Real' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_iter`` or ``real_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of op (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Nolist' : valueless
        Default

        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'List' : valueless
        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'Monitoring' : int
        Default = -1

        If i > 0, monitoring information is output to unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`)
        i during the solution of each problem; this may be the same as
        the 'Advisory' unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).
        The type of information produced is dependent on the value of
        'Print Level', see the description of the option 'Print Level'
        for details of the information produced.
        Please see the ``FileObjManager`` method
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`
        to associate a file with a given unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).

    'Pointers' : valueless
        Default = 'NO'

        During the iterative process and reverse communication calls to
        ``real_iter``, required data can be communicated to and from
        ``real_iter`` in one of two ways.
        When 'Pointers' = 'NO' is selected (the default) then the array
        arguments x and mx are used to supply you with required data and
        used to return computed values back to ``real_iter``.
        For example, when irevcm = 1, ``real_iter`` returns the vector x
        in x and the matrix-vector product Bx in mx and expects the
        result or the linear operation op(x) to be returned in x.

        If 'Pointers' = 'YES' is selected then the data is passed
        through sections of the array argument comm.
        The section corresponding to x when 'Pointers' = 'NO' begins at
        a location given by the first element of icomm; similarly the
        section corresponding to mx begins at a location given by the
        second element of icomm.
        This option allows ``real_iter`` to perform fewer copy
        operations on each intermediate exit and entry, but can also
        lead to less elegant code in the calling program.

        This option has no affect on ``real_band_solve`` which sets
        'Pointers' = 'YES' internally.

    'Print Level' : int
        Default = 0

        This controls the amount of printing produced by ``real_option``
        as follows.

        [table omitted]

    'Random Residual' : valueless
        Default

        To begin the Arnoldi iterative process, ``real_iter`` or
        ``real_band_solve`` requires an initial residual vector.
        By default ``real_iter`` or ``real_band_solve`` provides its own
        random initial residual vector; this option can also be set
        using option 'Random Residual'.
        Alternatively, you can supply an initial residual vector
        (perhaps from a previous computation) to ``real_iter`` or
        ``real_band_solve`` through the array argument resid; this
        option can be set using option 'Initial Residual'.

    'Initial Residual' : valueless
        To begin the Arnoldi iterative process, ``real_iter`` or
        ``real_band_solve`` requires an initial residual vector.
        By default ``real_iter`` or ``real_band_solve`` provides its own
        random initial residual vector; this option can also be set
        using option 'Random Residual'.
        Alternatively, you can supply an initial residual vector
        (perhaps from a previous computation) to ``real_iter`` or
        ``real_band_solve`` through the array argument resid; this
        option can be set using option 'Initial Residual'.

    'Regular' : valueless
        Default

        These options define the computational mode which in turn
        defines the form of operation op(x) to be performed by
        ``real_band_solve`` or when ``real_iter`` returns with irevcm =
        -1 or 1 and the matrix-vector product Bx when ``real_iter``
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

    'Regular Inverse' : valueless
        These options define the computational mode which in turn
        defines the form of operation op(x) to be performed by
        ``real_band_solve`` or when ``real_iter`` returns with irevcm =
        -1 or 1 and the matrix-vector product Bx when ``real_iter``
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

    'Shifted Inverse Imaginary' : valueless
        These options define the computational mode which in turn
        defines the form of operation op(x) to be performed by
        ``real_band_solve`` or when ``real_iter`` returns with irevcm =
        -1 or 1 and the matrix-vector product Bx when ``real_iter``
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

    'Shifted Inverse Real' : valueless
        These options define the computational mode which in turn
        defines the form of operation op(x) to be performed by
        ``real_band_solve`` or when ``real_iter`` returns with irevcm =
        -1 or 1 and the matrix-vector product Bx when ``real_iter``
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

    'Standard' : valueless
        Default

        The problem to be solved is either a standard eigenvalue
        problem, Ax = lambda x, or a generalized eigenvalue problem, Ax
        = lambda Bx.
        The option 'Standard' should be used when a standard eigenvalue
        problem is being solved and the option 'Generalized' should be
        used when a generalized eigenvalue problem is being solved.

    'Generalized' : valueless
        The problem to be solved is either a standard eigenvalue
        problem, Ax = lambda x, or a generalized eigenvalue problem, Ax
        = lambda Bx.
        The option 'Standard' should be used when a standard eigenvalue
        problem is being solved and the option 'Generalized' should be
        used when a generalized eigenvalue problem is being solved.

    'Tolerance' : float
        Default = epsilon

        An approximate eigenvalue has deemed to have converged when the
        corresponding Ritz estimate is within 'Tolerance' relative to
        the magnitude of the eigenvalue.

    'Vectors' : valueless
        Default = 'RITZ'

        The function ``real_proc`` or ``real_band_solve`` can optionally
        compute the Schur vectors and/or the eigenvectors corresponding
        to the converged eigenvalues.
        To turn off computation of any vectors the option 'Vectors' =
        'NONE' should be set.
        To compute only the Schur vectors (at very little extra cost),
        the option 'Vectors' = 'SCHUR' should be set and these will be
        returned in the array argument v of ``real_proc`` or
        ``real_band_solve``.
        To compute the eigenvectors (Ritz vectors) ­corresponding to the
        eigenvalue estimates, the option 'Vectors' = 'RITZ' should be
        set and these will be returned in the array argument z of
        ``real_proc`` or ``real_band_solve``, if z is set equal to v
        then the Schur vectors in v are overwritten by the eigenvectors
        computed by ``real_proc`` or ``real_band_solve``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Ambiguous keyword: *<value>*

        (`errno` 2)
            Keyword not recognized: *<value>*

        (`errno` 3)
            Second keyword not recognized: *<value>*

        (`errno` 4)
            Either the initialization function has not been called prior
            to the call of this function or a communication array has
            become corrupted.

    Notes
    -----
    ``real_option`` may be used to supply values for options to
    ``real_iter`` and ``real_proc``, or to ``real_band_solve``.
    It is only necessary to call ``real_option`` for those arguments
    whose values are to be different from their default values.
    One call to ``real_option`` sets one argument value.

    Each option is defined by a single character string consisting of
    one or more items.
    The items associated with a given option must be separated by
    spaces, or equals signs [ = ].
    Alphabetic characters may be upper or lower case.
    The string

    ::

      Iteration Limit = 500

    is an example of a string used to set an option.
    For each option the string contains one or more of the following
    items:

    -   a mandatory keyword;

    -   a phrase that qualifies the keyword;

    -   a number that specifies an `int` or `float` value. Such numbers
        may be up to 16 contiguous characters in Fortran's I, F, E or D
        format.

    ``real_option`` does not have an equivalent function from the ARPACK
    package which passes options by directly setting values to scalar
    arguments or to specific elements of array arguments.
    ``real_option`` is intended to make the passing of options more
    transparent and follows the same principle as the single option
    setting functions in submodule ``opt``.

    The setup function ``real_init`` or ``real_band_init`` must be
    called prior to the first call to ``real_option``, and all calls to
    ``real_option`` must precede the first call to ``real_iter`` or
    ``real_band_solve``.

    A complete list of options, their abbreviations, synonyms and
    default values is given in [Other Parameters].

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_option'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12adft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_STRING, # Input, optstr(len=*).
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[60].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of optstr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _optstr = utils._EngineCharScalarType(
        optstr, {'fun_name': fun_name, 'entity': 'optstr'}, exp_chrlen=None,
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm', 'comm'),
    )
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12aaf_licomm_py = 0
    _icomm = comm['icomm']
    _comm = comm['comm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _optstr,
        _icomm,
        _comm,
        _errbuf,
        _ierr,
        _optstr._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Ambiguous keyword: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Keyword not recognized: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Second keyword not recognized: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 4:
            _ierr_fl = 4
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called prior',
                    'to the call of this function or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = None
    _err_class.manage()
    return return_data

def real_monit(ritzr, ritzi, rzest, comm):
    u"""
    Provides monitoring information for ``real_iter``.

    ``real_monit`` can be used to return additional monitoring
    information during computation.
    It is in a suite of functions consisting of ``real_init``,
    ``real_iter``, ``real_proc``, ``real_option`` and ``real_monit``.

    For full information please refer to the NAG Library document for
    f12ae

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12aef.html

    Parameters
    ----------
    ritzr : float, shape (ncv), modified in place
        `On exit`: the first `nconv` locations of the array `ritzr`
        contain the real parts of the converged approximate eigenvalues.

    ritzi : float, shape (ncv), modified in place
        `On exit`: the first `nconv` locations of the array `ritzi`
        contain the imaginary parts of the converged approximate
        eigenvalues.

    rzest : float, shape (ncv), modified in place
        `On exit`: the first `nconv` locations of the array `rzest`
        contain the Ritz estimates (error bounds) on the converged
        approximate eigenvalues.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_init``.

    Returns
    -------
    niter : int
        The number of the current Arnoldi iteration.

    nconv : int
        The number of converged eigenvalues so far.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, (and optionally the corresponding eigenvectors,
    x) of a standard eigenvalue problem Ax = lambda x, or of a
    generalized eigenvalue problem Ax = lambda Bx of order n, where n is
    large and the coefficient matrices A and B are sparse, real and
    nonsymmetric.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale dense, real and nonsymmetric problems.

    On an intermediate exit from ``real_iter`` with irevcm = 4,
    ``real_monit`` may be called to return monitoring information on the
    progress of the Arnoldi iterative process.
    The information returned by ``real_monit`` is:

    -   the number of the current Arnoldi iteration;

    -   the number of converged eigenvalues at this point;

    -   the real and imaginary parts of the converged eigenvalues;

    -   the error bounds on the converged eigenvalues.

    ``real_monit`` does not have an equivalent function from the ARPACK
    package which prints various levels of detail of monitoring
    information through an output channel controlled via an argument
    value (see Lehoucq `et al.` (1998) for details of ARPACK routines).
    ``real_monit`` should not be called at any time other than
    immediately following an irevcm = 4 return from ``real_iter``.

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_monit'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12aeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Output, niter.
        utils._BYREF_INT, # Output, nconv.
        utils._EngineFloat64ArrayType.c_type, # Output, ritzr[*].
        utils._EngineFloat64ArrayType.c_type, # Output, ritzi[*].
        utils._EngineFloat64ArrayType.c_type, # Output, rzest[*].
        utils._EngineIntArrayType.c_type, # Input, icomm[*].
        utils._EngineFloat64ArrayType.c_type, # Input, comm[*].
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _niter = utils._EngineIntScalarType.empty_ctypes()
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm', 'comm'),
    )
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+2,), validate=False)
    else:
        _vl_f12aaf_ncv_py = 0
    _ritzr_cls = utils._EngineFloat64ArrayType(
        ritzr, {'fun_name': fun_name, 'entity': 'ritzr'},
        exp_shape=(_vl_f12aaf_ncv_py,),
        check_contents=False,
    )
    _ritzr = _ritzr_cls.to_ctypes(can_reference_contents=False)
    _ritzi_cls = utils._EngineFloat64ArrayType(
        ritzi, {'fun_name': fun_name, 'entity': 'ritzi'},
        exp_shape=(_vl_f12aaf_ncv_py,),
        check_contents=False,
    )
    _ritzi = _ritzi_cls.to_ctypes(can_reference_contents=False)
    _rzest_cls = utils._EngineFloat64ArrayType(
        rzest, {'fun_name': fun_name, 'entity': 'rzest'},
        exp_shape=(_vl_f12aaf_ncv_py,),
        check_contents=False,
    )
    _rzest = _rzest_cls.to_ctypes(can_reference_contents=False)
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12aaf_licomm_py = 0
    _icomm = comm['icomm']
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12aaf_lcomm_py = 0
    _comm = comm['comm']
    engine_ctypes(
        _en_data,
        _niter,
        _nconv,
        _ritzr,
        _ritzi,
        _rzest,
        _icomm,
        _comm,
    )
    utils._handle_cb_exception(excs)
    _ritzr_cls.refresh_py(_ritzr)
    _ritzi_cls.refresh_py(_ritzi)
    _rzest_cls.refresh_py(_rzest)
    _niter_py = utils._EngineIntScalarType.to_py(_niter)
    _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
    return_data = _collections.namedtuple(
        'nag_return_data',
        ['niter', 'nconv'],
    )(
        _niter_py,
        _nconv_py,
    )
    return return_data

def real_band_init(n, nev, ncv):
    u"""
    Initialization routine for (``real_band_solve``) computing selected
    eigenvalues and, optionally, eigenvectors of a real nonsymmetric
    banded (standard or generalized) eigenproblem.

    ``real_band_init`` is a setup function for ``real_band_solve`` which
    may be used for finding some eigenvalues (and optionally the
    corresponding eigenvectors) of a standard or generalized eigenvalue
    problem defined by real, banded, nonsymmetric matrices.
    The banded matrix must be stored using the LAPACK column ordered
    storage format for real banded nonsymmetric matrices (see [the F07
    Introduction]).

    For full information please refer to the NAG Library document for
    f12af

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12aff.html

    Parameters
    ----------
    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    nev : int
        The number of eigenvalues to be computed.

    ncv : int
        The number of Lanczos basis vectors to use during the
        computation.

        At present there is no `a priori` analysis to guide the
        selection of `ncv` relative to `nev`.

        However, it is recommended that `ncv` >= 2*`nev`+1.

        If many problems of the same type are to be solved, you should
        experiment with increasing `ncv` while keeping `nev` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the work and storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to CPU time is problem
        dependent and must be determined empirically.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nev` = *<value>*.

            Constraint: `nev` > 0.

        (`errno` 3)
            On entry, `ncv` = *<value>*, `nev` = *<value>* and `n` =
            *<value>*.

            Constraint: `ncv` > `nev`+1 and `ncv` <= `n`.

    Notes
    -----
    The pair of functions ``real_band_init`` and ``real_band_solve``
    together with the option setting function ``real_option`` are
    designed to calculate some of the eigenvalues, lambda, (and
    optionally the corresponding eigenvectors, x) of a standard
    eigenvalue problem Ax = lambda x, or of a generalized eigenvalue
    problem Ax = lambda Bx of order n, where n is large and the
    coefficient matrices A and B are banded real and nonsymmetric.

    ``real_band_init`` is a setup function which must be called before
    the option setting function ``real_option`` and the solver function
    ``real_band_solve``.
    Internally, ``real_band_solve`` makes calls to ``real_iter`` and
    ``real_proc``; the function documents for ``real_iter`` and
    ``real_proc`` should be consulted for details of the algorithm used.

    This setup function initializes the communication arrays, sets (to
    their default values) all options that can be set by you via the
    option setting function ``real_option``, and checks that the lengths
    of the communication arrays as passed by you are of sufficient
    length.
    For details of the options available and how to set them, see [Other
    Parameters for real_option].

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_band_init'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12afft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nev.
        utils._BYREF_INT, # Input, ncv.
        utils._EngineIntArrayType.c_type, # Output, icomm[max(1, licomm)].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineFloat64ArrayType.c_type, # Output, comm[max(1, lcomm)].
        utils._BYREF_INT, # Input, lcomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nev = utils._EngineIntScalarType(
        nev, {'fun_name': fun_name, 'entity': 'nev'},
    ).to_ctypes()
    _ncv = utils._EngineIntScalarType(
        ncv, {'fun_name': fun_name, 'entity': 'ncv'},
    ).to_ctypes()
    _licomm = utils._EngineIntScalarType.c_type(140)
    _licomm_py = utils._EngineIntScalarType.to_py(_licomm)
    _icomm = utils._EngineIntArrayType.empty_ctypes(max(1, _licomm_py))
    _comm_py = {}
    _comm_py['icomm'] = _icomm
    _lcomm = utils._EngineIntScalarType.c_type(60)
    _lcomm_py = utils._EngineIntScalarType.to_py(_lcomm)
    _comm = utils._EngineFloat64ArrayType.empty_ctypes(max(1, _lcomm_py))
    _comm_py['comm'] = _comm
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _nev,
        _ncv,
        _icomm,
        _licomm,
        _comm,
        _lcomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nev > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncv = {:s}, nev = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: ncv > nev+1 and ncv <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = _comm_py
    _err_class.manage()
    return return_data

def real_band_solve(kl, ku, ab, mb, sigmar, sigmai, dr, di, z, resid, v, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a real
    nonsymmetric banded eigenproblem, driver.

    ``real_band_solve`` is the main solver function in a suite of
    functions consisting of ``real_option``, ``real_band_init`` and
    ``real_band_solve``.
    It must be called following an initial call to ``real_band_init``
    and following any calls to ``real_option``.

    ``real_band_solve`` returns approximations to selected eigenvalues,
    and (optionally) the corresponding eigenvectors, of a standard or
    generalized eigenvalue problem defined by real banded nonsymmetric
    matrices.
    The banded matrix must be stored using the LAPACK storage format for
    real banded nonsymmetric matrices.

    For full information please refer to the NAG Library document for
    f12ag

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12agf.html

    Parameters
    ----------
    kl : int
        The number of subdiagonals of the matrices A and B.

    ku : int
        The number of superdiagonals of the matrices A and B.

    ab : float, shape (2*`kl`+`ku`+1, n)
        Must contain the matrix A in LAPACK banded storage format for
        nonsymmetric matrices (see [the F07 Introduction]).

    mb : float, shape (2*`kl`+`ku`+1, n)
        Must contain the matrix B in LAPACK banded storage format for
        nonsymmetric matrices (see [the F07 Introduction]).

    sigmar : float
        If one of the 'Shifted Inverse Real' modes (see ``real_option``)
        have been selected then `sigmar` must contain the real part of
        the shift used; otherwise `sigmar` is not referenced. [the F12
        Introduction] describes the use of shift and inverse
        transformations.

    sigmai : float
        If one of the 'Shifted Inverse Real' modes (see ``real_option``)
        have been selected then `sigmai` must contain the imaginary part
        of the shift used; otherwise `sigmai` is not referenced. [the
        F12 Introduction] describes the use of shift and inverse
        transformations.

    dr : float, shape (nev+1), modified in place
        `On exit`: the first `nconv` locations of the array `dr` contain
        the real parts of the converged approximate eigenvalues. The
        number of eigenvalues returned may be one more than the number
        requested by nev since complex values occur as conjugate pairs
        and the second in the pair can be returned in position nev+1 of
        the array.

    di : float, shape (nev+1), modified in place
        `On exit`: the first `nconv` locations of the array `di` contain
        the imaginary parts of the converged approximate eigenvalues.
        The number of eigenvalues returned may be one more than the
        number requested by nev since complex values occur as conjugate
        pairs and the second in the pair can be returned in position
        nev+1 of the array.

    z : float, shape (n, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if
        `comm`['icomm'][(int(`comm`['comm'][0])+7)-1]=1: nev; otherwise:
        1.

        `On exit`: if the default option 'Vectors' = 'RITZ' has been
        selected then `z` contains the final set of eigenvectors
        corresponding to the eigenvalues held in `dr` and `di`. The
        complex eigenvector associated with the eigenvalue with positive
        imaginary part is stored in two consecutive columns. The first
        column holds the real part of the eigenvector and the second
        column holds the imaginary part. The eigenvector associated with
        the eigenvalue with negative imaginary part is simply the
        complex conjugate of the eigenvector associated with the
        positive imaginary part.

    resid : float, shape (n), modified in place
        `On entry`: need not be set unless the option 'Initial Residual'
        has been set in a prior call to ``real_option`` in which case
        `resid` must contain an initial residual vector.

        `On exit`: contains the final residual vector.

    v : float, shape (n, ncv), modified in place
        `On exit`: if the option 'Vectors' = 'SCHUR' or 'RITZ' (see
        ``real_option``) then the first `nconv`*n elements of `v` will
        contain approximate Schur vectors that span the desired
        invariant subspace.

        The ith Schur vector is stored in the ith column of `v`.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_band_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged eigenvalues.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `kl` = *<value>*.

            Constraint: `kl` >= 0.

        (`errno` 2)
            On entry, `ku` = *<value>*.

            Constraint: `ku` >= 0.

        (`errno` 3)
            On entry, ldab = *<value>*, 2*`kl`+`ku`+1 = *<value>*.

            Constraint: ldab >= 2*`kl`+`ku`+1.

        (`errno` 4)
            The option 'Shifted Inverse Imaginary' has been selected and
            `sigmai` = zero on entry; `sigmai` must be nonzero for this
            mode of operation.

        (`errno` 5)
            The maximum number of iterations <= 0, the option 'Iteration
            Limit' has been set to *<value>*.

        (`errno` 6)
            The options 'Generalized' and 'Regular' are incompatible.

        (`errno` 7)
            The option 'Initial Residual' was selected but the starting
            vector held in `resid` is zero.

        (`errno` 8)
            Either the initialization function has not been called prior
            to the first call of this function or a communication array
            has become corrupted.

        (`errno` 10)
            On entry, 'Vectors' = Select, but this is not yet
            implemented.

        (`errno` 11)
            The number of eigenvalues found to sufficient accuracy is
            zero.

        (`errno` 12)
            Could not build an Arnoldi factorization. The size of the
            current Arnoldi factorization = *<value>*.

        (`errno` 13)
            Error in internal call to compute eigenvalues and
            corresponding error bounds of the current upper Hessenberg
            matrix. Please contact NAG.

        (`errno` 14)
            During calculation of a real Schur form, there was a failure
            to compute a number of eigenvalues. Please contact NAG.

        (`errno` 15)
            The computed Schur form could not be reordered by an
            internal call. Please contact NAG.

        (`errno` 16)
            Error in internal call to compute eigenvectors. Please
            contact NAG.

        (`errno` 17)
            Failure during internal factorization of real banded matrix.
            Please contact NAG.

        (`errno` 18)
            Failure during internal solution of real banded matrix.
            Please contact NAG.

        (`errno` 19)
            Failure during internal factorization of complex banded
            matrix. Please contact NAG.

        (`errno` 20)
            Failure during internal solution of complex banded matrix.
            Please contact NAG.

        (`errno` 21)
            The maximum number of iterations has been reached. The
            maximum number of iterations = *<value>*. The number of
            converged eigenvalues = *<value>*.

        (`errno` 22)
            No shifts could be applied during a cycle of the implicitly
            restarted Lanczos iteration.

        (`errno` 23)
            Overflow occurred during transformation of Ritz values to
            those of the original problem.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, (and optionally the corresponding eigenvectors,
    x) of a standard eigenvalue problem Ax = lambda x, or of a
    generalized eigenvalue problem Ax = lambda Bx of order n, where n is
    large and the coefficient matrices A and B are banded, real and
    nonsymmetric.

    Following a call to the initialization function ``real_band_init``,
    ``real_band_solve`` returns the converged approximations to
    eigenvalues and (optionally) the corresponding approximate
    eigenvectors and/or an orthonormal basis for the associated
    approximate invariant subspace.
    The eigenvalues (and eigenvectors) are selected from those of a
    standard or generalized eigenvalue problem defined by real banded
    nonsymmetric matrices.
    There is negligible additional computational cost to obtain
    eigenvectors; an orthonormal basis is always computed, but there is
    an additional storage cost if both are requested.

    The banded matrices A and B must be stored using the LAPACK column
    ordered storage format for banded nonsymmetric matrices; please
    refer to [the F07 Introduction] for details on this storage format.

    ``real_band_solve`` is based on the banded driver functions
    **dnbdr1** to **dnbdr6** from the ARPACK package, which uses the
    Implicitly Restarted Arnoldi iteration method.
    The method is described in Lehoucq and Sorensen (1996) and Lehoucq
    (2001) while its use within the ARPACK software is described in
    great detail in Lehoucq `et al.` (1998).
    An evaluation of software for computing eigenvalues of sparse
    nonsymmetric matrices is provided in Lehoucq and Scott (1996).
    This suite of functions offers the same functionality as the ARPACK
    banded driver software for real nonsymmetric problems, but the
    interface design is quite different in order to make the option
    setting clearer and to combine the different drivers into a general
    purpose function.

    ``real_band_solve``, is a general purpose function that must be
    called following initialization by ``real_band_init``.
    ``real_band_solve`` uses options, set either by default or
    explicitly by calling ``real_option``, to return the converged
    approximations to selected eigenvalues and (optionally):

    -   the corresponding approximate eigenvectors;

    -   an orthonormal basis for the associated approximate invariant
        subspace;

    -   both.

    Please note that for 'Generalized' problems, the 'Shifted Inverse
    Imaginary', 'Shifted Inverse Real' inverse modes are only
    appropriate if either A or B is symmetric semidefinite.
    Otherwise, if A or B is non-singular, the 'Standard' problem can be
    solved using the matrix B^-1A (say).

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_band_solve'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12agft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._EngineFloat64ArrayType.c_type, # Input, ab[ldab, *].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Input, mb[ldmb, *].
        utils._BYREF_INT, # Input, ldmb.
        utils._BYREF_FLOAT64, # Input, sigmar.
        utils._BYREF_FLOAT64, # Input, sigmai.
        utils._BYREF_INT, # Output, nconv.
        utils._EngineFloat64ArrayType.c_type, # Output, dr[*].
        utils._EngineFloat64ArrayType.c_type, # Output, di[*].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, resid[*].
        utils._EngineFloat64ArrayType.c_type, # Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[60].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[140].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (mb, 'mb'), (z, 'z'), (v, 'v')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aff_n_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)),), validate=False)
    else:
        _vl_f12aff_n_py = 0
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    _ab = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(2*_kl_py+_ku_py+1, _vl_f12aff_n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldmb_py = utils._EngineFloat64ArrayType.get_nd(mb, {'fun_name': fun_name, 'entity': 'mb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldmb = utils._EngineIntScalarType(
        _ldmb_py, {'fun_name': fun_name, 'entity': '_ldmb'},
    ).to_ctypes()
    _mb = utils._EngineFloat64ArrayType(
        mb, {'fun_name': fun_name, 'entity': 'mb'},
        exp_shape=(2*_kl_py+_ku_py+1, _vl_f12aff_n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _sigmar = utils._EngineFloat64ScalarType(
        sigmar, {'fun_name': fun_name, 'entity': 'sigmar'},
    ).to_ctypes()
    _sigmai = utils._EngineFloat64ScalarType(
        sigmai, {'fun_name': fun_name, 'entity': 'sigmai'},
    ).to_ctypes()
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aff_nev_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+1,), validate=False)
    else:
        _vl_f12aff_nev_py = 0
    _dr_cls = utils._EngineFloat64ArrayType(
        dr, {'fun_name': fun_name, 'entity': 'dr'},
        exp_shape=(_vl_f12aff_nev_py+1,),
        check_contents=False,
    )
    _dr = _dr_cls.to_ctypes(can_reference_contents=False)
    _di_cls = utils._EngineFloat64ArrayType(
        di, {'fun_name': fun_name, 'entity': 'di'},
        exp_shape=(_vl_f12aff_nev_py+1,),
        check_contents=False,
    )
    _di = _di_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _comm = comm['comm']
    _icomm = comm['icomm']
    if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+7)-1,), validate=False) == 1:
        _z_dim2 = _vl_f12aff_nev_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_vl_f12aff_n_py, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _resid_cls = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vl_f12aff_n_py,),
    )
    _resid = _resid_cls.to_ctypes()
    _ldv_py = utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aff_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+2,), validate=False)
    else:
        _vl_f12aff_ncv_py = 0
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_vl_f12aff_n_py, _vl_f12aff_ncv_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _routine_name = utils._EngineRoutineName(*b'f12ag ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _kl,
        _ku,
        _ab,
        _ldab,
        _mb,
        _ldmb,
        _sigmar,
        _sigmai,
        _nconv,
        _dr,
        _di,
        _z,
        _ldz,
        _resid,
        _v,
        _ldv,
        _comm,
        _icomm,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: kl >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ku = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ku >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldab = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '2*kl+ku+1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ldab >= 2*kl+ku+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Shifted Inverse Imaginary\' has been',
                    'selected and sigmai = zero on entry; sigmai must be nonzero',
                    'for this mode of operation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations <= 0, the option',
                    '\'Iteration Limit\' has been set to {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The options \'Generalized\' and \'Regular\' are incompatible.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Initial Residual\' was selected but the',
                    'starting vector held in resid is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called prior',
                    'to the first call of this function or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 10:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, \'Vectors\' = Select, but this is not yet implemented.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 11:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of eigenvalues found to sufficient accuracy is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 12:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Could not build an Arnoldi factorization. The size',
                    'of the current Arnoldi factorization = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 13:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error in internal call to compute eigenvalues and corresponding',
                    'error bounds of the current upper Hessenberg matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 14:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'During calculation of a real Schur form, there was a failure',
                    'to compute a number of eigenvalues.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 15:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The computed Schur form could not be reordered by an internal call.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 16 and _ifmt == 16:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error in internal call to compute eigenvectors.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 17 and _ifmt == 17:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal factorization of real banded matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 18 and _ifmt == 18:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal solution of real banded matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 19 and _ifmt == 19:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal factorization of complex banded matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 20 and _ifmt == 20:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal solution of complex banded matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 21 and _ifmt == 21:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations has been reached.',
                    'The maximum number of iterations = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The number of converged eigenvalues = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 22 and _ifmt == 22:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No shifts could be applied during a cycle of the',
                    'implicitly restarted Lanczos iteration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 23 and _ifmt == 23:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Overflow occurred during transformation of Ritz values',
                    'to those of the original problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _dr_cls.refresh_py(_dr)
        _di_cls.refresh_py(_di)
        _z_cls.refresh_py(_z)
        _resid_cls.refresh_py(_resid)
        _v_cls.refresh_py(_v)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _nconv_py
    _err_class.manage()
    return return_data

def complex_init(n, nev, ncv):
    u"""
    Initialization routine for (``complex_iter``) computing selected
    eigenvalues and, optionally, eigenvectors of a complex sparse
    (standard or generalized) eigenproblem.

    ``complex_init`` is a setup function in a suite of functions
    consisting of ``complex_init``, ``complex_iter``, ``complex_proc``,
    ``complex_option`` and ``complex_monit``.
    It is used to find some of the eigenvalues (and optionally the
    corresponding eigenvectors) of a standard or generalized eigenvalue
    problem defined by complex nonsymmetric matrices.

    The suite of functions is suitable for the solution of large sparse,
    standard or generalized, nonsymmetric complex eigenproblems where
    only a few eigenvalues from a selected range of the spectrum are
    required.

    For full information please refer to the NAG Library document for
    f12an

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12anf.html

    Parameters
    ----------
    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    nev : int
        The number of eigenvalues to be computed.

    ncv : int
        The number of Arnoldi basis vectors to use during the
        computation.

        At present there is no `a priori` analysis to guide the
        selection of `ncv` relative to `nev`.

        However, it is recommended that `ncv` >= 2*`nev`+1.

        If many problems of the same type are to be solved, you should
        experiment with increasing `ncv` while keeping `nev` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the work and storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to CPU time is problem
        dependent and must be determined empirically.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nev` = *<value>*.

            Constraint: `nev` > 0.

        (`errno` 3)
            On entry, `ncv` = *<value>*, `nev` = *<value>* and `n` =
            *<value>*.

            Constraint: `ncv` >= `nev`+1 and `ncv` <= `n`.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, (and optionally the corresponding eigenvectors,
    x) of a standard complex eigenvalue problem Ax = lambda x, or of a
    generalized complex eigenvalue problem Ax = lambda Bx of order n,
    where n is large and the coefficient matrices A and B are sparse,
    complex and nonsymmetric.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale dense, complex and nonsymmetric problems.

    ``complex_init`` is a setup function which must be called before
    ``complex_iter``, the reverse communication iterative solver, and
    before ``complex_option``, the options setting function.
    ``complex_proc`` is a post-processing function that must be called
    following a successful final exit from ``complex_iter``, while
    ``complex_monit`` can be used to return additional monitoring
    information during the computation.

    This setup function initializes the communication arrays, sets (to
    their default values) all options that can be set by you via the
    option setting function ``complex_option``, and checks that the
    lengths of the communication arrays as passed by you are of
    sufficient length.
    For details of the options available and how to set them see [Other
    Parameters for complex_option].

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.complex_init'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12anft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nev.
        utils._BYREF_INT, # Input, ncv.
        utils._EngineIntArrayType.c_type, # Output, icomm[max(1, licomm)].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineComplex128ArrayType.c_type, # Output, comm[max(1, lcomm)].
        utils._BYREF_INT, # Input, lcomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nev = utils._EngineIntScalarType(
        nev, {'fun_name': fun_name, 'entity': 'nev'},
    ).to_ctypes()
    _ncv = utils._EngineIntScalarType(
        ncv, {'fun_name': fun_name, 'entity': 'ncv'},
    ).to_ctypes()
    _licomm = utils._EngineIntScalarType.c_type(140)
    _licomm_py = utils._EngineIntScalarType.to_py(_licomm)
    _icomm = utils._EngineIntArrayType.empty_ctypes(max(1, _licomm_py))
    _comm_py = {}
    _comm_py['icomm'] = _icomm
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ncv_py = utils._EngineIntScalarType.to_py(_ncv)
    _lcomm = utils._EngineIntScalarType(
        3*_n_py+3*_ncv_py*_ncv_py+5*_ncv_py+60, {'fun_name': fun_name, 'entity': '_lcomm'},
    ).to_ctypes()
    _lcomm_py = utils._EngineIntScalarType.to_py(_lcomm)
    _comm = utils._EngineComplex128ArrayType.empty_ctypes(max(1, _lcomm_py))
    _comm_py['comm'] = _comm
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _nev,
        _ncv,
        _icomm,
        _licomm,
        _comm,
        _lcomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nev > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncv = {:s}, nev = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: ncv >= nev+1 and ncv <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = _comm_py
    _err_class.manage()
    return return_data

def complex_iter(irevcm, resid, v, x, mx, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a complex
    sparse eigenproblem, reverse communication.

    ``complex_iter`` is an iterative solver in a suite of functions
    consisting of ``complex_init``, ``complex_iter``, ``complex_proc``,
    ``complex_option`` and ``complex_monit``.
    It is used to find some of the eigenvalues (and optionally the
    corresponding eigenvectors) of a standard or generalized eigenvalue
    problem defined by complex nonsymmetric matrices.

    For full information please refer to the NAG Library document for
    f12ap

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12apf.html

    Parameters
    ----------
    irevcm : int
        `On initial entry`: `irevcm` = 0, otherwise an error condition
        will be raised.

        `On intermediate entry`: must be unchanged from its previous
        exit value. Changing `irevcm` to any other value between calls
        will result in an error.

    resid : complex, shape (n), modified in place
        `On initial entry`: need not be set unless the option 'Initial
        Residual' has been set in a prior call to ``complex_option`` in
        which case `resid` should contain an initial residual vector,
        possibly from a previous run.

        `On intermediate entry`: must be unchanged from its previous
        exit. Changing `resid` to any other value between calls may
        result in an error exit.

        `On intermediate exit`: contains the current residual vector.

        `On final exit`: contains the final residual vector.

    v : complex, shape (n, ncv), modified in place
        `On initial entry`: need not be set.

        `On intermediate entry`: must be unchanged from its previous
        exit.

        `On intermediate exit`: contains the current set of Arnoldi
        basis vectors.

        `On final exit`: contains the final set of Arnoldi basis
        vectors.

    x : complex, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if
        `comm`['icomm'][(int(Re(`comm`['comm'][0]))+24)-1]=0:
        `comm`['icomm'][(int(Re(`comm`['comm'][0]))+1)-1]; otherwise: 1.

        `On initial entry`: need not be set, it is used as a convenient
        mechanism for accessing elements of `comm`['comm'].

        `On intermediate entry`: if 'Pointers' = YES, `x` need not be
        set.

        If 'Pointers' = NO, `x` must contain the result of y = op(x)
        when `irevcm` returns the value -1 or +1.

        It must return the computed shifts when `irevcm` returns the
        value 3.

        `On intermediate exit`: if 'Pointers' = YES, `x` is not
        referenced.

        If 'Pointers' = NO, `x` contains the vector x when `irevcm`
        returns the value -1 or +1.

        `On final exit`: does not contain useful data.

    mx : complex, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if
        `comm`['icomm'][(int(Re(`comm`['comm'][0]))+24)-1]=0:
        `comm`['icomm'][(int(Re(`comm`['comm'][0]))+1)-1]; otherwise: 1.

        `On initial entry`: need not be set, it is used as a convenient
        mechanism for accessing elements of `comm`['comm'].

        `On intermediate entry`: if 'Pointers' = YES, `mx` need not be
        set.

        If 'Pointers' = NO, `mx` must contain the result of y = Bx when
        `irevcm` returns the value 2.

        `On intermediate exit`: if 'Pointers' = YES, `mx` is not
        referenced.

        If 'Pointers' = NO, `mx` contains the vector Bx when `irevcm`
        returns the value +1.

        `On final exit`: does not contain any useful data.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``complex_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    irevcm : int
        `On intermediate exit`: has the following meanings.

        `irevcm` = -1

            The calling program must compute the matrix-vector product y
            = op(x), where x is stored in `x` (by default) or in the
            array `comm`['comm'] (starting from the location given by
            the first element of `comm`['icomm']) when the option
            'Pointers' = 'YES' is set in a prior call to
            ``complex_option``. The result y is returned in `x` (by
            default) or in the array `comm`['comm'] (starting from the
            location given by the second element of `comm`['icomm'])
            when the option 'Pointers' = 'YES' is set.

        `irevcm` = 1

            The calling program must compute the matrix-vector product y
            = op(x). This is similar to the case `irevcm` = -1 except
            that the result of the matrix-vector product Bx (as required
            in some computational modes) has already been computed and
            is available in `mx` (by default) or in the array
            `comm`['comm'] (starting from the location given by the
            third element of `comm`['icomm']) when the option 'Pointers'
            = 'YES' is set.

        `irevcm` = 2

            The calling program must compute the matrix-vector product y
            = Bx, where x is stored in `x` and y is returned in `mx` (by
            default) or in the array `comm`['comm'] (starting from the
            location given by the second element of `comm`['icomm'])
            when the option 'Pointers' = 'YES' is set.

        `irevcm` = 3

            Compute the `nshift` complex shifts. This value of `irevcm`
            will only arise if the option 'Supplied Shifts' is set in a
            prior call to ``complex_option`` which is intended for
            experienced users only; the default and recommended option
            is to use exact shifts (see Lehoucq `et al.` (1998) for
            details).

        `irevcm` = 4

            Monitoring step: a call to ``complex_monit`` can now be made
            to return the number of Arnoldi iterations, the number of
            converged Ritz values, the array of converged values, and
            the corresponding Ritz estimates.

        `On final exit`: `irevcm` = 5: ``complex_iter`` has completed
        its tasks. The value of errno determines whether the iteration
        has been successfully completed, or whether errors have been
        detected. On successful completion ``complex_proc`` must be
        called to return the requested eigenvalues and eigenvectors
        (and/or Schur vectors).

    nshift : int
        `On intermediate exit`: if the option 'Supplied Shifts' is set
        and `irevcm` returns a value of 3, `nshift` returns the number
        of complex shifts required.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The maximum number of iterations <= 0, the option 'Iteration
            Limit' has been set to *<value>*.

        (`errno` 2)
            The options 'Generalized' and 'Regular' are incompatible.

        (`errno` 5)
            No shifts could be applied during a cycle of the implicitly
            restarted Arnoldi iteration.

        (`errno` 6)
            Could not build an Arnoldi factorization. The size of the
            current Arnoldi factorization = *<value>*.

        (`errno` 7)
            Error in internal call to compute eigenvalues and
            corresponding error bounds of the current upper Hessenberg
            matrix. Please contact NAG.

        (`errno` 8)
            Either the initialization function has not been called prior
            to the first call of this function or a communication array
            has become corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            The option 'Initial Residual' was selected but the starting
            vector held in `resid` is zero.

        (`errno` 4)
            The maximum number of iterations has been reached. The
            maximum number of iterations = *<value>*. The number of
            converged eigenvalues = *<value>*. See the function document
            for further details.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, (and optionally the corresponding eigenvectors,
    x) of a standard eigenvalue problem Ax = lambda x, or of a
    generalized eigenvalue problem Ax = lambda Bx of order n, where n is
    large and the coefficient matrices A and B are sparse, complex and
    nonsymmetric.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale dense, complex and nonsymmetric problems.

    ``complex_iter`` is a **reverse communication** function, based on
    the ARPACK routine **znaupd**, using the Implicitly Restarted
    Arnoldi iteration method.
    The method is described in Lehoucq and Sorensen (1996) and Lehoucq
    (2001) while its use within the ARPACK software is described in
    great detail in Lehoucq `et al.` (1998).
    An evaluation of software for computing eigenvalues of sparse
    nonsymmetric matrices is provided in Lehoucq and Scott (1996).
    This suite of functions offers the same functionality as the ARPACK
    software for complex nonsymmetric problems, but the interface design
    is quite different in order to make the option setting clearer and
    to simplify the interface of ``complex_iter``.

    The setup function ``complex_init`` must be called before
    ``complex_iter``, the reverse communication iterative solver.
    Options may be set for ``complex_iter`` by prior calls to the option
    setting function ``complex_option`` and a post-processing function
    ``complex_proc`` must be called following a successful final exit
    from ``complex_iter``. ``complex_monit`` may be called following
    certain flagged, intermediate exits from ``complex_iter`` to provide
    additional monitoring information about the computation.

    ``complex_iter`` uses **reverse communication**, i.e., it returns
    repeatedly to the calling program with the argument `irevcm` (see
    [Parameters]) set to specified values which require the calling
    program to carry out one of the following tasks:

    -   compute the matrix-vector product y = op(x), where op is defined
        by the computational mode;

    -   compute the matrix-vector product y = Bx;

    -   notify the completion of the computation;

    -   allow the calling program to monitor the solution.

    The problem type to be solved (standard or generalized), the
    spectrum of eigenvalues of interest, the mode used (regular, regular
    inverse, shifted inverse, shifted real or shifted imaginary) and
    other options can all be set using the option setting function
    ``complex_option`` (see [Other Parameters for complex_option] for
    details on setting options and of the default settings).

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.complex_iter'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12apft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, resid[*].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x[*].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, mx[*].
        utils._BYREF_INT, # Output, nshift.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, comm[*].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((v, 'v'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_n_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real),), validate=False)
    else:
        _vl_f12anf_n_py = 0
    _resid_cls = utils._EngineComplex128ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vl_f12anf_n_py,),
    )
    _resid = _resid_cls.to_ctypes()
    _ldv_py = utils._EngineComplex128ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+2,), validate=False)
    else:
        _vl_f12anf_ncv_py = 0
    _v_cls = utils._EngineComplex128ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_vl_f12anf_n_py, _vl_f12anf_ncv_py),
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12anf_lcomm_py = 0
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12anf_licomm_py = 0
    _comm = comm['comm']
    _icomm = comm['icomm']
    if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+24)-1,), validate=False) == 0:
        _x_dim1 = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+1)-1,), validate=False)
    else:
        _x_dim1 = 1
    exp_shape_x = (_x_dim1,)
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=exp_shape_x,
    )
    _x = _x_cls.to_ctypes()
    if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+24)-1,), validate=False) == 0:
        _mx_dim1 = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+1)-1,), validate=False)
    else:
        _mx_dim1 = 1
    exp_shape_mx = (_mx_dim1,)
    _mx_cls = utils._EngineComplex128ArrayType(
        mx, {'fun_name': fun_name, 'entity': 'mx'},
        exp_shape=exp_shape_mx,
    )
    _mx = _mx_cls.to_ctypes()
    _nshift = utils._EngineIntScalarType.empty_ctypes()
    _routine_name = utils._EngineRoutineName(*b'f12ap ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _irevcm,
        _resid,
        _v,
        _ldv,
        _x,
        _mx,
        _nshift,
        _comm,
        _icomm,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 5 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations <= 0, the option',
                    '\'Iteration Limit\' has been set to {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The options \'Generalized\' and \'Regular\' are incompatible.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Initial Residual\' was selected but the',
                    'starting vector held in resid is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations has been reached.',
                    'The maximum number of iterations = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The number of converged eigenvalues = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'See the function document for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No shifts could be applied during a cycle of the',
                    'implicitly restarted Arnoldi iteration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Could not build an Arnoldi factorization. The size',
                    'of the current Arnoldi factorization = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error in internal call to compute eigenvalues and corresponding',
                    'error bounds of the current upper Hessenberg matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called prior',
                    'to the first call of this function or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _resid_cls.refresh_py(_resid)
        _v_cls.refresh_py(_v)
        _x_cls.refresh_py(_x)
        _mx_cls.refresh_py(_mx)
        _nshift_py = utils._EngineIntScalarType.to_py(_nshift)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['irevcm', 'nshift'],
        )(
            _irevcm_py,
            _nshift_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_proc(d, z, sigma, resid, v, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a complex
    sparse eigenproblem, postprocessing for ``complex_iter``.

    ``complex_proc`` is a post-processing function in a suite of
    functions consisting of ``complex_init``, ``complex_iter``,
    ``complex_proc``, ``complex_option`` and ``complex_monit``.
    It must be called following a final exit from ``complex_iter``.

    For full information please refer to the NAG Library document for
    f12aq

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12aqf.html

    Parameters
    ----------
    d : complex, shape (ncv), modified in place
        `On exit`: the first `nconv` locations of the array `d` contain
        the converged approximate eigenvalues.

    z : complex, shape (n, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if
        `comm`['icomm'][(int(Re(`comm`['comm'][0]))+7)-1]=1:
        `comm`['icomm'][(int(Re(`comm`['comm'][0]))+2)-1]; otherwise: 1.

        `On exit`: if the default option 'Vectors' = 'RITZ' (see
        ``real_option``) has been selected then `z` contains the final
        set of eigenvectors corresponding to the eigenvalues held in
        `d`. The complex eigenvector associated with an eigenvalue is
        stored in the corresponding of `z`.

    sigma : complex
        If one of the 'Shifted Inverse' (see ``complex_option``) modes
        has been selected then `sigma` contains the shift used;
        otherwise `sigma` is not referenced.

    resid : complex, shape (n)
        Must not be modified following a call to ``complex_iter`` since
        it contains data required by ``complex_proc``.

    v : complex, shape (n, ncv), modified in place
        `On entry`: the ncv columns of `v` contain the Arnoldi basis
        vectors for op as constructed by ``complex_iter``.

        `On exit`: if the option 'Vectors' = 'SCHUR' or 'RITZ' has been
        set and a separate array `z` has been passed (i.e., `z` does not
        equal `v`), then the first `nconv` columns of `v` will contain
        approximate Schur vectors that span the desired invariant
        subspace.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``complex_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged eigenvalues as found by
        ``complex_option``.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, 'Vectors' = Select, but this is not yet
            implemented.

        (`errno` 3)
            The number of eigenvalues found to sufficient accuracy, as
            communicated through the argument `comm`['icomm'], is zero.

        (`errno` 4)
            Got a different count of the number of converged Ritz values
            than the value passed to it through the argument
            `comm`['icomm']: number counted = *<value>*, number expected
            = *<value>*.

        (`errno` 5)
            During calculation of a Schur form, there was a failure to
            compute *<value>* eigenvalues in a total of *<value>*
            iterations.

        (`errno` 6)
            The computed Schur form could not be reordered by an
            internal call. This function returned with errno =
            *<value>*. Please contact NAG.

        (`errno` 7)
            In calculating eigenvectors, an internal call returned with
            an error. The function returned with errno = *<value>*.
            Please contact NAG.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, (and optionally the corresponding eigenvectors,
    x) of a standard eigenvalue problem Ax = lambda x, or of a
    generalized eigenvalue problem Ax = lambda Bx of order n, where n is
    large and the coefficient matrices A and B are sparse, complex and
    nonsymmetric.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale dense, complex and nonsymmetric problems.

    Following a call to ``complex_iter``, ``complex_proc`` returns the
    converged approximations to eigenvalues and (optionally) the
    corresponding approximate eigenvectors and/or an orthonormal basis
    for the associated approximate invariant subspace.
    The eigenvalues (and eigenvectors) are selected from those of a
    standard or generalized eigenvalue problem defined by complex
    nonsymmetric matrices.
    There is negligible additional cost to obtain eigenvectors; an
    orthonormal basis is always computed, but there is an additional
    storage cost if both are requested.

    ``complex_proc`` is based on the function **zneupd** from the ARPACK
    package, which uses the Implicitly Restarted Arnoldi iteration
    method.
    The method is described in Lehoucq and Sorensen (1996) and Lehoucq
    (2001) while its use within the ARPACK software is described in
    great detail in Lehoucq `et al.` (1998).
    An evaluation of software for computing eigenvalues of sparse
    nonsymmetric matrices is provided in Lehoucq and Scott (1996).
    This suite of functions offers the same functionality as the ARPACK
    software for complex nonsymmetric problems, but the interface design
    is quite different in order to make the option setting clearer and
    to simplify some of the interfaces.

    ``complex_proc`` is a post-processing function that must be called
    following a successful final exit from ``complex_iter``.
    ``complex_proc`` uses data returned from ``complex_iter`` and
    options, set either by default or explicitly by calling
    ``complex_option``, to return the converged approximations to
    selected eigenvalues and (optionally):

    -   the corresponding approximate eigenvectors;

    -   an orthonormal basis for the associated approximate invariant
        subspace;

    -   both.

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.complex_proc'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12aqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Output, nconv.
        utils._EngineComplex128ArrayType.c_type, # Output, d[*].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._BYREF_COMPLEX128, # Input, sigma.
        utils._EngineComplex128ArrayType.c_type, # Input, resid[*].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, comm[*].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'), (v, 'v')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+2,), validate=False)
    else:
        _vl_f12anf_ncv_py = 0
    _d_cls = utils._EngineComplex128ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_vl_f12anf_ncv_py,),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12anf_lcomm_py = 0
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12anf_licomm_py = 0
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _comm = comm['comm']
    _icomm = comm['icomm']
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_n_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real),), validate=False)
    else:
        _vl_f12anf_n_py = 0
    if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+7)-1,), validate=False) == 1:
        _z_dim2 = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+2)-1,), validate=False)
    else:
        _z_dim2 = 1
    exp_shape_z = (_vl_f12anf_n_py, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _sigma = utils._EngineComplex128ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _resid = utils._EngineComplex128ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vl_f12anf_n_py,),
    ).to_ctypes()
    _ldv_py = utils._EngineComplex128ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    _v_cls = utils._EngineComplex128ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_vl_f12anf_n_py, _vl_f12anf_ncv_py),
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    _routine_name = utils._EngineRoutineName(*b'f12aq ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _nconv,
        _d,
        _z,
        _ldz,
        _sigma,
        _resid,
        _v,
        _ldv,
        _comm,
        _icomm,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, \'Vectors\' = Select, but this is not yet implemented.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of eigenvalues found to sufficient accuracy,',
                    'as communicated through the argument comm[\'icomm\'], is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Got a different count of the number of converged Ritz values',
                    'than the value passed to it through the argument comm[\'icomm\']:',
                    'number counted = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'number expected = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'During calculation of a Schur form, there was a failure',
                    'to compute {:s} eigenvalues in a total of'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The computed Schur form could not be reordered by an internal call.',
                    'This function returned with errno = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In calculating eigenvectors, an internal call returned with an error.',
                    'The function returned with errno = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        _z_cls.refresh_py(_z)
        _v_cls.refresh_py(_v)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _nconv_py
    _err_class.manage()
    return return_data

def complex_option(optstr, comm, io_manager=None):
    u"""
    Set a single option from a string
    (``complex_iter``/``complex_proc``).

    Note: this function uses optional algorithmic parameters, see also:
    ``complex_iter``, ``complex_proc``, ``complex_band_solve``,
    ``complex_init``, ``complex_band_init``.

    ``complex_option`` is an option setting function in a suite of
    functions consisting of ``complex_init``, ``complex_iter``,
    ``complex_proc``, ``complex_option`` and ``complex_monit``; it
    supplies individual options to ``complex_iter`` and
    ``complex_proc``. ``complex_option`` is also an option setting
    function in a suite of functions consisting of ``complex_option``,
    ``complex_band_init`` and ``complex_band_solve``; it supplies
    individual options to ``complex_band_solve``.

    The initialization function for the appropriate suite,
    ``complex_init`` or ``complex_band_init``, **must** have been called
    prior to calling ``complex_option``.

    For full information please refer to the NAG Library document for
    f12ar

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12arf.html

    Parameters
    ----------
    optstr : str
        A single valid option string (as described in [Notes] and [Other
        Parameters]).

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``complex_init`` or ``complex_band_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Other Parameters
    ----------------
    'Advisory' : int
        Default = advisory message unit number

        The destination for advisory messages.

    'Defaults' : valueless
        This special keyword may be used to reset all options to their
        default values.

    'Exact Shifts' : valueless
        Default

        During the Arnoldi iterative process, shifts are applied as part
        of the implicit restarting scheme.
        The shift strategy used by default and selected by the option
        'Exact Shifts' is strongly recommended over the alternative
        'Supplied Shifts' and will always be used by
        ``complex_band_solve``.

        If 'Exact Shifts' are used then these are computed internally by
        the algorithm in the implicit restarting scheme.
        This strategy is generally effective and cheaper to apply in
        terms of number of operations than using explicit shifts.

        If 'Supplied Shifts' are used then, during the Arnoldi iterative
        process, you must supply shifts through array arguments of
        ``complex_iter`` when ``complex_iter`` returns with irevcm = 3;
        the complex shifts are returned in x (or in comm when the option
        'Pointers' = 'YES' is set). This option should only be used if
        you are an experienced user since this requires some algorithmic
        knowledge and because more operations are usually required than
        for the implicit shift scheme.
        Details on the use of explicit shifts and further references on
        shift strategies are available in Lehoucq `et al.` (1998).

    'Supplied Shifts' : valueless
        During the Arnoldi iterative process, shifts are applied as part
        of the implicit restarting scheme.
        The shift strategy used by default and selected by the option
        'Exact Shifts' is strongly recommended over the alternative
        'Supplied Shifts' and will always be used by
        ``complex_band_solve``.

        If 'Exact Shifts' are used then these are computed internally by
        the algorithm in the implicit restarting scheme.
        This strategy is generally effective and cheaper to apply in
        terms of number of operations than using explicit shifts.

        If 'Supplied Shifts' are used then, during the Arnoldi iterative
        process, you must supply shifts through array arguments of
        ``complex_iter`` when ``complex_iter`` returns with irevcm = 3;
        the complex shifts are returned in x (or in comm when the option
        'Pointers' = 'YES' is set). This option should only be used if
        you are an experienced user since this requires some algorithmic
        knowledge and because more operations are usually required than
        for the implicit shift scheme.
        Details on the use of explicit shifts and further references on
        shift strategies are available in Lehoucq `et al.` (1998).

    'Iteration Limit' : int
        Default = 300

        The limit on the number of Arnoldi iterations that can be
        performed before ``complex_iter`` or ``complex_band_solve``
        exits.
        If not all requested eigenvalues have converged to within
        'Tolerance' and the number of Arnoldi iterations has reached
        this limit then ``complex_iter`` or ``complex_band_solve`` exits
        with an error; ``complex_band_solve`` returns the number of
        converged eigenvalues, the converged eigenvalues and, if
        requested, the corresponding eigenvectors, while
        ``complex_proc`` can be called subsequent to ``complex_iter`` to
        do the same.

    'Largest Magnitude' : valueless
        Default

        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``complex_iter`` or ``complex_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of op (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Largest Imaginary' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``complex_iter`` or ``complex_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of op (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Largest Real' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``complex_iter`` or ``complex_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of op (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Imaginary' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``complex_iter`` or ``complex_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of op (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Magnitude' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``complex_iter`` or ``complex_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of op (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Real' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``complex_iter`` or ``complex_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of op (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Nolist' : valueless
        Default

        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'List' : valueless
        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'Monitoring' : int
        Default = -1

        If i > 0, monitoring information is output to unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`)
        i during the solution of each problem; this may be the same as
        the 'Advisory' unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).
        The type of information produced is dependent on the value of
        'Print Level', see the description of the option 'Print Level'
        for details of the information produced.
        Please see the ``FileObjManager`` method
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`
        to associate a file with a given unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).

    'Pointers' : valueless
        Default = 'NO'

        During the iterative process and reverse communication calls to
        ``complex_iter``, required data can be communicated to and from
        ``complex_iter`` in one of two ways.
        When 'Pointers' = 'NO' is selected (the default) then the array
        arguments x and mx are used to supply you with required data and
        used to return computed values back to ``complex_iter``.
        For example, when irevcm = 1, ``complex_iter`` returns the
        vector x in x and the matrix-vector product Bx in mx and expects
        the result or the linear operation op(x) to be returned in x.

        If 'Pointers' = 'YES' is selected then the data is passed
        through sections of the array argument comm.
        The section corresponding to x when 'Pointers' = 'NO' begins at
        a location given by the first element of icomm; similarly the
        section corresponding to mx begins at a location given by the
        second element of icomm.
        This option allows ``complex_iter`` to perform fewer copy
        operations on each intermediate exit and entry, but can also
        lead to less elegant code in the calling program.

        This option has no affect on ``complex_band_solve`` which sets
        'Pointers' = 'YES' internally.

    'Print Level' : int
        Default = 0

        This controls the amount of printing produced by
        ``complex_option`` as follows.

        [table omitted]

    'Random Residual' : valueless
        Default

        To begin the Arnoldi iterative process, ``complex_iter`` or
        ``complex_band_solve`` requires an initial residual vector.
        By default ``complex_iter`` or ``complex_band_solve`` provides
        its own random initial residual vector; this option can also be
        set using option 'Random Residual'.
        Alternatively, you can supply an initial residual vector
        (perhaps from a previous computation) to ``complex_iter`` or
        ``complex_band_solve`` through the array argument resid; this
        option can be set using option 'Initial Residual'.

    'Initial Residual' : valueless
        To begin the Arnoldi iterative process, ``complex_iter`` or
        ``complex_band_solve`` requires an initial residual vector.
        By default ``complex_iter`` or ``complex_band_solve`` provides
        its own random initial residual vector; this option can also be
        set using option 'Random Residual'.
        Alternatively, you can supply an initial residual vector
        (perhaps from a previous computation) to ``complex_iter`` or
        ``complex_band_solve`` through the array argument resid; this
        option can be set using option 'Initial Residual'.

    'Regular' : valueless
        Default

        These options define the computational mode which in turn
        defines the form of operation op(x) to be performed by
        ``complex_band_solve`` or when ``complex_iter`` returns with
        irevcm = -1 or 1 and the matrix-vector product Bx when
        ``complex_iter`` returns with irevcm = -2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

    'Regular Inverse' : valueless
        These options define the computational mode which in turn
        defines the form of operation op(x) to be performed by
        ``complex_band_solve`` or when ``complex_iter`` returns with
        irevcm = -1 or 1 and the matrix-vector product Bx when
        ``complex_iter`` returns with irevcm = -2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

    'Shifted Inverse' : valueless
        These options define the computational mode which in turn
        defines the form of operation op(x) to be performed by
        ``complex_band_solve`` or when ``complex_iter`` returns with
        irevcm = -1 or 1 and the matrix-vector product Bx when
        ``complex_iter`` returns with irevcm = -2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

    'Standard' : valueless
        Default

        The problem to be solved is either a standard eigenvalue
        problem, Ax = lambda x, or a generalized eigenvalue problem, Ax
        = lambda Bx.
        The option 'Standard' should be used when a standard eigenvalue
        problem is being solved and the option 'Generalized' should be
        used when a generalized eigenvalue problem is being solved.

    'Generalized' : valueless
        The problem to be solved is either a standard eigenvalue
        problem, Ax = lambda x, or a generalized eigenvalue problem, Ax
        = lambda Bx.
        The option 'Standard' should be used when a standard eigenvalue
        problem is being solved and the option 'Generalized' should be
        used when a generalized eigenvalue problem is being solved.

    'Tolerance' : float
        Default = epsilon

        An approximate eigenvalue has deemed to have converged when the
        corresponding Ritz estimate is within 'Tolerance' relative to
        the magnitude of the eigenvalue.

    'Vectors' : valueless
        Default = 'RITZ'

        The function ``complex_proc`` or ``complex_band_solve`` can
        optionally compute the Schur vectors and/or the eigenvectors
        corresponding to the converged eigenvalues.
        To turn off computation of any vectors the option 'Vectors' =
        'NONE' should be set.
        To compute only the Schur vectors (at very little extra cost),
        the option 'Vectors' = 'SCHUR' should be set and these will be
        returned in the array argument v of ``complex_proc`` or
        ``complex_band_solve``.
        To compute the eigenvectors (Ritz vectors) ­corresponding to the
        eigenvalue estimates, the option 'Vectors' = 'RITZ' should be
        set and these will be returned in the array argument z of
        ``complex_proc`` or ``complex_band_solve``, if z is set equal to
        v then the Schur vectors in v are overwritten by the
        eigenvectors computed by ``complex_proc`` or
        ``complex_band_solve``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Ambiguous keyword: *<value>*

        (`errno` 2)
            Keyword not recognized: *<value>*

        (`errno` 3)
            Second keyword not recognized: *<value>*

        (`errno` 4)
            Either the initialization function has not been called prior
            to the call of this function or a communication array has
            become corrupted.

    Notes
    -----
    ``complex_option`` may be used to supply values for options to
    ``complex_iter`` and ``complex_proc``, or to ``complex_band_solve``.
    It is only necessary to call ``complex_option`` for those arguments
    whose values are to be different from their default values.
    One call to ``complex_option`` sets one argument value.

    Each option is defined by a single character string consisting of
    one or more items.
    The items associated with a given option must be separated by
    spaces, or equals signs [ = ].
    Alphabetic characters may be upper or lower case.
    The string

    ::

      Iteration Limit = 500

    is an example of a string used to set an option.
    For each option the string contains one or more of the following
    items:

    -   a mandatory keyword;

    -   a phrase that qualifies the keyword;

    -   a number that specifies an `int` or `float` value. Such numbers
        may be up to 16 contiguous characters in Fortran's I, F, E or D
        format.

    ``complex_option`` does not have an equivalent function from the
    ARPACK package which passes options by directly setting values to
    scalar arguments or to specific elements of array arguments.
    ``complex_option`` is intended to make the passing of options more
    transparent and follows the same principle as the single option
    setting functions in submodule ``opt`` (see
    ``opt.qpconvex2_sparse_option_string`` for an example).

    The setup function ``complex_init`` or ``complex_band_init`` must be
    called prior to the first call to ``complex_option``, and all calls
    to ``complex_option`` must precede the first call to
    ``complex_iter`` and ``complex_band_solve``.

    A complete list of options, their abbreviations, synonyms and
    default values is given in [Other Parameters].

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.complex_option'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12arft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_STRING, # Input, optstr(len=*).
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, comm[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of optstr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _optstr = utils._EngineCharScalarType(
        optstr, {'fun_name': fun_name, 'entity': 'optstr'}, exp_chrlen=None,
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm', 'comm'),
    )
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12anf_licomm_py = 0
    _icomm = comm['icomm']
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12anf_lcomm_py = 0
    _comm = comm['comm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _optstr,
        _icomm,
        _comm,
        _errbuf,
        _ierr,
        _optstr._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Ambiguous keyword: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Keyword not recognized: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Second keyword not recognized: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 4:
            _ierr_fl = 4
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called prior',
                    'to the call of this function or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = None
    _err_class.manage()
    return return_data

def complex_monit(ritz, rzest, comm):
    u"""
    Provides monitoring information for ``complex_iter``.

    ``complex_monit`` can be used to return additional monitoring
    information during computation.
    It is in a suite of functions consisting of ``complex_init``,
    ``complex_iter``, ``complex_proc``, ``complex_option`` and
    ``complex_monit``.

    For full information please refer to the NAG Library document for
    f12as

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12asf.html

    Parameters
    ----------
    ritz : complex, shape (ncv), modified in place
        `On exit`: the first `nconv` locations of the array `ritz`
        contain the converged approximate eigenvalues.

    rzest : complex, shape (ncv), modified in place
        `On exit`: the first `nconv` locations of the array `rzest`
        contain the complex Ritz estimates on the converged approximate
        eigenvalues.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``complex_init``.

    Returns
    -------
    niter : int
        The number of the current Arnoldi iteration.

    nconv : int
        The number of converged eigenvalues so far.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, (and optionally the corresponding eigenvectors,
    x) of a standard complex eigenvalue problem Ax = lambda x, or of a
    generalized complex eigenvalue problem Ax = lambda Bx of order n,
    where n is large and the coefficient matrices A and B are sparse and
    complex.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale dense complex problems.

    On an intermediate exit from ``complex_iter`` with irevcm = 4,
    ``complex_monit`` may be called to return monitoring information on
    the progress of the Arnoldi iterative process.
    The information returned by ``complex_monit`` is:

    -   the number of the current Arnoldi iteration;

    -   the number of converged eigenvalues at this point;

    -   the converged eigenvalues;

    -   the error bounds on the converged eigenvalues.

    ``complex_monit`` does not have an equivalent function from the
    ARPACK package which prints various levels of detail of monitoring
    information through an output channel controlled via an argument
    value (see Lehoucq `et al.` (1998) for details of ARPACK routines).
    ``complex_monit`` should not be called at any time other than
    immediately following an irevcm = 4 return from ``complex_iter``.

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.complex_monit'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12asft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Output, niter.
        utils._BYREF_INT, # Output, nconv.
        utils._EngineComplex128ArrayType.c_type, # Output, ritz[*].
        utils._EngineComplex128ArrayType.c_type, # Output, rzest[*].
        utils._EngineIntArrayType.c_type, # Input, icomm[*].
        utils._EngineComplex128ArrayType.c_type, # Input, comm[*].
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _niter = utils._EngineIntScalarType.empty_ctypes()
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm', 'comm'),
    )
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+2,), validate=False)
    else:
        _vl_f12anf_ncv_py = 0
    _ritz_cls = utils._EngineComplex128ArrayType(
        ritz, {'fun_name': fun_name, 'entity': 'ritz'},
        exp_shape=(_vl_f12anf_ncv_py,),
        check_contents=False,
    )
    _ritz = _ritz_cls.to_ctypes(can_reference_contents=False)
    _rzest_cls = utils._EngineComplex128ArrayType(
        rzest, {'fun_name': fun_name, 'entity': 'rzest'},
        exp_shape=(_vl_f12anf_ncv_py,),
        check_contents=False,
    )
    _rzest = _rzest_cls.to_ctypes(can_reference_contents=False)
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12anf_licomm_py = 0
    _icomm = comm['icomm']
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12anf_lcomm_py = 0
    _comm = comm['comm']
    engine_ctypes(
        _en_data,
        _niter,
        _nconv,
        _ritz,
        _rzest,
        _icomm,
        _comm,
    )
    utils._handle_cb_exception(excs)
    _ritz_cls.refresh_py(_ritz)
    _rzest_cls.refresh_py(_rzest)
    _niter_py = utils._EngineIntScalarType.to_py(_niter)
    _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
    return_data = _collections.namedtuple(
        'nag_return_data',
        ['niter', 'nconv'],
    )(
        _niter_py,
        _nconv_py,
    )
    return return_data

def complex_band_init(n, nev, ncv):
    u"""
    Initialization function for ``complex_band_solve`` computing
    selected eigenvalues and, optionally, eigenvectors of a complex
    banded (standard or generalized) eigenproblem.

    ``complex_band_init`` is a setup function for ``complex_band_solve``
    which may be used for finding some eigenvalues (and optionally the
    corresponding eigenvectors) of a standard or generalized eigenvalue
    problem defined by complex, banded, non-Hermitian matrices.
    The banded matrix must be stored using the LAPACK column ordered
    storage format for complex banded non-Hermitian matrices (see [the
    F07 Introduction]).

    For full information please refer to the NAG Library document for
    f12at

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12atf.html

    Parameters
    ----------
    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    nev : int
        The number of eigenvalues to be computed.

    ncv : int
        The number of Lanczos basis vectors to use during the
        computation.

        At present there is no `a priori` analysis to guide the
        selection of `ncv` relative to `nev`.

        However, it is recommended that `ncv` >= 2*`nev`+1.

        If many problems of the same type are to be solved, you should
        experiment with increasing `ncv` while keeping `nev` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the work and storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to CPU time is problem
        dependent and must be determined empirically.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nev` = *<value>*.

            Constraint: `nev` > 0.

        (`errno` 3)
            On entry, `ncv` = *<value>*, `nev` = *<value>* and `n` =
            *<value>*.

            Constraint: `ncv` > `nev`+1 and `ncv` <= `n`.

    Notes
    -----
    The pair of functions ``complex_band_init`` and
    ``complex_band_solve`` together with the option setting function
    ``complex_option`` are designed to calculate some of the
    eigenvalues, lambda, (and optionally the corresponding eigenvectors,
    x) of a standard eigenvalue problem Ax = lambda x, or of a
    generalized eigenvalue problem Ax = lambda Bx of order n, where n is
    large and the coefficient matrices A and B are banded complex and
    non-Hermitian.

    ``complex_band_init`` is a setup function which must be called
    before the option setting function ``complex_option`` and the solver
    function ``complex_band_solve``.
    Internally, ``complex_band_solve`` makes calls to ``complex_iter``
    and ``complex_proc``; the function documents for ``complex_iter``
    and ``complex_proc`` should be consulted for details of the
    algorithm used.

    This setup function initializes the communication arrays, sets (to
    their default values) all options that can be set by you via the
    option setting function ``complex_option``, and checks that the
    lengths of the communication arrays as passed by you are of
    sufficient length.
    For details of the options available and how to set them, see [Other
    Parameters for complex_option].

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.complex_band_init'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12atft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nev.
        utils._BYREF_INT, # Input, ncv.
        utils._EngineIntArrayType.c_type, # Output, icomm[max(1, licomm)].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineComplex128ArrayType.c_type, # Output, comm[max(1, lcomm)].
        utils._BYREF_INT, # Input, lcomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nev = utils._EngineIntScalarType(
        nev, {'fun_name': fun_name, 'entity': 'nev'},
    ).to_ctypes()
    _ncv = utils._EngineIntScalarType(
        ncv, {'fun_name': fun_name, 'entity': 'ncv'},
    ).to_ctypes()
    _licomm = utils._EngineIntScalarType.c_type(140)
    _licomm_py = utils._EngineIntScalarType.to_py(_licomm)
    _icomm = utils._EngineIntArrayType.empty_ctypes(max(1, _licomm_py))
    _comm_py = {}
    _comm_py['icomm'] = _icomm
    _lcomm = utils._EngineIntScalarType.c_type(60)
    _lcomm_py = utils._EngineIntScalarType.to_py(_lcomm)
    _comm = utils._EngineComplex128ArrayType.empty_ctypes(max(1, _lcomm_py))
    _comm_py['comm'] = _comm
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _nev,
        _ncv,
        _icomm,
        _licomm,
        _comm,
        _lcomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nev > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncv = {:s}, nev = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: ncv > nev+1 and ncv <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = _comm_py
    _err_class.manage()
    return return_data

def complex_band_solve(kl, ku, ab, mb, sigma, d, z, resid, v, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of complex
    non-Hermitian banded eigenproblem, driver.

    ``complex_band_solve`` is the main solver function in a suite of
    functions consisting of ``complex_option``, ``complex_band_init``
    and ``complex_band_solve``.
    It must be called following an initial call to ``complex_band_init``
    and following any calls to ``complex_option``.

    ``complex_band_solve`` returns approximations to selected
    eigenvalues, and (optionally) the corresponding eigenvectors, of a
    standard or generalized eigenvalue problem defined by complex banded
    non-Hermitian matrices.
    The banded matrix must be stored using the LAPACK storage format for
    complex banded non-Hermitian matrices.

    For full information please refer to the NAG Library document for
    f12au

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12auf.html

    Parameters
    ----------
    kl : int
        The number of subdiagonals of the matrices A and B.

    ku : int
        The number of superdiagonals of the matrices A and B.

    ab : complex, shape (2*`kl`+`ku`+1, n)
        Must contain the matrix A in LAPACK banded storage format for
        non-Hermitian matrices (see [the F07 Introduction]).

    mb : complex, shape (2*`kl`+`ku`+1, n)
        Must contain the matrix B in LAPACK banded storage format for
        non-Hermitian matrices (see [the F07 Introduction]).

    sigma : complex
        If the 'Shifted Inverse' mode (see ``complex_option``) has been
        selected then `sigma` must contain the shift used; otherwise
        `sigma` is not referenced. [the F12 Introduction] describes the
        use of shift and invert transformations.

    d : complex, shape (nev), modified in place
        `On exit`: the first `nconv` locations of the array `d` contain
        the converged approximate eigenvalues.

    z : None or complex, shape (n, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if
        `comm`['icomm'][(int(Re(`comm`['comm'][0]))+7)-1]=1: nev;
        otherwise: 1.

        `On exit`, if not None on entry: if the default option 'Vectors'
        = 'RITZ' (see ``complex_option``) has been selected then `z`
        contains the final set of eigenvectors corresponding to the
        eigenvalues held in `d`, otherwise `z` is not referenced. The
        complex eigenvector associated with an eigenvalue `d`[j] is
        stored in the corresponding array section of `z`, namely
        `z`[i-1,j-1], for j = 1,2,...,`nconv`, for i = 1,2,...,n.

    resid : complex, shape (n), modified in place
        `On entry`: need not be set unless the option 'Initial Residual'
        has been set in a prior call to ``complex_option`` in which case
        `resid` must contain an initial residual vector.

        `On exit`: contains the final residual vector. This can be used
        as the starting residual to improve convergence on the solution
        of a closely related eigenproblem. This has no relation to the
        error residual Ax-lambda x or Ax-lambda Bx.

    v : complex, shape (n, ncv), modified in place
        `On exit`: if the option 'Vectors' = 'SCHUR' or 'RITZ' (see
        ``complex_option``) has been set and a separate array `z` has
        been passed (i.e., `z` does not equal `v`), then the first
        `nconv` columns of `v` will contain approximate Schur vectors
        that span the desired invariant subspace.

        The jth Schur vector is stored in the ith column of `v`.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``complex_band_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged eigenvalues.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `kl` = *<value>*.

            Constraint: `kl` >= 0.

        (`errno` 2)
            On entry, `ku` = *<value>*.

            Constraint: `ku` >= 0.

        (`errno` 3)
            On entry, ldab = *<value>*, 2*`kl`+`ku`+1 = *<value>*.

            Constraint: ldab >= 2*`kl`+`ku`+1.

        (`errno` 5)
            The maximum number of iterations <= 0, the option 'Iteration
            Limit' has been set to *<value>*.

        (`errno` 6)
            The options 'Generalized' and 'Regular' are incompatible.

        (`errno` 7)
            The option 'Initial Residual' was selected but the starting
            vector held in `resid` is zero.

        (`errno` 8)
            Either the initialization function has not been called prior
            to the first call of this function or a communication array
            has become corrupted.

        (`errno` 10)
            On entry, 'Vectors' = Select, but this is not yet
            implemented.

        (`errno` 11)
            The number of eigenvalues found to sufficient accuracy is
            zero.

        (`errno` 12)
            Could not build an Arnoldi factorization. The size of the
            current Arnoldi factorization = *<value>*.

        (`errno` 13)
            Error in internal call to compute eigenvalues and
            corresponding error bounds of the current upper Hessenberg
            matrix. Please contact NAG.

        (`errno` 14)
            During calculation of a Schur form, there was a failure to
            compute a number of eigenvalues Please contact NAG.

        (`errno` 15)
            The computed Schur form could not be reordered by an
            internal call. Please contact NAG.

        (`errno` 16)
            Error in internal call to compute eigenvectors. Please
            contact NAG.

        (`errno` 17)
            Failure during internal factorization of real banded matrix.
            Please contact NAG.

        (`errno` 18)
            Failure during internal solution of real banded matrix.
            Please contact NAG.

        (`errno` 19)
            Failure during internal factorization of complex banded
            matrix. Please contact NAG.

        (`errno` 20)
            Failure during internal solution of complex banded matrix.
            Please contact NAG.

        (`errno` 21)
            The maximum number of iterations has been reached. The
            maximum number of iterations = *<value>*. The number of
            converged eigenvalues = *<value>*.

        (`errno` 22)
            No shifts could be applied during a cycle of the implicitly
            restarted Arnoldi iteration.

        (`errno` 23)
            Overflow occurred during transformation of Ritz values to
            those of the original problem.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, (and optionally the corresponding eigenvectors,
    x) of a standard eigenvalue problem Ax = lambda x, or of a
    generalized eigenvalue problem Ax = lambda Bx of order n, where n is
    large and the coefficient matrices A and B are banded, complex and
    non-Hermitian.

    Following a call to the initialization function
    ``complex_band_init``, ``complex_band_solve`` returns the converged
    approximations to eigenvalues and (optionally) the corresponding
    approximate eigenvectors and/or a unitary basis for the associated
    approximate invariant subspace.
    The eigenvalues (and eigenvectors) are selected from those of a
    standard or generalized eigenvalue problem defined by complex banded
    non-Hermitian matrices.
    There is negligible additional computational cost to obtain
    eigenvectors; a unitary basis is always computed, but there is an
    additional storage cost if both are requested.

    The banded matrices A and B must be stored using the LAPACK column
    ordered storage format for banded non-Hermitian matrices; please
    refer to [the F07 Introduction] for details on this storage format.

    ``complex_band_solve`` is based on the banded driver functions
    **znbdr1** to **znbdr4** from the ARPACK package, which uses the
    Implicitly Restarted Arnoldi iteration method.
    The method is described in Lehoucq and Sorensen (1996) and Lehoucq
    (2001) while its use within the ARPACK software is described in
    great detail in Lehoucq `et al.` (1998).
    An evaluation of software for computing eigenvalues of sparse
    non-Hermitian matrices is provided in Lehoucq and Scott (1996).
    This suite of functions offers the same functionality as the ARPACK
    banded driver software for complex non-Hermitian problems, but the
    interface design is quite different in order to make the option
    setting clearer and to combine the different drivers into a general
    purpose function.

    ``complex_band_solve``, is a general purpose function that must be
    called following initialization by ``complex_band_init``.
    ``complex_band_solve`` uses options, set either by default or
    explicitly by calling ``complex_option``, to return the converged
    approximations to selected eigenvalues and (optionally):

    -   the corresponding approximate eigenvectors;

    -   a unitary basis for the associated approximate invariant
        subspace;

    -   both.

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.complex_band_solve'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12auft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._EngineComplex128ArrayType.c_type, # Input, ab[ldab, *].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineComplex128ArrayType.c_type, # Input, mb[ldmb, *].
        utils._BYREF_INT, # Input, ldmb.
        utils._BYREF_COMPLEX128, # Input, sigma.
        utils._BYREF_INT, # Output, nconv.
        utils._EngineComplex128ArrayType.c_type, # Output, d[*].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, resid[*].
        utils._EngineComplex128ArrayType.c_type, # Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, comm[60].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[140].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (mb, 'mb'), (z, 'z'), (v, 'v')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12atf_n_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (20,), validate=False)
    else:
        _vl_f12atf_n_py = 0
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    _ab = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(2*_kl_py+_ku_py+1, _vl_f12atf_n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldmb_py = utils._EngineComplex128ArrayType.get_nd(mb, {'fun_name': fun_name, 'entity': 'mb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldmb = utils._EngineIntScalarType(
        _ldmb_py, {'fun_name': fun_name, 'entity': '_ldmb'},
    ).to_ctypes()
    _mb = utils._EngineComplex128ArrayType(
        mb, {'fun_name': fun_name, 'entity': 'mb'},
        exp_shape=(2*_kl_py+_ku_py+1, _vl_f12atf_n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _sigma = utils._EngineComplex128ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12atf_nev_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (21,), validate=False)
    else:
        _vl_f12atf_nev_py = 0
    _d_cls = utils._EngineComplex128ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_vl_f12atf_nev_py,),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _comm = comm['comm']
    _icomm = comm['icomm']
    if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+6,), validate=False) == 1:
        if z is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'z'},
                'must not be None when \'Vectors\' = \'RITZ\'.',
            )
    if z is None:
        if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+7)-1,), validate=False) == 1:
            _z_dim2 = _vl_f12atf_nev_py
        else:
            _z_dim2 = 1
        _z = utils._EngineComplex128ArrayType.empty_ctypes((_vl_f12atf_n_py)*(_z_dim2))
    else:
        if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+7)-1,), validate=False) == 1:
            _z_dim2 = _vl_f12atf_nev_py
        else:
            _z_dim2 = 1
        exp_shape_z = (_vl_f12atf_n_py, _z_dim2)
        _z_cls = utils._EngineComplex128ArrayType(
            z, {'fun_name': fun_name, 'entity': 'z'},
            exp_shape=exp_shape_z,
            mutual_sorder=_m_sorder,
            check_contents=False,
            allow_none=True,
        )
        _z = _z_cls.to_ctypes(can_reference_contents=False)
    _resid_cls = utils._EngineComplex128ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vl_f12atf_n_py,),
    )
    _resid = _resid_cls.to_ctypes()
    _ldv_py = utils._EngineComplex128ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12atf_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (22,), validate=False)
    else:
        _vl_f12atf_ncv_py = 0
    _v_cls = utils._EngineComplex128ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_vl_f12atf_n_py, _vl_f12atf_ncv_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _routine_name = utils._EngineRoutineName(*b'f12au ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _kl,
        _ku,
        _ab,
        _ldab,
        _mb,
        _ldmb,
        _sigma,
        _nconv,
        _d,
        _z,
        _ldz,
        _resid,
        _v,
        _ldv,
        _comm,
        _icomm,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: kl >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ku = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ku >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldab = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '2*kl+ku+1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ldab >= 2*kl+ku+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations <= 0, the option',
                    '\'Iteration Limit\' has been set to {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The options \'Generalized\' and \'Regular\' are incompatible.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Initial Residual\' was selected but the',
                    'starting vector held in resid is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called prior',
                    'to the first call of this function or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 10:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, \'Vectors\' = Select, but this is not yet implemented.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 11:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of eigenvalues found to sufficient accuracy is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 12:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Could not build an Arnoldi factorization. The size',
                    'of the current Arnoldi factorization = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 13:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error in internal call to compute eigenvalues and corresponding',
                    'error bounds of the current upper Hessenberg matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 14:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'During calculation of a Schur form, there was a failure',
                    'to compute a number of eigenvalues',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 15:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The computed Schur form could not be reordered by an internal call.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 16 and _ifmt == 16:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error in internal call to compute eigenvectors.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 17 and _ifmt == 17:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal factorization of real banded matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 18 and _ifmt == 18:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal solution of real banded matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 19 and _ifmt == 19:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal factorization of complex banded matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 20 and _ifmt == 20:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal solution of complex banded matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 21 and _ifmt == 21:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations has been reached.',
                    'The maximum number of iterations = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The number of converged eigenvalues = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 22 and _ifmt == 22:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No shifts could be applied during a cycle of the',
                    'implicitly restarted Arnoldi iteration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 23 and _ifmt == 23:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Overflow occurred during transformation of Ritz values',
                    'to those of the original problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        if z is not None:
            _z_cls.refresh_py(_z)
        _resid_cls.refresh_py(_resid)
        _v_cls.refresh_py(_v)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _nconv_py
    _err_class.manage()
    return return_data

def real_symm_init(n, nev, ncv):
    u"""
    Initialization routine for (``real_symm_iter``) computing selected
    eigenvalues and, optionally, eigenvectors of a real symmetric sparse
    (standard or generalized) eigenproblem.

    ``real_symm_init`` is a setup function in a suite of functions
    consisting of ``real_symm_init``, ``real_symm_iter``,
    ``real_symm_proc``, ``real_symm_option`` and ``real_symm_monit``.
    It is used to find some of the eigenvalues (and optionally the
    corresponding eigenvectors) of a standard or generalized eigenvalue
    problem defined by real symmetric matrices.

    The suite of functions is suitable for the solution of large sparse,
    standard or generalized, symmetric eigenproblems where only a few
    eigenvalues from a selected range of the spectrum are required.

    For full information please refer to the NAG Library document for
    f12fa

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12faf.html

    Parameters
    ----------
    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    nev : int
        The number of eigenvalues to be computed.

    ncv : int
        The number of Lanczos basis vectors to use during the
        computation.

        At present there is no `a priori` analysis to guide the
        selection of `ncv` relative to `nev`.

        However, it is recommended that `ncv` >= 2*`nev`+1.

        If many problems of the same type are to be solved, you should
        experiment with increasing `ncv` while keeping `nev` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the work and storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to CPU time is problem
        dependent and must be determined empirically.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nev` = *<value>*.

            Constraint: `nev` > 0.

        (`errno` 3)
            On entry, `ncv` = *<value>*, `nev` = *<value>* and `n` =
            *<value>*.

            Constraint: `ncv` > `nev` and `ncv` <= `n`.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, (and optionally the corresponding eigenvectors,
    x) of a standard eigenvalue problem Ax = lambda x, or of a
    generalized eigenvalue problem Ax = lambda Bx of order n, where n is
    large and the coefficient matrices A and B are sparse, real and
    symmetric.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale dense, real and symmetric problems.

    ``real_symm_init`` is a setup function which must be called before
    ``real_symm_iter``, the reverse communication iterative solver, and
    before ``real_symm_option``, the options setting function.
    ``real_symm_proc``, is a post-processing function that must be
    called following a successful final exit from ``real_symm_iter``,
    while ``real_symm_monit`` can be used to return additional
    monitoring information during the computation.

    This setup function initializes the communication arrays, sets (to
    their default values) all options that can be set by you via the
    option setting function ``real_symm_option``, and checks that the
    lengths of the communication arrays as passed by you are of
    sufficient length.
    For details of the options available and how to set them see [Other
    Parameters for real_symm_option].

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_symm_init'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12faft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nev.
        utils._BYREF_INT, # Input, ncv.
        utils._EngineIntArrayType.c_type, # Output, icomm[max(1, licomm)].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineFloat64ArrayType.c_type, # Output, comm[max(1, lcomm)].
        utils._BYREF_INT, # Input, lcomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nev = utils._EngineIntScalarType(
        nev, {'fun_name': fun_name, 'entity': 'nev'},
    ).to_ctypes()
    _ncv = utils._EngineIntScalarType(
        ncv, {'fun_name': fun_name, 'entity': 'ncv'},
    ).to_ctypes()
    _licomm = utils._EngineIntScalarType.c_type(140)
    _licomm_py = utils._EngineIntScalarType.to_py(_licomm)
    _icomm = utils._EngineIntArrayType.empty_ctypes(max(1, _licomm_py))
    _comm_py = {}
    _comm_py['icomm'] = _icomm
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ncv_py = utils._EngineIntScalarType.to_py(_ncv)
    _lcomm = utils._EngineIntScalarType(
        3*_n_py+_ncv_py*_ncv_py+8*_ncv_py+60, {'fun_name': fun_name, 'entity': '_lcomm'},
    ).to_ctypes()
    _lcomm_py = utils._EngineIntScalarType.to_py(_lcomm)
    _comm = utils._EngineFloat64ArrayType.empty_ctypes(max(1, _lcomm_py))
    _comm_py['comm'] = _comm
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _nev,
        _ncv,
        _icomm,
        _licomm,
        _comm,
        _lcomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nev > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncv = {:s}, nev = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: ncv > nev and ncv <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = _comm_py
    _err_class.manage()
    return return_data

def real_symm_iter(irevcm, resid, v, x, mx, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a real
    symmetric sparse eigenproblem, reverse communication.

    ``real_symm_iter`` is an iterative solver in a suite of functions
    consisting of ``real_symm_init``, ``real_symm_iter``,
    ``real_symm_proc``, ``real_symm_option`` and ``real_symm_monit``.
    It is used to find some of the eigenvalues (and optionally the
    corresponding eigenvectors) of a standard or generalized eigenvalue
    problem defined by real symmetric matrices.

    For full information please refer to the NAG Library document for
    f12fb

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12fbf.html

    Parameters
    ----------
    irevcm : int
        `On initial entry`: `irevcm` = 0, otherwise an error condition
        will be raised.

        `On intermediate entry`: must be unchanged from its previous
        exit value. Changing `irevcm` to any other value between calls
        will result in an error.

    resid : float, shape (n), modified in place
        `On initial entry`: need not be set unless the option 'Initial
        Residual' has been set in a prior call to ``real_symm_option``
        in which case `resid` should contain an initial residual vector,
        possibly from a previous run.

        `On intermediate entry`: must be unchanged from its previous
        exit. Changing `resid` to any other value between calls may
        result in an error exit.

        `On intermediate exit`: contains the current residual vector.

        `On final exit`: contains the final residual vector.

    v : float, shape (n, ncv), modified in place
        `On initial entry`: need not be set.

        `On intermediate entry`: must be unchanged from its previous
        exit.

        `On intermediate exit`: contains the current set of Arnoldi
        basis vectors.

        `On final exit`: contains the final set of Arnoldi basis
        vectors.

    x : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if int(`comm`['comm'][0])>=1 and
        int(`comm`['comm'][0])<=1000: if
        `comm`['icomm'][(int(`comm`['comm'][0])+24)-1]=0:
        `comm`['icomm'][(int(`comm`['comm'][0])+1)-1]; otherwise: 1;
        otherwise: 0.

        `On initial entry`: need not be set, it is used as a convenient
        mechanism for accessing elements of `comm`['comm'].

        `On intermediate entry`: if 'Pointers' = YES, `x` need not be
        set.

        If 'Pointers' = NO, `x` must contain the result of y = op(x)
        when `irevcm` returns the value -1 or +1.

        It must return the real parts of the computed shifts when
        `irevcm` returns the value 3.

        `On intermediate exit`: if 'Pointers' = YES, `x` is not
        referenced.

        If 'Pointers' = NO, `x` contains the vector x when `irevcm`
        returns the value -1 or +1.

        `On final exit`: does not contain useful data.

    mx : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if int(`comm`['comm'][0])>=1 and
        int(`comm`['comm'][0])<=1000: if
        `comm`['icomm'][(int(`comm`['comm'][0])+24)-1]=0:
        `comm`['icomm'][(int(`comm`['comm'][0])+1)-1]; otherwise: 1;
        otherwise: 0.

        `On initial entry`: need not be set, it is used as a convenient
        mechanism for accessing elements of `comm`['comm'].

        `On intermediate entry`: if 'Pointers' = YES, `mx` need not be
        set.

        If 'Pointers' = NO, `mx` must contain the result of y = Bx when
        `irevcm` returns the value 2.

        It must return the imaginary parts of the computed shifts when
        `irevcm` returns the value 3.

        `On intermediate exit`: if 'Pointers' = YES, `mx` is not
        referenced.

        If 'Pointers' = NO, `mx` contains the vector Bx when `irevcm`
        returns the value +1.

        `On final exit`: does not contain any useful data.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_symm_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    irevcm : int
        `On intermediate exit`: has the following meanings.

        `irevcm` = -1

            The calling program must compute the matrix-vector product y
            = op(x), where x is stored in `x` (by default) or in the
            array `comm`['comm'] (starting from the location given by
            the first element of `comm`['icomm']) when the option
            'Pointers' = 'YES' is set in a prior call to
            ``real_symm_option``. The result y is returned in `x` (by
            default) or in the array `comm`['comm'] (starting from the
            location given by the second element of `comm`['icomm'])
            when the option 'Pointers' = 'YES' is set.

        `irevcm` = 1

            The calling program must compute the matrix-vector product y
            = op(x). This is similar to the case `irevcm` = -1 except
            that the result of the matrix-vector product Bx (as required
            in some computational modes) has already been computed and
            is available in `mx` (by default) or in the array
            `comm`['comm'] (starting from the location given by the
            third element of `comm`['icomm']) when the option 'Pointers'
            = 'YES' is set.

        `irevcm` = 2

            The calling program must compute the matrix-vector product y
            = Bx, where x is stored in `x` and y is returned in `mx` (by
            default) or in the array `comm`['comm'] (starting from the
            location given by the second element of `comm`['icomm'])
            when the option 'Pointers' = 'YES' is set.

        `irevcm` = 3

            Compute the `nshift` real and imaginary parts of the shifts
            where the real parts are to be returned in the first
            `nshift` locations of the array `x` and the imaginary parts
            are to be returned in the first `nshift` locations of the
            array `mx`. Only complex conjugate pairs of shifts may be
            applied and the pairs must be placed in consecutive
            locations. This value of `irevcm` will only arise if the
            option 'Supplied Shifts' is set in a prior call to
            ``real_symm_option`` which is intended for experienced users
            only; the default and recommended option is to use exact
            shifts (see Lehoucq `et al.` (1998) for details and guidance
            on the choice of shift strategies).

        `irevcm` = 4

            Monitoring step: a call to ``real_symm_monit`` can now be
            made to return the number of Arnoldi iterations, the number
            of converged Ritz values, their real and imaginary parts,
            and the corresponding Ritz estimates.

        `On final exit`: `irevcm` = 5: ``real_symm_iter`` has completed
        its tasks. The value of errno determines whether the iteration
        has been successfully completed, or whether errors have been
        detected. On successful completion ``real_symm_proc`` must be
        called to return the requested eigenvalues and eigenvectors
        (and/or Schur vectors).

    nshift : int
        `On intermediate exit`: if the option 'Supplied Shifts' is set
        and `irevcm` returns a value of 3, `nshift` returns the number
        of complex shifts required.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The maximum number of iterations <= 0, the option 'Iteration
            Limit' has been set to *<value>*.

        (`errno` 2)
            The options 'Generalized' and 'Regular' are incompatible.

        (`errno` 3)
            Eigenvalues from both ends of the spectrum were requested,
            but the number of eigenvalues (see nev in
            ``real_symm_init``) requested is one.

        (`errno` 4)
            The option 'Initial Residual' was selected but the starting
            vector held in `resid` is zero.

        (`errno` 6)
            No shifts could be applied during a cycle of the implicitly
            restarted Lanczos iteration.

        (`errno` 7)
            Could not build a Lanczos factorization. The size of the
            current Lanczos factorization = *<value>*.

        (`errno` 8)
            Error in internal call to compute eigenvalues and
            corresponding error bounds of the current upper Hessenberg
            matrix. Please contact NAG.

        (`errno` 9)
            Either the function was called without an initial call to
            the setup function or the communication arrays have become
            corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 5)
            The maximum number of iterations has been reached. The
            maximum number of iterations = *<value>*. The number of
            converged eigenvalues = *<value>*. See the function document
            for further details.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, (and optionally the corresponding eigenvectors,
    x) of a standard eigenvalue problem Ax = lambda x, or of a
    generalized eigenvalue problem Ax = lambda Bx of order n, where n is
    large and the coefficient matrices A and B are sparse, real and
    symmetric.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale dense, real and symmetric problems.

    ``real_symm_iter`` is a **reverse communication** function, based on
    the ARPACK routine **dsaupd**, using the Implicitly Restarted
    Arnoldi iteration method, which for symmetric problems reduces to a
    variant of the Lanczos method.
    The method is described in Lehoucq and Sorensen (1996) and Lehoucq
    (2001) while its use within the ARPACK software is described in
    great detail in Lehoucq `et al.` (1998).
    An evaluation of software for computing eigenvalues of sparse
    symmetric matrices is provided in Lehoucq and Scott (1996).
    This suite of functions offers the same functionality as the ARPACK
    software for real symmetric problems, but the interface design is
    quite different in order to make the option setting clearer and to
    simplify the interface of ``real_symm_iter``.

    The setup function ``real_symm_init`` must be called before
    ``real_symm_iter``, the reverse communication iterative solver.
    Options may be set for ``real_symm_iter`` by prior calls to the
    option setting function ``real_symm_option`` and a post-processing
    function ``real_symm_proc`` must be called following a successful
    final exit from ``real_symm_iter``. ``real_symm_monit``, may be
    called following certain flagged, intermediate exits from
    ``real_symm_iter`` to provide additional monitoring information
    about the computation.

    ``real_symm_iter`` uses **reverse communication**, i.e., it returns
    repeatedly to the calling program with the argument `irevcm` (see
    [Parameters]) set to specified values which require the calling
    program to carry out one of the following tasks:

    -   compute the matrix-vector product y = op(x), where op is defined
        by the computational mode;

    -   compute the matrix-vector product y = Bx;

    -   notify the completion of the computation;

    -   allow the calling program to monitor the solution.

    The problem type to be solved (standard or generalized), the
    spectrum of eigenvalues of interest, the mode used (regular, regular
    inverse, shifted inverse, Buckling or Cayley) and other options can
    all be set using the option setting function ``real_symm_option``.

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_symm_iter'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12fbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, resid[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, mx[*].
        utils._BYREF_INT, # Output, nshift.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[*].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((v, 'v'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_n_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (20,), validate=False)
    else:
        _vl_f12faf_n_py = 0
    _resid_cls = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vl_f12faf_n_py,),
    )
    _resid = _resid_cls.to_ctypes()
    _ldv_py = utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (22,), validate=False)
    else:
        _vl_f12faf_ncv_py = 0
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_vl_f12faf_n_py, _vl_f12faf_ncv_py),
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12faf_lcomm_py = 0
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12faf_licomm_py = 0
    _comm = comm['comm']
    _icomm = comm['icomm']
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)) <= 1000:
        if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+24)-1,), validate=False) == 0:
            _x_dim1 = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+1)-1,), validate=False)
        else:
            _x_dim1 = 1
    else:
        _x_dim1 = 0
    exp_shape_x = (_x_dim1,)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=exp_shape_x,
    )
    _x = _x_cls.to_ctypes()
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)) <= 1000:
        if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+24)-1,), validate=False) == 0:
            _mx_dim1 = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+1)-1,), validate=False)
        else:
            _mx_dim1 = 1
    else:
        _mx_dim1 = 0
    exp_shape_mx = (_mx_dim1,)
    _mx_cls = utils._EngineFloat64ArrayType(
        mx, {'fun_name': fun_name, 'entity': 'mx'},
        exp_shape=exp_shape_mx,
    )
    _mx = _mx_cls.to_ctypes()
    _nshift = utils._EngineIntScalarType.empty_ctypes()
    _routine_name = utils._EngineRoutineName(*b'f12fb ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _irevcm,
        _resid,
        _v,
        _ldv,
        _x,
        _mx,
        _nshift,
        _comm,
        _icomm,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 5 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations <= 0, the option',
                    '\'Iteration Limit\' has been set to {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The options \'Generalized\' and \'Regular\' are incompatible.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Eigenvalues from both ends of the spectrum were requested,',
                    'but the number of eigenvalues (see nev in real_symm_init) requested is one.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Initial Residual\' was selected but the',
                    'starting vector held in resid is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations has been reached.',
                    'The maximum number of iterations = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The number of converged eigenvalues = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'See the function document for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No shifts could be applied during a cycle of the',
                    'implicitly restarted Lanczos iteration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Could not build a Lanczos factorization. The size',
                    'of the current Lanczos factorization = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error in internal call to compute eigenvalues and corresponding',
                    'error bounds of the current upper Hessenberg matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the function was called without an initial call to the setup',
                    'function or the communication arrays have become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _resid_cls.refresh_py(_resid)
        _v_cls.refresh_py(_v)
        _x_cls.refresh_py(_x)
        _mx_cls.refresh_py(_mx)
        _nshift_py = utils._EngineIntScalarType.to_py(_nshift)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['irevcm', 'nshift'],
        )(
            _irevcm_py,
            _nshift_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_symm_proc(d, z, sigma, resid, v, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a real
    symmetric sparse eigenproblem, postprocessing for
    ``real_symm_iter``.

    ``real_symm_proc`` is a post-processing function in a suite of
    functions which includes ``real_symm_init``, ``real_symm_iter``,
    ``real_symm_option`` and ``real_symm_monit``. ``real_symm_proc``
    must be called following a final exit from ``real_symm_iter``.

    For full information please refer to the NAG Library document for
    f12fc

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12fcf.html

    Parameters
    ----------
    d : float, shape (ncv), modified in place
        `On exit`: the first `nconv` locations of the array `d` contain
        the converged approximate eigenvalues.

    z : float, shape (n, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if int(`comm`['comm'][0])>=1 and
        int(`comm`['comm'][0])<=1000: if
        `comm`['icomm'][(int(`comm`['comm'][0])+7)-1]=1: ncv; otherwise:
        1; otherwise: 1.

        `On exit`: if the default option 'Vectors' = 'RITZ' (see
        ``real_symm_option``) has been selected then `z` contains the
        final set of eigenvectors corresponding to the eigenvalues held
        in `d`. The real eigenvector associated with an eigenvalue is
        stored in the corresponding of `z`.

    sigma : float
        If one of the 'Shifted Inverse' (see ``real_symm_option``) modes
        has been selected then `sigma` contains the real shift used;
        otherwise `sigma` is not referenced.

    resid : float, shape (n)
        Must not be modified following a call to ``real_symm_iter``
        since it contains data required by ``real_symm_proc``.

    v : float, shape (n, ncv), modified in place
        `On entry`: the ncv columns of `v` contain the Lanczos basis
        vectors for op as constructed by ``real_symm_iter``.

        `On exit`: if the option 'Vectors' = 'SCHUR' has been set, or
        the option 'Vectors' = 'RITZ' has been set and a separate array
        `z` has been passed (i.e., `z` does not equal `v`), then the
        first `nconv` columns of `v` will contain approximate Schur
        vectors that span the desired invariant subspace.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_symm_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged eigenvalues as found by
        ``real_symm_iter``.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, 'Vectors' = 'SELECT', but this is not yet
            implemented.

        (`errno` 3)
            The number of eigenvalues found to sufficient accuracy, as
            communicated through the argument `comm`['icomm'], is zero.

        (`errno` 4)
            Got a different count of the number of converged Ritz values
            than the value passed to it through the argument
            `comm`['icomm']: number counted = *<value>*, number expected
            = *<value>*.

        (`errno` 5)
            During calculation of a tridiagonal form, there was a
            failure to compute *<value>* eigenvalues in a total of
            *<value>* iterations.

        (`errno` 8)
            Either the function was called out of sequence (following an
            initial call to the setup function and following completion
            of calls to the reverse communication function) or the
            communication arrays have become corrupted.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, (and optionally the corresponding eigenvectors,
    x) of a standard eigenvalue problem Ax = lambda x, or of a
    generalized eigenvalue problem Ax = lambda Bx of order n, where n is
    large and the coefficient matrices A and B are sparse, real and
    symmetric.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale dense, real and symmetric problems.

    Following a call to ``real_symm_iter``, ``real_symm_proc`` returns
    the converged approximations to eigenvalues and (optionally) the
    corresponding approximate eigenvectors and/or an orthonormal basis
    for the associated approximate invariant subspace.
    The eigenvalues (and eigenvectors) are selected from those of a
    standard or generalized eigenvalue problem defined by real symmetric
    matrices.
    There is negligible additional cost to obtain eigenvectors; an
    orthonormal basis is always computed, but there is an additional
    storage cost if both are requested.

    ``real_symm_proc`` is based on the function **dseupd** from the
    ARPACK package, which uses the Implicitly Restarted Lanczos
    iteration method.
    The method is described in Lehoucq and Sorensen (1996) and Lehoucq
    (2001) while its use within the ARPACK software is described in
    great detail in Lehoucq `et al.` (1998).
    An evaluation of software for computing eigenvalues of sparse
    symmetric matrices is provided in Lehoucq and Scott (1996).
    This suite of functions offers the same functionality as the ARPACK
    software for real symmetric problems, but the interface design is
    quite different in order to make the option setting clearer and to
    simplify some of the interfaces.

    ``real_symm_proc``, is a post-processing function that must be
    called following a successful final exit from ``real_symm_iter``.
    ``real_symm_proc`` uses data returned from ``real_symm_iter`` and
    options, set either by default or explicitly by calling
    ``real_symm_option``, to return the converged approximations to
    selected eigenvalues and (optionally):

    -   the corresponding approximate eigenvectors;

    -   an orthonormal basis for the associated approximate invariant
        subspace;

    -   both.

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_symm_proc'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12fcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Output, nconv.
        utils._EngineFloat64ArrayType.c_type, # Output, d[*].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._EngineFloat64ArrayType.c_type, # Input, resid[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[*].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'), (v, 'v')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (22,), validate=False)
    else:
        _vl_f12faf_ncv_py = 0
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_vl_f12faf_ncv_py,),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12faf_lcomm_py = 0
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12faf_licomm_py = 0
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _comm = comm['comm']
    _icomm = comm['icomm']
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_n_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (20,), validate=False)
    else:
        _vl_f12faf_n_py = 0
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)) <= 1000:
        if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+7)-1,), validate=False) == 1:
            _z_dim2 = _vl_f12faf_ncv_py
        else:
            _z_dim2 = 1
    else:
        _z_dim2 = 1
    exp_shape_z = (_vl_f12faf_n_py, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _resid = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vl_f12faf_n_py,),
    ).to_ctypes()
    _ldv_py = utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_vl_f12faf_n_py, _vl_f12faf_ncv_py),
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    _routine_name = utils._EngineRoutineName(*b'f12fc ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _nconv,
        _d,
        _z,
        _ldz,
        _sigma,
        _resid,
        _v,
        _ldv,
        _comm,
        _icomm,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, \'Vectors\' = \'SELECT\', but this is not yet implemented.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of eigenvalues found to sufficient accuracy,',
                    'as communicated through the argument comm[\'icomm\'], is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Got a different count of the number of converged Ritz values',
                    'than the value passed to it through the argument comm[\'icomm\']:',
                    'number counted = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'number expected = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'During calculation of a tridiagonal form, there was a failure',
                    'to compute {:s} eigenvalues in a total of'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the function was called out of sequence (following an initial call',
                    'to the setup function and following completion of calls to the reverse',
                    'communication function) or the communication arrays have become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        _z_cls.refresh_py(_z)
        _v_cls.refresh_py(_v)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _nconv_py
    _err_class.manage()
    return return_data

def real_symm_option(optstr, comm, io_manager=None):
    u"""
    Set a single option from a string
    (``real_symm_iter``/``real_symm_proc``/``real_symm_band_solve``).

    Note: this function uses optional algorithmic parameters, see also:
    ``real_symm_iter``, ``real_symm_proc``, ``real_symm_init``.

    ``real_symm_option`` is an option setting function in a suite of
    functions consisting of ``real_symm_init``, ``real_symm_iter``,
    ``real_symm_proc``, ``real_symm_option`` and ``real_symm_monit``,
    and may be used to supply individual options to ``real_symm_iter``
    and ``real_symm_proc``.
    The initialization function ``real_symm_init`` **must** have been
    called prior to calling ``real_symm_option``.

    For full information please refer to the NAG Library document for
    f12fd

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12fdf.html

    Parameters
    ----------
    optstr : str
        A single valid option string (as described in [Notes] and [Other
        Parameters]).

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_symm_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Other Parameters
    ----------------
    'Advisory' : int
        Default = advisory message unit number

        The destination for advisory messages.

    'Defaults' : valueless
        This special keyword may be used to reset all options to their
        default values.

    'Exact Shifts' : valueless
        Default

        During the Lanczos iterative process, shifts are applied
        internally as part of the implicit restarting scheme.
        The shift strategy used by default and selected by the 'Exact
        Shifts' is strongly recommended over the alternative 'Supplied
        Shifts' (see Lehoucq `et al.` (1998) for details of shift
        strategies).

        If 'Exact Shifts' are used then these are computed internally by
        the algorithm in the implicit restarting scheme.

        If 'Supplied Shifts' are used then, during the Lanczos iterative
        process, you must supply shifts through array arguments of
        ``real_symm_iter`` when ``real_symm_iter`` returns with irevcm =
        3; the real and imaginary parts of the shifts are returned in x
        and mx respectively (or in comm when the option 'Pointers' =
        'YES' is set). This option should only be used if you are an
        experienced user since this requires some algorithmic knowledge
        and because more operations are usually required than for the
        implicit shift scheme.
        Details on the use of explicit shifts and further references on
        shift strategies are available in Lehoucq `et al.` (1998).

    'Supplied Shifts' : valueless
        During the Lanczos iterative process, shifts are applied
        internally as part of the implicit restarting scheme.
        The shift strategy used by default and selected by the 'Exact
        Shifts' is strongly recommended over the alternative 'Supplied
        Shifts' (see Lehoucq `et al.` (1998) for details of shift
        strategies).

        If 'Exact Shifts' are used then these are computed internally by
        the algorithm in the implicit restarting scheme.

        If 'Supplied Shifts' are used then, during the Lanczos iterative
        process, you must supply shifts through array arguments of
        ``real_symm_iter`` when ``real_symm_iter`` returns with irevcm =
        3; the real and imaginary parts of the shifts are returned in x
        and mx respectively (or in comm when the option 'Pointers' =
        'YES' is set). This option should only be used if you are an
        experienced user since this requires some algorithmic knowledge
        and because more operations are usually required than for the
        implicit shift scheme.
        Details on the use of explicit shifts and further references on
        shift strategies are available in Lehoucq `et al.` (1998).

    'Iteration Limit' : int
        Default = 300

        The limit on the number of Lanczos iterations that can be
        performed before ``real_symm_iter`` exits.
        If not all requested eigenvalues have converged to within
        'Tolerance' and the number of Lanczos iterations has reached
        this limit then ``real_symm_iter`` exits with an error;
        ``real_symm_proc`` can still be called subsequently to return
        the number of converged eigenvalues, the converged eigenvalues
        and, if requested, the corresponding eigenvectors.

    'Largest Magnitude' : valueless
        Default

        The Lanczos iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_symm_iter`` to compute the eigenvalues
        of largest magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Algebraic' part, 'Smallest Magnitude', or 'Smallest Algebraic'
        part; or eigenvalues which are from 'Both Ends' of the algebraic
        spectrum.

        Note that these options select the eigenvalue properties for
        eigenvalues of op (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Both Ends' : valueless
        The Lanczos iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_symm_iter`` to compute the eigenvalues
        of largest magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Algebraic' part, 'Smallest Magnitude', or 'Smallest Algebraic'
        part; or eigenvalues which are from 'Both Ends' of the algebraic
        spectrum.

        Note that these options select the eigenvalue properties for
        eigenvalues of op (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Largest Algebraic' : valueless
        The Lanczos iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_symm_iter`` to compute the eigenvalues
        of largest magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Algebraic' part, 'Smallest Magnitude', or 'Smallest Algebraic'
        part; or eigenvalues which are from 'Both Ends' of the algebraic
        spectrum.

        Note that these options select the eigenvalue properties for
        eigenvalues of op (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Algebraic' : valueless
        The Lanczos iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_symm_iter`` to compute the eigenvalues
        of largest magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Algebraic' part, 'Smallest Magnitude', or 'Smallest Algebraic'
        part; or eigenvalues which are from 'Both Ends' of the algebraic
        spectrum.

        Note that these options select the eigenvalue properties for
        eigenvalues of op (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Magnitude' : valueless
        The Lanczos iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_symm_iter`` to compute the eigenvalues
        of largest magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Algebraic' part, 'Smallest Magnitude', or 'Smallest Algebraic'
        part; or eigenvalues which are from 'Both Ends' of the algebraic
        spectrum.

        Note that these options select the eigenvalue properties for
        eigenvalues of op (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Nolist' : valueless
        Default

        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'List' : valueless
        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'Monitoring' : int
        Default = -1

        If i > 0, monitoring information is output to unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`)
        i during the solution of each problem; this may be the same as
        the 'Advisory' unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).
        The type of information produced is dependent on the value of
        'Print Level', see the description of the option 'Print Level'
        for details of the information produced.
        Please see the ``FileObjManager`` method
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`
        to associate a file with a given unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).

    'Pointers' : valueless
        Default = NO

        During the iterative process and reverse communication calls to
        ``real_symm_iter``, required data can be communicated to and
        from ``real_symm_iter`` in one of two ways.
        When 'Pointers' = 'NO' is selected (the default) then the array
        arguments x and mx are used to supply you with required data and
        used to return computed values back to ``real_symm_iter``.
        For example, when irevcm = 1, ``real_symm_iter`` returns the
        vector x in x and the matrix-vector product Bx in mx and expects
        the result or the linear operation op(x) to be returned in x.

        If 'Pointers' = 'YES' is selected then the data is passed
        through sections of the array argument comm.
        The section corresponding to x when 'Pointers' = 'NO' begins at
        a location given by the first element of icomm; similarly the
        section corresponding to mx begins at a location given by the
        second element of icomm.
        This option allows ``real_symm_iter`` to perform fewer copy
        operations on each intermediate exit and entry, but can also
        lead to less elegant code in the calling program.

    'Print Level' : int
        Default = 0

        This controls the amount of printing produced by
        ``real_symm_option`` as follows.

        [table omitted]

        Note that setting 'Print Level' >= 30 can result in very lengthy
        'Monitoring' output.

    'Random Residual' : valueless
        Default

        To begin the Lanczos iterative process, ``real_symm_iter``
        requires an initial residual vector.
        By default ``real_symm_iter`` provides its own random initial
        residual vector; this option can also be set using option
        'Random Residual'.
        Alternatively, you can supply an initial residual vector
        (perhaps from a previous computation) to ``real_symm_iter``
        through the array argument resid; this option can be set using
        option 'Initial Residual'.

    'Initial Residual' : valueless
        To begin the Lanczos iterative process, ``real_symm_iter``
        requires an initial residual vector.
        By default ``real_symm_iter`` provides its own random initial
        residual vector; this option can also be set using option
        'Random Residual'.
        Alternatively, you can supply an initial residual vector
        (perhaps from a previous computation) to ``real_symm_iter``
        through the array argument resid; this option can be set using
        option 'Initial Residual'.

    'Regular' : valueless
        Default

        These options define the computational mode which in turn
        defines the form of operation op(x) to be performed when
        ``real_symm_iter`` returns with irevcm = -1 or 1 and the
        matrix-vector product Bx when ``real_symm_iter`` returns with
        irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

    'Regular Inverse' : valueless
        These options define the computational mode which in turn
        defines the form of operation op(x) to be performed when
        ``real_symm_iter`` returns with irevcm = -1 or 1 and the
        matrix-vector product Bx when ``real_symm_iter`` returns with
        irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

    'Shifted Inverse' : valueless
        These options define the computational mode which in turn
        defines the form of operation op(x) to be performed when
        ``real_symm_iter`` returns with irevcm = -1 or 1 and the
        matrix-vector product Bx when ``real_symm_iter`` returns with
        irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

    'Buckling' : valueless
        These options define the computational mode which in turn
        defines the form of operation op(x) to be performed when
        ``real_symm_iter`` returns with irevcm = -1 or 1 and the
        matrix-vector product Bx when ``real_symm_iter`` returns with
        irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

    'Cayley' : valueless
        These options define the computational mode which in turn
        defines the form of operation op(x) to be performed when
        ``real_symm_iter`` returns with irevcm = -1 or 1 and the
        matrix-vector product Bx when ``real_symm_iter`` returns with
        irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator op(x).

        [table omitted]

    'Standard' : valueless
        Default

        The problem to be solved is either a standard eigenvalue
        problem, Ax = lambda x, or a generalized eigenvalue problem, Ax
        = lambda Bx.
        The option 'Standard' should be used when a standard eigenvalue
        problem is being solved and the option 'Generalized' should be
        used when a generalized eigenvalue problem is being solved.

    'Generalized' : valueless
        The problem to be solved is either a standard eigenvalue
        problem, Ax = lambda x, or a generalized eigenvalue problem, Ax
        = lambda Bx.
        The option 'Standard' should be used when a standard eigenvalue
        problem is being solved and the option 'Generalized' should be
        used when a generalized eigenvalue problem is being solved.

    'Tolerance' : float
        Default = epsilon

        An approximate eigenvalue has deemed to have converged when the
        corresponding Ritz estimate is within 'Tolerance' relative to
        the magnitude of the eigenvalue.

    'Vectors' : valueless
        Default = RITZ

        The function ``real_symm_proc`` can optionally compute the Schur
        vectors and/or the eigenvectors corresponding to the converged
        eigenvalues.
        To turn off computation of any vectors the option 'Vectors' =
        'NONE' should be set.
        To compute only the Schur vectors (at very little extra cost),
        the option 'Vectors' = 'SCHUR' should be set and these will be
        returned in the array argument v of ``real_symm_proc``.
        To compute the eigenvectors (Ritz vectors) ­corresponding to the
        eigenvalue estimates, the option 'Vectors' = 'RITZ' should be
        set and these will be returned in the array argument z of
        ``real_symm_proc``, if z is set equal to v then the Schur
        vectors in v are overwritten by the eigenvectors computed by
        ``real_symm_proc``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Ambiguous keyword: *<value>*

        (`errno` 2)
            Keyword not recognized: *<value>*

        (`errno` 3)
            Second keyword not recognized: *<value>*

    Notes
    -----
    ``real_symm_option`` may be used to supply values for options to
    ``real_symm_iter`` and ``real_symm_proc``.
    It is only necessary to call ``real_symm_option`` for those
    arguments whose values are to be different from their default
    values.
    One call to ``real_symm_option`` sets one argument value.

    Each option is defined by a single character string consisting of
    one or more items.
    The items associated with a given option must be separated by
    spaces, or equals signs [ = ].
    Alphabetic characters may be upper or lower case.
    The string

    ::

      Iteration Limit = 500

    is an example of a string used to set an option.
    For each option the string contains one or more of the following
    items:

    -   a mandatory keyword;

    -   a phrase that qualifies the keyword;

    -   a number that specifies an `int` or `float` value. Such numbers
        may be up to 16 contiguous characters in Fortran's I, F, E or D
        format.

    ``real_symm_option`` does not have an equivalent function from the
    ARPACK package which passes options by directly setting values to
    scalar arguments or to specific elements of array arguments.
    ``real_symm_option`` is intended to make the passing of options more
    transparent and follows the same principle as the single option
    setting functions in submodule ``opt``.

    The setup function ``real_symm_init`` must be called prior to the
    first call to ``real_symm_option`` and all calls to
    ``real_symm_option`` must precede the first call to
    ``real_symm_iter``, the reverse communication iterative solver.

    A complete list of options, their abbreviations, synonyms and
    default values is given in [Other Parameters].

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_symm_option'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12fdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_STRING, # Input, optstr(len=*).
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[60].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of optstr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _optstr = utils._EngineCharScalarType(
        optstr, {'fun_name': fun_name, 'entity': 'optstr'}, exp_chrlen=None,
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm', 'comm'),
    )
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12faf_licomm_py = 0
    _icomm = comm['icomm']
    _comm = comm['comm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _optstr,
        _icomm,
        _comm,
        _errbuf,
        _ierr,
        _optstr._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Ambiguous keyword: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Keyword not recognized: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Second keyword not recognized: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = None
    _err_class.manage()
    return return_data

def real_symm_monit(ritz, rzest, comm):
    u"""
    Provides monitoring information for ``real_symm_iter``.

    ``real_symm_monit`` can be used to return additional monitoring
    information during computation.
    It is in a suite of functions which includes ``real_symm_init``,
    ``real_symm_iter``, ``real_symm_proc`` and ``real_symm_option``.

    For full information please refer to the NAG Library document for
    f12fe

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12fef.html

    Parameters
    ----------
    ritz : float, shape (ncv), modified in place
        `On exit`: the first `nconv` locations of the array `ritz`
        contain the real converged approximate eigenvalues.

    rzest : float, shape (ncv), modified in place
        `On exit`: the first `nconv` locations of the array `rzest`
        contain the Ritz estimates (error bounds) on the real `nconv`
        converged approximate eigenvalues.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_symm_init``.

    Returns
    -------
    niter : int
        The number of the current Arnoldi iteration.

    nconv : int
        The number of converged eigenvalues so far.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, (and optionally the corresponding eigenvectors,
    x) of a standard eigenvalue problem Ax = lambda x, or of a
    generalized eigenvalue problem Ax = lambda Bx of order n, where n is
    large and the coefficient matrices A and B are sparse, real and
    symmetric.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale dense, real and symmetric problems.

    On an intermediate exit from ``real_symm_iter`` with irevcm = 4,
    ``real_symm_monit`` may be called to return monitoring information
    on the progress of the Arnoldi iterative process.
    The information returned by ``real_symm_monit`` is:

    -   the number of the current Arnoldi iteration;

    -   the number of converged eigenvalues at this point;

    -   the real and imaginary parts of the converged eigenvalues;

    -   the error bounds on the converged eigenvalues.

    ``real_symm_monit`` does not have an equivalent function from the
    ARPACK package which prints various levels of detail of monitoring
    information through an output channel controlled via an argument
    value (see Lehoucq `et al.` (1998) for details of ARPACK routines).
    ``real_symm_monit`` should not be called at any time other than
    immediately following an irevcm = 4 return from ``real_symm_iter``.

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_symm_monit'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12feft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Output, niter.
        utils._BYREF_INT, # Output, nconv.
        utils._EngineFloat64ArrayType.c_type, # Output, ritz[*].
        utils._EngineFloat64ArrayType.c_type, # Output, rzest[*].
        utils._EngineIntArrayType.c_type, # Input, icomm[*].
        utils._EngineFloat64ArrayType.c_type, # Input, comm[*].
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _niter = utils._EngineIntScalarType.empty_ctypes()
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm', 'comm'),
    )
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (22,), validate=False)
    else:
        _vl_f12faf_ncv_py = 0
    _ritz_cls = utils._EngineFloat64ArrayType(
        ritz, {'fun_name': fun_name, 'entity': 'ritz'},
        exp_shape=(_vl_f12faf_ncv_py,),
        check_contents=False,
    )
    _ritz = _ritz_cls.to_ctypes(can_reference_contents=False)
    _rzest_cls = utils._EngineFloat64ArrayType(
        rzest, {'fun_name': fun_name, 'entity': 'rzest'},
        exp_shape=(_vl_f12faf_ncv_py,),
        check_contents=False,
    )
    _rzest = _rzest_cls.to_ctypes(can_reference_contents=False)
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12faf_licomm_py = 0
    _icomm = comm['icomm']
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12faf_lcomm_py = 0
    _comm = comm['comm']
    engine_ctypes(
        _en_data,
        _niter,
        _nconv,
        _ritz,
        _rzest,
        _icomm,
        _comm,
    )
    utils._handle_cb_exception(excs)
    _ritz_cls.refresh_py(_ritz)
    _rzest_cls.refresh_py(_rzest)
    _niter_py = utils._EngineIntScalarType.to_py(_niter)
    _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
    return_data = _collections.namedtuple(
        'nag_return_data',
        ['niter', 'nconv'],
    )(
        _niter_py,
        _nconv_py,
    )
    return return_data

def real_symm_band_init(n, nev, ncv):
    u"""
    Initialization routine for (``real_symm_band_solve``) computing
    selected eigenvalues and, optionally, eigenvectors of a real
    symmetric banded (standard or generalized) eigenproblem.

    ``real_symm_band_init`` is a setup function for
    ``real_symm_band_solve`` which can be used to find some eigenvalues
    (and optionally the corresponding eigenvectors) of a standard or
    generalized eigenvalue problem defined by real, banded, symmetric
    matrices.
    The banded matrix must be stored using the LAPACK storage format for
    real banded nonsymmetric matrices.

    For full information please refer to the NAG Library document for
    f12ff

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12fff.html

    Parameters
    ----------
    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    nev : int
        The number of eigenvalues to be computed.

    ncv : int
        The number of Lanczos basis vectors to use during the
        computation.

        At present there is no `a priori` analysis to guide the
        selection of `ncv` relative to `nev`.

        However, it is recommended that `ncv` >= 2*`nev`+1.

        If many problems of the same type are to be solved, you should
        experiment with increasing `ncv` while keeping `nev` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the work and storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to CPU time is problem
        dependent and must be determined empirically.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nev` = *<value>*.

            Constraint: `nev` > 0.

        (`errno` 3)
            On entry, `ncv` = *<value>*, `nev` = *<value>* and `n` =
            *<value>*.

            Constraint: `ncv` > `nev`+1 and `ncv` <= `n`.

    Notes
    -----
    The pair of functions ``real_symm_band_init`` and
    ``real_symm_band_solve`` together with the option setting function
    ``real_symm_option`` are designed to calculate some of the
    eigenvalues, lambda, (and optionally the corresponding eigenvectors,
    x) of a standard eigenvalue problem Ax = lambda x, or of a
    generalized eigenvalue problem Ax = lambda Bx of order n, where n is
    large and the coefficient matrices A and B are banded real and
    symmetric.

    ``real_symm_band_init`` is a setup function which must be called
    before the option setting function ``real_symm_option`` and the
    solver function ``real_symm_band_solve``.
    Internally, ``real_symm_band_solve`` makes calls to
    ``real_symm_iter`` and ``real_symm_proc``; the function documents
    for ``real_symm_iter`` and ``real_symm_proc`` should be consulted
    for details of the algorithm used.

    This setup function initializes the communication arrays, sets (to
    their default values) all options that can be set by you via the
    option setting function ``real_symm_option``, and checks that the
    lengths of the communication arrays as passed by you are of
    sufficient length.
    For details of the options available and how to set them, see [Other
    Parameters for real_symm_option].

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_symm_band_init'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12ffft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nev.
        utils._BYREF_INT, # Input, ncv.
        utils._EngineIntArrayType.c_type, # Output, icomm[max(1, licomm)].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineFloat64ArrayType.c_type, # Output, comm[max(1, lcomm)].
        utils._BYREF_INT, # Input, lcomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nev = utils._EngineIntScalarType(
        nev, {'fun_name': fun_name, 'entity': 'nev'},
    ).to_ctypes()
    _ncv = utils._EngineIntScalarType(
        ncv, {'fun_name': fun_name, 'entity': 'ncv'},
    ).to_ctypes()
    _licomm = utils._EngineIntScalarType.c_type(140)
    _licomm_py = utils._EngineIntScalarType.to_py(_licomm)
    _icomm = utils._EngineIntArrayType.empty_ctypes(max(1, _licomm_py))
    _comm_py = {}
    _comm_py['icomm'] = _icomm
    _lcomm = utils._EngineIntScalarType.c_type(60)
    _lcomm_py = utils._EngineIntScalarType.to_py(_lcomm)
    _comm = utils._EngineFloat64ArrayType.empty_ctypes(max(1, _lcomm_py))
    _comm_py['comm'] = _comm
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _nev,
        _ncv,
        _icomm,
        _licomm,
        _comm,
        _lcomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nev > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncv = {:s}, nev = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: ncv > nev+1 and ncv <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = _comm_py
    _err_class.manage()
    return return_data

def real_symm_band_solve(kl, ku, ab, mb, sigma, d, z, resid, v, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a real
    symmetric banded eigenproblem, driver.

    ``real_symm_band_solve`` is the main solver function in a suite of
    functions which includes ``real_symm_option`` and
    ``real_symm_band_init``. ``real_symm_band_solve`` must be called
    following an initial call to ``real_symm_band_init`` and following
    any calls to ``real_symm_option``.

    ``real_symm_band_solve`` returns approximations to selected
    eigenvalues, and (optionally) the corresponding eigenvectors, of a
    standard or generalized eigenvalue problem defined by real banded
    symmetric matrices.
    The banded matrix must be stored using the LAPACK storage format for
    real banded nonsymmetric matrices.

    For full information please refer to the NAG Library document for
    f12fg

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12fgf.html

    Parameters
    ----------
    kl : int
        The number of subdiagonals of the matrices A and B.

    ku : int
        The number of superdiagonals of the matrices A and B. Since A
        and B are symmetric, the normal case is `ku` = `kl`.

    ab : float, shape (2*`kl`+`ku`+1, n)
        Must contain the matrix A in LAPACK banded storage format for
        nonsymmetric matrices (see [the F07 Introduction]).

    mb : float, shape (:, n)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `comm`['comm'][0]>=1.0 and
        `comm`['comm'][0]<=1000.0: if `comm`['icomm'][28]=2 or
        (`comm`['icomm'][23]=1 and `comm`['icomm'][28] in (3, 4, 5)):
        2*`kl`+`ku`+1; otherwise: 0; otherwise: 0.

        Must contain the matrix B in LAPACK banded storage format for
        nonsymmetric matrices (see [the F07 Introduction]).

    sigma : float
        If one of the 'Shifted Inverse' (see ``real_symm_option``) modes
        has been selected then `sigma` contains the real shift used;
        otherwise `sigma` is not referenced.

    d : float, shape (ncv), modified in place
        `On exit`: the first `nconv` locations of the array `d` contain
        the converged approximate eigenvalues.

    z : float, shape (n, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if int(`comm`['comm'][0])>=1 and
        int(`comm`['comm'][0])<=1000: if
        `comm`['icomm'][(int(`comm`['comm'][0])+7)-1]=1: nev+1;
        otherwise: 1; otherwise: 1.

        `On exit`: if the default option 'Vectors' = 'RITZ' (see
        ``real_symm_option``) has been selected then `z` contains the
        final set of eigenvectors corresponding to the eigenvalues held
        in `d`. The real eigenvector associated with eigenvalue i-1, for
        i = 1,2,...,`nconv`, is stored in the ith column of `z`.

    resid : float, shape (n), modified in place
        `On entry`: need not be set unless the option 'Initial Residual'
        has been set in a prior call to ``real_symm_option`` in which
        case `resid` must contain an initial residual vector.

        `On exit`: contains the final residual vector.

    v : float, shape (n, ncv), modified in place
        `On exit`: if the option 'Vectors' = 'SCHUR' or 'RITZ' (see
        ``real_symm_option``) and a separate array `z` has been passed
        then the first `nconv`*n elements of `v` will contain
        approximate Schur vectors that span the desired invariant
        subspace.

        The jth Schur vector is stored in the ith column of `v`.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_symm_option``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged eigenvalues.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `kl` = *<value>*.

            Constraint: `kl` >= 0.

        (`errno` 2)
            On entry, `ku` = *<value>*.

            Constraint: `ku` >= 0.

        (`errno` 3)
            On entry, ldab = *<value>*, 2*`kl`+`ku`+1 = *<value>*.

            Constraint: ldab >= 2*`kl`+`ku`+1.

        (`errno` 4)
            The maximum number of iterations <= 0, the option 'Iteration
            Limit' has been set to *<value>*.

        (`errno` 5)
            The options 'Generalized' and 'Regular' are incompatible.

        (`errno` 6)
            Eigenvalues from both ends of the spectrum were requested,
            but the number of eigenvalues (nev in
            ``real_symm_band_init``) requested is one.

        (`errno` 7)
            The option 'Initial Residual' was selected but the starting
            vector held in `resid` is zero.

        (`errno` 9)
            On entry, 'Vectors' = Select, but this is not yet
            implemented.

        (`errno` 10)
            The number of eigenvalues found to sufficient accuracy is
            zero.

        (`errno` 11)
            Could not build a Lanczos factorization. The size of the
            current Lanczos factorization = *<value>*.

        (`errno` 12)
            Error in internal call to compute eigenvalues and
            corresponding error bounds of the current upper Hessenberg
            matrix. Please contact NAG.

        (`errno` 13)
            During calculation of a tridiagonal form, there was a
            failure to compute *<value>* eigenvalues in a total of
            *<value>* iterations.

        (`errno` 14)
            Failure during internal factorization of banded matrix.
            Please contact NAG.

        (`errno` 15)
            Failure during internal solution of banded system. Please
            contact NAG.

        (`errno` 17)
            No shifts could be applied during a cycle of the implicitly
            restarted Lanczos iteration.

        (`errno` 18)
            Either an initial call to the setup function has not been
            made or the communication arrays have become corrupted.

        (`errno` 19)
            On entry, ldmb = *<value>*, 2*`kl`+`ku`+1 = *<value>*.

            Constraint: ldmb >= 2*`kl`+`ku`+1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 16)
            The maximum number of iterations has been reached: there
            have been *<value>* iterations. There are *<value>*
            converged eigenvalues.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, (and optionally the corresponding eigenvectors,
    x) of a standard eigenvalue problem Ax = lambda x, or of a
    generalized eigenvalue problem Ax = lambda Bx of order n, where n is
    large and the coefficient matrices A and B are banded, real and
    symmetric.

    Following a call to the initialization function
    ``real_symm_band_init``, ``real_symm_band_solve`` returns the
    converged approximations to eigenvalues and (optionally) the
    corresponding approximate eigenvectors and/or an orthonormal basis
    for the associated approximate invariant subspace.
    The eigenvalues (and eigenvectors) are selected from those of a
    standard or generalized eigenvalue problem defined by real banded
    symmetric matrices.
    There is negligible additional computational cost to obtain
    eigenvectors; an orthonormal basis is always computed, but there is
    an additional storage cost if both are requested.

    The banded matrices A and B must be stored using the LAPACK storage
    format for banded nonsymmetric matrices; please refer to [the F07
    Introduction] for details on this storage format.

    ``real_symm_band_solve`` is based on the banded driver functions
    **dsbdr1** to **dsbdr6** from the ARPACK package, which uses the
    Implicitly Restarted Lanczos iteration method.
    The method is described in Lehoucq and Sorensen (1996) and Lehoucq
    (2001) while its use within the ARPACK software is described in
    great detail in Lehoucq `et al.` (1998).
    This suite of functions offers the same functionality as the ARPACK
    banded driver software for real symmetric problems, but the
    interface design is quite different in order to make the option
    setting clearer and to combine the different drivers into a general
    purpose function.

    ``real_symm_band_solve``, is a general purpose direct communication
    function that must be called following initialization by
    ``real_symm_band_init``. ``real_symm_band_solve`` uses options, set
    either by default or explicitly by calling ``real_symm_option``, to
    return the converged approximations to selected eigenvalues and
    (optionally):

    -   the corresponding approximate eigenvectors;

    -   an orthonormal basis for the associated approximate invariant
        subspace;

    -   both.

    References
    ----------
    Lehoucq, R B, 2001, `Implicitly restarted Arnoldi methods and
    subspace iteration`, SIAM Journal on Matrix Analysis and
    Applications (23), 551--562

    Lehoucq, R B and Scott, J A, 1996, `An evaluation of software for
    computing eigenvalues of sparse nonsymmetric matrices`, Preprint
    MCS-P547-1195, Argonne National Laboratory

    Lehoucq, R B and Sorensen, D C, 1996, `Deflation techniques for an
    implicitly restarted Arnoldi iteration`, SIAM Journal on Matrix
    Analysis and Applications (17), 789--821

    Lehoucq, R B, Sorensen, D C and Yang, C, 1998, `ARPACK Users' Guide:
    Solution of Large-scale Eigenvalue Problems with Implicitly
    Restarted Arnoldi Methods`, SIAM, Philadelphia
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.real_symm_band_solve'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12fgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._EngineFloat64ArrayType.c_type, # Input, ab[ldab, *].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Input, mb[ldmb, *].
        utils._BYREF_INT, # Input, ldmb.
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._BYREF_INT, # Output, nconv.
        utils._EngineFloat64ArrayType.c_type, # Output, d[*].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, resid[*].
        utils._EngineFloat64ArrayType.c_type, # Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[*].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (mb, 'mb'), (z, 'z'), (v, 'v')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12fff_n_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (20,), validate=False)
    else:
        _vl_f12fff_n_py = 0
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    _ab = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(2*_kl_py+_ku_py+1, _vl_f12fff_n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldmb_py = utils._EngineFloat64ArrayType.get_nd(mb, {'fun_name': fun_name, 'entity': 'mb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldmb = utils._EngineIntScalarType(
        _ldmb_py, {'fun_name': fun_name, 'entity': '_ldmb'},
    ).to_ctypes()
    if utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False) >= 1.0 and utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False) <= 1000.0:
        if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (28,), validate=False) == 2 or (utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (23,), validate=False) == 1 and utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (28,), validate=False) in (3, 4, 5)):
            _mb_dim1 = 2*_kl_py+_ku_py+1
        else:
            _mb_dim1 = 0
    else:
        _mb_dim1 = 0
    exp_shape_mb = (_mb_dim1, _vl_f12fff_n_py)
    _mb = utils._EngineFloat64ArrayType(
        mb, {'fun_name': fun_name, 'entity': 'mb'},
        exp_shape=exp_shape_mb,
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12fff_ncv_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (22,), validate=False)
    else:
        _vl_f12fff_ncv_py = 0
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_vl_f12fff_ncv_py,),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12fff_lcomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+29,), validate=False)
    else:
        _vl_f12fff_lcomm_py = 0
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12fff_licomm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+28,), validate=False)
    else:
        _vl_f12fff_licomm_py = 0
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _comm = comm['comm']
    _icomm = comm['icomm']
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12fff_nev_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (21,), validate=False)
    else:
        _vl_f12fff_nev_py = 0
    if int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)) >= 1 and int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)) <= 1000:
        if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+7)-1,), validate=False) == 1:
            _z_dim2 = _vl_f12fff_nev_py+1
        else:
            _z_dim2 = 1
    else:
        _z_dim2 = 1
    exp_shape_z = (_vl_f12fff_n_py, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _resid_cls = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vl_f12fff_n_py,),
    )
    _resid = _resid_cls.to_ctypes()
    _ldv_py = utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_vl_f12fff_n_py, _vl_f12fff_ncv_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _routine_name = utils._EngineRoutineName(*b'f12fg ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _kl,
        _ku,
        _ab,
        _ldab,
        _mb,
        _ldmb,
        _sigma,
        _nconv,
        _d,
        _z,
        _ldz,
        _resid,
        _v,
        _ldv,
        _comm,
        _icomm,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: kl >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ku = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ku >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldab = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '2*kl+ku+1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ldab >= 2*kl+ku+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations <= 0, the option',
                    '\'Iteration Limit\' has been set to {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The options \'Generalized\' and \'Regular\' are incompatible.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Eigenvalues from both ends of the spectrum were requested,',
                    'but the number of eigenvalues (nev in real_symm_band_init) requested is one.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Initial Residual\' was selected but the',
                    'starting vector held in resid is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, \'Vectors\' = Select, but this is not yet implemented.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 10:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of eigenvalues found to sufficient accuracy is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 11:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Could not build a Lanczos factorization. The size',
                    'of the current Lanczos factorization = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 12:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error in internal call to compute eigenvalues and corresponding',
                    'error bounds of the current upper Hessenberg matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 13:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'During calculation of a tridiagonal form, there was a failure',
                    'to compute {:s} eigenvalues in a total of'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 14:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal factorization of banded matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 15:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal solution of banded system.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 16 and _ifmt == 16:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations has been reached:',
                    'there have been {:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'There are {:s} converged eigenvalues.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 17 and _ifmt == 17:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No shifts could be applied during a cycle of the',
                    'implicitly restarted Lanczos iteration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 18 and _ifmt == 18:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either an initial call to the setup function has not been made',
                    'or the communication arrays have become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 19 and _ifmt == 19:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldmb = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '2*kl+ku+1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ldmb >= 2*kl+ku+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _d_cls.refresh_py(_d)
        _z_cls.refresh_py(_z)
        _resid_cls.refresh_py(_resid)
        _v_cls.refresh_py(_v)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _nconv_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['nconv'],
            )(
                _nconv_py,
            )
    _err_class.manage()
    return return_data

def feast_init():
    u"""
    Initialization function for (``feast_real_symm_solve``,
    ``feast_real_gen_solve``, ``feast_complex_herm_solve``,
    ``feast_complex_symm_solve``, ``feast_complex_gen_solve``,
    ``feast_poly_symm_solve`` and ``feast_poly_gen_solve``) computing
    eigenvalues within a selected region of the complex plane, and
    eigenvectors, of a standard, generalized or polynomial eigenproblem.

    ``feast_init`` initializes a data structure for the NAG FEAST suite
    of functions consisting of ``feast_init``, ``feast_option``,
    ``feast_symm_contour``, ``feast_gen_contour``,
    ``feast_custom_contour``, ``feast_real_symm_solve``,
    ``feast_real_gen_solve``, ``feast_complex_herm_solve``,
    ``feast_complex_symm_solve``, ``feast_complex_gen_solve``,
    ``feast_poly_symm_solve``, ``feast_poly_gen_solve`` and
    ``feast_free``.
    It is used to find some of the eigenvalues, and the corresponding
    eigenvectors, of a standard, generalized or polynomial eigenvalue
    problem.

    The suite of functions is suitable for the solution of large, sparse
    eigenproblems where only those eigenvalues within a selected region
    of the complex plane are required.

    For full information please refer to the NAG Library document for
    f12ja

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12jaf.html

    Returns
    -------
    handle : Handle
        Holds a handle to the internal data structure used by the NAG
        FEAST suite.

    Notes
    -----
    The NAG FEAST suite of functions is designed to calculate some of
    the eigenvalues, lambda, and the corresponding eigenvectors, x, of a
    standard eigenvalue problem Ax = lambda x, a generalized eigenvalue
    problem Ax = lambda Bx, where A and B are large and sparse, or a
    polynomial eigenvalue problem ∑_ilambda^iA_i x = 0.
    It can also be used to find eigenvalues/eigenvectors of smaller
    scale dense problems.
    The NAG FEAST suite is based on the FEAST library, using contour
    integration to find the eigenvalues within a contour in the complex
    plane.

    ``feast_init`` is a setup function which must be called before the
    option setting function ``feast_option``, before the contour
    definition functions ``feast_symm_contour``, ``feast_gen_contour``
    or ``feast_custom_contour``, and before the reverse communication
    solvers ``feast_real_symm_solve``, ``feast_real_gen_solve``,
    ``feast_complex_herm_solve``, ``feast_complex_symm_solve``,
    ``feast_complex_gen_solve``, ``feast_poly_symm_solve`` or
    ``feast_poly_gen_solve``.

    This setup function initializes the handle to a data structure used
    internally by the NAG FEAST suite and sets (to their default values)
    all options that can be set by you via the option setting function
    ``feast_option``.
    For details of the options available and how to set them see [Other
    Parameters for feast_option].
    When the handle is no longer needed, ``feast_free`` must be called
    to destroy it and deallocate all the allocated memory and data
    within.

    References
    ----------
    Polizzi, E, 2009, `Density-Matrix-Based Algorithms for Solving
    Eigenvalue Problems`, Phys. Rev. B. (79), 115112

    See Also
    --------
    :meth:`naginterfaces.library.examples.sparseig.feast_poly_gen_solve_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.feast_init'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12jaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_HANDLE, # Output, handle.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _handle = utils._EngineCptrScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _handle,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _ = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _handle_py = utils._EngineCptrScalarType.to_py(_handle)
        return_data = _handle_py
    _err_class.manage()
    return return_data

def feast_option(handle, optstr, io_manager=None):
    u"""
    Set a single option from a string (``feast_real_symm_solve``,
    ``feast_real_gen_solve``, ``feast_complex_herm_solve``,
    ``feast_complex_symm_solve``, ``feast_complex_gen_solve``,
    ``feast_poly_symm_solve`` and ``feast_poly_gen_solve``).

    Note: this function uses optional algorithmic parameters, see also:
    ``feast_symm_contour``, ``feast_gen_contour``,
    ``feast_custom_contour``, ``feast_real_symm_solve``,
    ``feast_real_gen_solve``, ``feast_complex_herm_solve``,
    ``feast_complex_symm_solve``, ``feast_complex_gen_solve``,
    ``feast_poly_symm_solve``, ``feast_poly_gen_solve``, ``feast_init``.

    ``feast_option`` is part of NAG FEAST suite of functions and may be
    used to supply individual options to either the contour setting
    functions or the reverse communication solvers.

    For full information please refer to the NAG Library document for
    f12jb

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12jbf.html

    Parameters
    ----------
    handle : Handle
        The handle to the internal data structure used by the NAG FEAST
        suite. It needs to be initialized by ``feast_init``. **It must
        not be changed between calls to the NAG FEAST suite.**

    optstr : str
        A single valid option string (as described in [Notes] and [Other
        Parameters]).

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Other Parameters
    ----------------
    'Advisory' : int
        Default = advisory message unit number

        The destination for advisory messages.

    'Contour Points Hermitian' : int
        Default = 8

        For real symmetric (``feast_real_symm_solve``) or Hermitian
        (``feast_complex_herm_solve``) eigenproblems, this is the number
        of points used on the half-contour which defines the contour
        integral.
        If the option 'Integration Type' is set to 'Gauss' or 'Zol', the
        values permitted are 1--20, 24, 32, 40, 48, 56.
        If the option 'Integration Type' is set to 'Trap', then all
        values greater than or equal to 1 are permitted.

    'Contour Points Non-Hermitian' : int
        Default = 8

        For general real (``feast_real_gen_solve``), symmetric complex
        (``feast_complex_symm_solve``, ``feast_poly_symm_solve``) or
        general complex (``feast_complex_gen_solve``,
        ``feast_poly_gen_solve``) eigenproblems, this is the number of
        integration points used on the circular or elliptical contour
        within which eigenvalues are sought.
        If the option 'Integration Type' is set to 'Gauss', the values
        permitted are 2--40 (even values only), 48, 64, 80, 96, 112.
        If the option 'Integration Type' is set to 'Trap', then all
        values greater than or equal to 2 are permitted.
        Note this option is only relevant if the contour definition
        functions ``feast_symm_contour`` or ``feast_gen_contour`` are
        used.
        If a custom contour is required (using the contour setting
        routine ``feast_custom_contour``) then this option is not
        relevant.

    'Convergence Criteria' : str
        Default = 'Residual'

        These are the convergence criteria used to test for convergence
        of eigenpairs in the search region.

        'Convergence Criteria' = 'Trace'
            The relative error of the trace of the eigenproblem in the
            search subspace is tested against delta.

        'Convergence Criteria' = 'Residual'
            The relative maximum residual over all eigenpairs found is
            tested against delta.

        Note: the value of delta is controlled using the option
        'Tolerance'.

    'Defaults' : valueless
        This special keyword may be used to reset all options to their
        default values.

    'Ellipse Contour Ratio' : int
        Default = 100

        The ratio, as a percentage, of the length of the vertical axis
        to the length of the horizontal axis if an elliptical contour is
        to be used in the contour integral.
        This option should be set prior to a call to either of the
        contour definition functions ``feast_symm_contour`` or
        ``feast_gen_contour``.

        Note: For a Hermitian eigenvalue problem, if the option
        'Integration Type' is set to 'Zol' then a circular contour is
        used and this option is not referenced.

    'Ellipse Rotation Angle' : int
        Default = 0

        The rotation angle in degrees from the horizontal (between -180
        and 180) of the major axis if an elliptical contour is to be
        used in the contour integral.
        This option should be set prior to calls to the contour
        definition function ``feast_gen_contour`` and the reverse
        communication solvers ``feast_real_gen_solve``,
        ``feast_complex_symm_solve``, ``feast_complex_gen_solve``,
        ``feast_poly_symm_solve`` and ``feast_poly_gen_solve`` (which
        deal with problems that are neither real symmetric nor complex
        Hermitian).

    'Execution Mode' : str
        Default = 'Normal'

        The execution mode used by the reverse communication solvers.

        'Execution Mode' = 'Normal'
            Normal execution.

        'Execution Mode' = 'Subspace'
            Return the search subspace and an estimate of the
            eigenvalues after one contour integral.

        'Execution Mode' = 'Estimate'
            Return an estimate of the number of eigenvalues inside the
            search contour. Note that this option is not available for
            polynomial eigenvalue problems (``feast_poly_symm_solve``
            and ``feast_poly_gen_solve``).

    'Integration Type' : str
        Default = 'Gauss'

        The integration type.
        The allowed values are 'Gauss', 'Trap' (Trapezoidal) or 'Zol'
        (Zolotarev).
        Note that Zolotarev can only be used for real symmetric or
        complex Hermitian eigenproblems.
        This option should be set prior to a call to either of the
        contour definition functions ``feast_symm_contour`` or
        ``feast_gen_contour``.
        If a custom contour is required (using the contour setting
        routine ``feast_custom_contour``) then this option is not
        relevant.

    'Max Iter' : int
        Default = 20

        The maximum number of iterations used in the reverse
        communication solvers.
        If this is set to a negative value then no maximum will be used.

    'Nolist' : valueless
        Default

        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'List' : valueless
        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'Print Level' : int
        Default = 0

        This controls the amount of printing produced by the contour
        definition routines and the reverse communication solvers.

        'Print Level' = '0'
            No output.

        'Print Level' = '1'
            Print runtime comments to the advisory channel.

    'Tolerance' : float
        Default = x-3, where x is the value returned by
        ``machine.decimal_digits``

        The stopping criterion, delta, used to determine convergence of
        the eigensolver.
        If a value of k is supplied then delta = 10^(-k) is used.

    'Eigenvectors' : str
        Default = 'Two-sided'

        Determines whether both left and right eigenvectors are computed
        or whether just right eigenvectors are computed.

        Note: this option only applies to the solvers
        ``feast_real_gen_solve``, ``feast_complex_gen_solve`` and
        ``feast_poly_gen_solve``, which deal with non-symmetric
        eigenvalue problems. For the symmetric/Hermitian solvers
        (``feast_real_symm_solve``, ``feast_complex_herm_solve``,
        ``feast_complex_symm_solve`` and ``feast_poly_symm_solve``) the
        left eigenvectors can be obtained from the right eigenvectors by
        complex conjugation.

        'Eigenvectors' = 'Two-sided'
            Both left and right eigenvectors are computed.

        'Eigenvectors' = 'One-sided'
            Only right eigenvectors are computed.

    'Subspace' : str
        Default = 'No'

        This option determines whether or not you will supply an initial
        guess for the eigenvector search subspace.

        'Subspace' = 'Yes'
            The reverse communication solvers will use your initial
            guess for the search subspace and the eigenvalues therein.
            This option is useful if a previous run of the FEAST
            routines has yielded a partially converged answer to your
            eigenproblem.

        'Subspace' = 'No'
            Random initial vectors will be used for the search subspace.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Ambiguous keyword: *<value>*

        (`errno` 2)
            Keyword not recognized: *<value>*

        (`errno` 3)
            Second keyword not recognized: *<value>*

        (`errno` 4)
            `handle` has not been initialized properly or is corrupted.

        (`errno` 5)
            One of the contour setting functions ``feast_symm_contour``,
            ``feast_gen_contour`` or ``feast_custom_contour`` has
            already been called. All calls to ``feast_option`` must be
            made prior to calling the contour setting functions.

    Notes
    -----
    After the `handle` has been initialized (e.g., ``feast_init`` has
    been called), ``feast_option`` may be used to supply values for
    options to ``feast_symm_contour``, ``feast_gen_contour``,
    ``feast_custom_contour``, ``feast_real_symm_solve``,
    ``feast_real_gen_solve``, ``feast_complex_herm_solve``,
    ``feast_complex_symm_solve``, ``feast_complex_gen_solve``,
    ``feast_poly_symm_solve`` and ``feast_poly_gen_solve``.
    It is only necessary to call ``feast_option`` for those arguments
    whose values are to be different from their default values.
    One call to ``feast_option`` sets one argument value.

    Each option is defined by a single character string consisting of
    one or more items.
    The items associated with a given option must be separated by spaces
    or equals signs [ = ].
    Alphabetic characters may be upper or lower case.
    The string

    ::

      'Integration type = Gauss'

    is an example of a string used to set an option.
    For each option the string contains one or more of the following
    items:

    -   a mandatory keyword;

    -   a phrase that qualifies the keyword;

    -   a number that specifies an `int`.

    The NAG FEAST suite of functions of which ``feast_option`` is a part
    is based on the FEAST library, which solves eigenproblems using
    contour integration to find eigenvalues with a given contour in the
    complex plane.
    However, ``feast_option`` itself does not have an equivalent
    function in the FEAST package, which passes options by directly
    setting values to scalar arguments or to specific elements of array
    arguments. ``feast_option`` is intended to make the passing of
    options more transparent and follows the same principle as the
    single option setting functions in submodule ``opt``.

    The initialization function ``feast_init`` must be called prior to
    the first call to ``feast_option`` and all calls to ``feast_option``
    must precede the first call to the contour setting functions
    ``feast_symm_contour``, ``feast_gen_contour`` and
    ``feast_custom_contour``, and the reverse communication solvers
    ``feast_real_symm_solve``, ``feast_real_gen_solve``,
    ``feast_complex_herm_solve``, ``feast_complex_symm_solve``,
    ``feast_complex_gen_solve``, ``feast_poly_symm_solve`` and
    ``feast_poly_gen_solve``, and the deallocation function
    ``feast_free``.

    A complete list of options, their abbreviations, synonyms and
    default values is given in [Other Parameters].

    See Also
    --------
    :meth:`naginterfaces.library.examples.sparseig.feast_poly_gen_solve_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.feast_option'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12jbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_HANDLE, # Input, handle.
        utils._BYREF_STRING, # Input, optstr(len=*).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of optstr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _handle = utils._EngineCptrScalarType(
        handle, {'fun_name': fun_name, 'entity': 'handle'},
    ).to_ctypes()
    _optstr = utils._EngineCharScalarType(
        optstr, {'fun_name': fun_name, 'entity': 'optstr'}, exp_chrlen=None,
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _handle,
        _optstr,
        _errbuf,
        _ierr,
        _optstr._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Ambiguous keyword: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Keyword not recognized: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Second keyword not recognized: {:s}'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'handle has not been initialized properly or is corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'One of the contour setting functions feast_symm_contour, feast_gen_contour or feast_custom_contour has already',
                    'been called. All calls to feast_option must be made prior to calling the',
                    'contour setting functions.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = None
    _err_class.manage()
    return return_data

def feast_symm_contour(handle, emin, emax):
    u"""
    Setup function for ``feast_real_symm_solve`` and
    ``feast_complex_herm_solve``. Computes nodes and weights for an
    elliptical contour, symmetric about the real line.

    ``feast_symm_contour`` is a setup function in a suite of functions
    consisting of ``feast_init``, ``feast_option``,
    ``feast_symm_contour``, ``feast_real_symm_solve`` and
    ``feast_complex_herm_solve``.
    It is used to find some of the eigenvalues, and the corresponding
    eigenvectors, of a standard or generalized eigenvalue problem
    defined by real symmetric or complex Hermitian matrices.
    The initialization function ``feast_init`` **must** have been called
    prior to calling ``feast_symm_contour``.
    In addition calls to ``feast_option`` can be made to supply
    individual options to ``feast_symm_contour``.

    The suite of functions is suitable for the solution of large sparse
    eigenproblems where only a few eigenvalues from a selected range of
    the spectrum are required.

    For full information please refer to the NAG Library document for
    f12je

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12jef.html

    Parameters
    ----------
    handle : Handle
        The handle to the internal data structure used by the NAG FEAST
        suite. It needs to be initialized by ``feast_init``. **It must
        not be changed between calls to the NAG FEAST suite.**

    emin : float
        The lower bound of the eigenvalue search interval.

    emax : float
        The upper bound of the eigenvalue search interval.

    Raises
    ------
    NagValueError
        (`errno` 1)
            `handle` has not been initialized properly or is corrupted.

        (`errno` 2)
            An invalid number of integration points was specified. For
            Gauss or Zolotarev integration, the values permitted are
            1--20, 24, 32, 40, 48, 56.

        (`errno` 3)
            On entry, E_min = *<value>* and E_max = *<value>*.

            Constraint: E_min < E_max.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, and the corresponding eigenvectors, x, of a
    standard eigenvalue problem Ax = lambda x, or a generalized
    eigenvalue problem Ax = lambda Bx, where the coefficient matrices A
    and B are sparse, real symmetric or complex Hermitian.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale dense, real symmetric or complex Hermitian
    problems.

    ``feast_symm_contour`` is used to specify a search interval on the
    real line, [E_min,E_max], within which eigenvalues will be sought
    (note that the eigenvalues of real symmetric and complex Hermitian
    eigenproblems are themselves real). ``feast_symm_contour`` uses this
    interval to define nodes and weights for an elliptical contour to be
    used by the solvers ``feast_real_symm_solve`` or
    ``feast_complex_herm_solve``.
    Since this contour is symmetric when reflected in the real line, the
    function needs only to define the nodes and weights for the upper
    half-contour.

    References
    ----------
    Polizzi, E, 2009, `Density-Matrix-Based Algorithms for Solving
    Eigenvalue Problems`, Phys. Rev. B. (79), 115112
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.feast_symm_contour'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12jeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_HANDLE, # Input, handle.
        utils._BYREF_FLOAT64, # Input, emin.
        utils._BYREF_FLOAT64, # Input, emax.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _handle = utils._EngineCptrScalarType(
        handle, {'fun_name': fun_name, 'entity': 'handle'},
    ).to_ctypes()
    _emin = utils._EngineFloat64ScalarType(
        emin, {'fun_name': fun_name, 'entity': 'emin'},
    ).to_ctypes()
    _emax = utils._EngineFloat64ScalarType(
        emax, {'fun_name': fun_name, 'entity': 'emax'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _handle,
        _emin,
        _emax,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'handle has not been initialized properly or is corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An invalid number of integration points was specified.',
                    'For Gauss or',
                    'Zolotarev integration, the values permitted are 1--20, 24, 32, 40, 48, 56.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, E_min = {:s} and E_max = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: E_min < E_max.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = None
    _err_class.manage()
    return return_data

def feast_gen_contour(handle, emid, r):
    u"""
    Setup function for ``feast_real_gen_solve``,
    ``feast_complex_symm_solve``, ``feast_complex_gen_solve``,
    ``feast_poly_symm_solve`` and ``feast_poly_gen_solve``. Computes
    nodes and weights for an elliptical contour in the complex plane.

    ``feast_gen_contour`` is a setup function in a suite of functions
    consisting of ``feast_init``, ``feast_option``,
    ``feast_gen_contour``, ``feast_real_gen_solve``,
    ``feast_complex_symm_solve``, ``feast_complex_gen_solve``,
    ``feast_poly_symm_solve`` and ``feast_poly_gen_solve``.
    It is used to find some of the eigenvalues, and the corresponding
    eigenvectors, of a standard, generalized or polynomial eigenvalue
    problem.
    The initialization function ``feast_init`` **must** have been called
    prior to calling ``feast_gen_contour``.
    In addition calls to ``feast_option`` can be made to supply
    individual options to ``feast_gen_contour``.

    The suite of functions is suitable for the solution of large sparse
    eigenproblems where only a few eigenvalues from a selected range of
    the spectrum are required.

    For full information please refer to the NAG Library document for
    f12jf

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12jff.html

    Parameters
    ----------
    handle : Handle
        The handle to the internal data structure used by the NAG FEAST
        suite. It needs to be initialized by ``feast_init``. **It must
        not be changed between calls to the NAG FEAST suite.**

    emid : complex
        The centre of the ellipse.

    r : float
        The radius of the horizontal axis of the ellipse (that is, the
        axis that would be horizontal if the rotation angle of the
        ellipse was set to zero).

    Raises
    ------
    NagValueError
        (`errno` 1)
            `handle` has not been initialized properly or is corrupted.

        (`errno` 2)
            An invalid number of integration points was specified. For
            Gauss integration, the values permitted are 2--40 (even
            values only), 48, 64, 80, 96, 112.

        (`errno` 3)
            The option 'Integration Type' was set to 'Zol'. For
            non-Hermitian eigenvalue problems the allowed values are
            'Gauss' or 'Trap'.

        (`errno` 4)
            On entry, `r` = *<value>*.

            Constraint: `r` > 0.0.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, and the corresponding eigenvectors, x, of a
    standard eigenvalue problem Ax = lambda x, a generalized eigenvalue
    problem Ax = lambda Bx, or a polynomial eigenvalue problem
    ∑_ilambda^iA_i x = 0, where the coefficient matrices are large and
    sparse.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale dense problems.

    ``feast_gen_contour`` is used to specify a circle or ellipse in the
    complex plane within which eigenvalues will be sought.
    By default, a circle centred at emid with a radius of r is created.
    Optionally, ``feast_option`` can be called prior to calling
    ``feast_gen_contour``, using the options 'Ellipse Contour Ratio',
    'Ellipse Rotation Angle' to change the eccentricity of the ellipse
    and the inclination angle of its axis. ``feast_gen_contour`` uses
    these details to define nodes and weights for the elliptical
    contour, to be used by the solvers ``feast_real_gen_solve``,
    ``feast_complex_symm_solve``, ``feast_complex_gen_solve``,
    ``feast_poly_symm_solve`` or ``feast_poly_gen_solve``.

    For details of the other options available and how to set them see
    [Other Parameters for feast_option].

    References
    ----------
    Polizzi, E, 2009, `Density-Matrix-Based Algorithms for Solving
    Eigenvalue Problems`, Phys. Rev. B. (79), 115112
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.feast_gen_contour'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12jfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_HANDLE, # Input, handle.
        utils._BYREF_COMPLEX128, # Input, emid.
        utils._BYREF_FLOAT64, # Input, r.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _handle = utils._EngineCptrScalarType(
        handle, {'fun_name': fun_name, 'entity': 'handle'},
    ).to_ctypes()
    _emid = utils._EngineComplex128ScalarType(
        emid, {'fun_name': fun_name, 'entity': 'emid'},
    ).to_ctypes()
    _r = utils._EngineFloat64ScalarType(
        r, {'fun_name': fun_name, 'entity': 'r'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _handle,
        _emid,
        _r,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'handle has not been initialized properly or is corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An invalid number of integration points was specified.',
                    'For Gauss',
                    'integration, the values permitted are 2--40 (even values only), 48, 64,',
                    '80, 96, 112.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Integration Type\' was set to \'Zol\'.',
                    'For non-Hermitian',
                    'eigenvalue problems the allowed values are \'Gauss\' or \'Trap\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: r > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = None
    _err_class.manage()
    return return_data

def feast_custom_contour(handle, ccn, nedge, tedge, zedge):
    u"""
    Setup function for ``feast_real_gen_solve``,
    ``feast_complex_symm_solve``, ``feast_complex_gen_solve``,
    ``feast_poly_symm_solve`` and ``feast_poly_gen_solve``. Creates
    nodes and weights for a custom contour in the complex plane.

    ``feast_custom_contour`` is a setup function in a suite of functions
    consisting of ``feast_init``, ``feast_option``,
    ``feast_custom_contour``, ``feast_real_gen_solve``,
    ``feast_complex_symm_solve``, ``feast_complex_gen_solve``,
    ``feast_poly_symm_solve`` and ``feast_poly_gen_solve``.
    It is used to find some of the eigenvalues, and the corresponding
    eigenvectors, of a standard, generalized or polynomial eigenvalue
    problem.
    The initialization function ``feast_init`` **must** have been called
    prior to calling ``feast_custom_contour``.
    In addition calls to ``feast_option`` can be made to supply
    individual options to ``feast_custom_contour``.

    The suite of functions is suitable for the solution of large sparse
    eigenproblems where only a few eigenvalues from a selected range of
    the spectrum are required.

    For full information please refer to the NAG Library document for
    f12jg

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12jgf.html

    Parameters
    ----------
    handle : Handle
        The handle to the internal data structure used by the NAG FEAST
        suite. It needs to be initialized by ``feast_init``. **It must
        not be changed between calls to the NAG FEAST suite.**

    ccn : int
        The number of pieces that make up the contour.

    nedge : int, shape (`ccn`)
        `nedge`[i-1] specifies how many integration nodes and weights
        ``feast_custom_contour`` should use for the ith piece of the
        contour.

    tedge : int, shape (`ccn`)
        `tedge`[i-1] specifies what shape the ith piece of the contour
        should be.

        `tedge`[i-1] = 0

            The ith piece of the contour is straight.

        `tedge`[i-1] > 0

            The ith piece of the contour is a (convex) half-ellipse,
            with `tedge`[i-1]/100 = a/b, where a is the primary radius
            from the endpoints of the piece, and b is the radius
            perpendicular to this. Thus, if `tedge`[i-1] = 100, then the
            ith piece of the contour is a semicircle.

    zedge : complex, shape (`ccn`)
        `zedge` specifies the endpoints of the contour piece.

        The ith piece has endpoints at `zedge`[i-1] and `zedge`[i], for
        i = 1,...,`ccn`-1.

        The last piece has endpoints at `zedge`[`ccn`-1] and `zedge`[0].

        Note: the contour should be described in a clockwise direction..

    Raises
    ------
    NagValueError
        (`errno` 1)
            `handle` has not been initialized properly or is corrupted.

        (`errno` 2)
            On entry, `ccn` = *<value>*.

            Constraint: `ccn` > 1.

        (`errno` 3)
            On entry, one or more elements of `nedge` were less than or
            equal to zero.

        (`errno` 4)
            On entry, one or more elements of `tedge` were negative.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, and the corresponding eigenvectors, x, of a
    standard eigenvalue problem Ax = lambda x, a generalized eigenvalue
    problem Ax = lambda Bx, or a polynomial eigenvalue problem
    ∑_ilambda^iA_i x = 0, where the coefficient matrices are large and
    sparse.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale dense problems.

    ``feast_custom_contour`` is used to specify a closed contour in the
    complex plane within which eigenvalues will be sought.
    The contour can be made up of a combination of line segments and
    half ellipses. ``feast_custom_contour`` uses this information to
    create a polygonal representation of the contour and to then define
    the integration nodes and weights to be used by the solvers
    ``feast_real_gen_solve``, ``feast_complex_symm_solve``,
    ``feast_complex_gen_solve``, ``feast_poly_symm_solve`` or
    ``feast_poly_gen_solve``.

    The arrays `zedge`, `tedge` and `nedge` are used to define the
    geometry of your contour.
    Each array is of size `ccn`, where `ccn` is the number of pieces
    that make up the contour.
    The entries in `zedge` specify the endpoints in the complex plane of
    each piece of the contour.
    The entries in `tedge` specify whether each piece of the contour is
    a line segment or a half ellipse.
    Finally, entries in `nedge` specify the number of integration points
    to use for each piece of the contour.
    See the individual argument descriptions in [Parameters] for further
    details.

    Prior to calling ``feast_custom_contour``, the option setting
    function ``feast_option`` can be called to specify various options
    for the solution of the eigenproblem.
    For details of the options available and how to set them see [Other
    Parameters for feast_option].

    References
    ----------
    Polizzi, E, 2009, `Density-Matrix-Based Algorithms for Solving
    Eigenvalue Problems`, Phys. Rev. B. (79), 115112

    See Also
    --------
    :meth:`naginterfaces.library.examples.sparseig.feast_poly_gen_solve_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.feast_custom_contour'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12jgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_HANDLE, # Input, handle.
        utils._BYREF_INT, # Input, ccn.
        utils._EngineIntArrayType.c_type, # Input, nedge[ccn].
        utils._EngineIntArrayType.c_type, # Input, tedge[ccn].
        utils._EngineComplex128ArrayType.c_type, # Input, zedge[ccn].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _handle = utils._EngineCptrScalarType(
        handle, {'fun_name': fun_name, 'entity': 'handle'},
    ).to_ctypes()
    _ccn = utils._EngineIntScalarType(
        ccn, {'fun_name': fun_name, 'entity': 'ccn'},
    ).to_ctypes()
    _ccn_py = utils._EngineIntScalarType.to_py(_ccn)
    _nedge = utils._EngineIntArrayType(
        nedge, {'fun_name': fun_name, 'entity': 'nedge'},
        exp_shape=(_ccn_py,),
    ).to_ctypes()
    _tedge = utils._EngineIntArrayType(
        tedge, {'fun_name': fun_name, 'entity': 'tedge'},
        exp_shape=(_ccn_py,),
    ).to_ctypes()
    _zedge = utils._EngineComplex128ArrayType(
        zedge, {'fun_name': fun_name, 'entity': 'zedge'},
        exp_shape=(_ccn_py,),
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _handle,
        _ccn,
        _nedge,
        _tedge,
        _zedge,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'handle has not been initialized properly or is corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ccn = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ccn > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, one or more elements of nedge were less than or equal to zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, one or more elements of tedge were negative.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = None
    _err_class.manage()
    return return_data

def feast_real_symm_solve(handle, irevcm, ze, n, x, y, m0, d, z, eps, itera, resid, io_manager=None):
    u"""
    Selected eigenvalues and eigenvectors of a real symmetric
    eigenproblem, reverse communication driver.

    ``feast_real_symm_solve`` is an iterative solver used to find some
    of the eigenvalues and the corresponding eigenvectors of a standard
    or generalized eigenvalue problem defined by real symmetric
    matrices.
    This is part of a suite of functions that also includes
    ``feast_init``, ``feast_option`` and ``feast_symm_contour``.

    For full information please refer to the NAG Library document for
    f12jj

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12jjf.html

    Parameters
    ----------
    handle : Handle
        The handle to the internal data structure used by the NAG FEAST
        suite. It needs to be initialized by ``feast_init``. **It must
        not be changed between calls to the NAG FEAST suite.**

    irevcm : int
        `On initial entry`: `irevcm` = 0, otherwise an error condition
        will be raised.

        `On intermediate entry`: must be unchanged from its previous
        exit value. Changing `irevcm` to any other value between calls
        will result in an error.

        Note: the matrices x, y and z referred to in this section are
        all of size `n`*`m0` and are stored in the arrays `x`, `y` and
        `z`, respectively.

    ze : complex
        `On initial entry`: need not be set.

    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    x : float, shape (`n`, m0), modified in place
        `On initial entry`: need not be set.

        `On intermediate exit`:

            if `irevcm` = 3, the calling program must compute Az,
            storing the result in x prior to re-entry.

            If `irevcm` = 4, the calling program must compute Bz,
            storing the result in x prior to re-entry.

        Note: the matrices x and z are stored in the first `m0` columns
        of the arrays `x` and `z`, respectively.

    y : complex, shape (`n`, m0), modified in place
        `On initial entry`: need not be set.

        `On intermediate exit`: if `irevcm` = 2, the calling program
        must compute the solution to the linear system (`ze`B-A)w = y,
        overwriting y with the result w, prior to re-entry. The linear
        system has `m0` right-hand sides.

    m0 : int
        `On initial entry`: the size of the search subspace used to find
        the eigenvalues. This should exceed the number of eigenvalues
        within the search contour. See [Further Comments] for further
        details.

        `On intermediate entry`: `m0` must remain unchanged.

    d : float, shape (m0), modified in place
        `On initial entry`: if the option 'Subspace' = 'Yes' was set
        using the option setting function ``feast_option``, then `d`
        should contain an initial guess at the eigenvalues lying within
        the eigenvector search subspace (this subspace should be
        specified by `z`), otherwise `d` need not be set.

        `On final exit`: the first `nconv` entries in `d` contain the
        eigenvalues.

        Note: if the option 'Subspace' = 'Yes' was set using the option
        setting function ``feast_option``, then on final exit `d`
        contains an estimate of the eigenvalues after a single contour
        integral.

    z : float, shape (`n`, m0), modified in place
        `On initial entry`: if the option 'Subspace' = 'Yes' was set
        using the option setting function ``feast_option``, then `z`
        should contain an initial guess at the eigenvector search
        subspace, otherwise `z` need not be set.

        `On intermediate exit`: must not be changed.

        `On final exit`: the first `nconv` columns of `z` contain the
        eigenvectors corresponding to the eigenvalues found within the
        contour.

        Note: if the option 'Execution Mode' = 'Subspace' was set using
        the option setting function ``feast_option``, then on final exit
        columns 1:`m0` of `z` contain the current search subspace after
        one contour integral.

    eps : float
        `On initial entry`: need not be set.

    itera : int
        `On initial entry`: need not be set.

    resid : float, shape (m0), modified in place
        `On initial entry`: need not be set.

        `On final exit`: for i = 1,...,`nconv`, `resid`[i-1] contains
        the relative residual, in the 1-norm, of the ith eigenpair
        found, that is `resid`[i-1] = ||Az_i-lambda_i
        Bz_i||/(||Bz_i||*max(|E_min|,|E_max|)), where E_min and E_max
        are the lower and upper bounds respectively of the eigenvalue
        search interval.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    irevcm : int
        `On intermediate exit`: has the following meanings.

        `irevcm` = 1

            The calling program must compute a factorization of the
            matrix `ze`B-A suitable for solving a linear system, for
            example using ``lapacklin.zsytrf`` which computes the
            Bunch--Kaufman factorization. All arguments to the routine
            must remain unchanged.

            **Note:** the factorization can be computed in single
            precision.

        `irevcm` = 2

            The calling program must compute the solution to the linear
            system (`ze`B-A)w = y, overwriting y with the result w. The
            matrix `ze`B-A has previously been factorized (when `irevcm`
            = 1 was returned) and this factorization can be reused here.

            **Note:** the solve can be performed in single precision.

        `irevcm` = 3

            The calling program must compute Az, storing the result in
            x.

        `irevcm` = 4

            The calling program must compute Bz, storing the result in
            x. If a standard eigenproblem is being solved (so that B =
            I) then the calling program should set x = z.

        `On final exit`: `irevcm` = 0: ``feast_real_symm_solve`` has
        completed its tasks. The value of errno determines whether the
        iteration has been successfully completed, or whether errors
        have been detected.

        Note: the matrices x, y and z referred to in this section are
        all of size `n`*`m0` and are stored in the arrays `x`, `y` and
        `z`, respectively.

    ze : complex
        `On intermediate exit`: contains the current point on the
        contour.

            If `irevcm` = 1, then this must be used by the calling
            program to form a factorization of the matrix `ze`B-A.

    m0 : int
        If the initial search subspace was found by
        ``feast_real_symm_solve`` to be too large, then a new smaller
        suitable choice is returned.

    nconv : int
        The number of eigenvalues found within the search contour.

        Note: if the option 'Execution Mode' = 'Estimate' was set in the
        option setting function ``feast_option``, then `nconv` contains
        a stochastic estimate of the number of eigenvalues within the
        search contour.

    eps : float
        The relative error on the trace. At iteration k, `eps` is given
        by the expression |trace_k-trace_{k-1}|/max(|E_min|,|E_max|),
        where trace_k is the sum of the eigenvalues found at the kth
        iteration, and E_min and E_max are the lower and upper bounds
        respectively of the eigenvalue search interval.

    itera : int
        The number of subspace iterations performed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Either the contour setting function ``feast_symm_contour``
            has not been called prior to the first call of this function
            or the supplied `handle` has become corrupted.

        (`errno` 7)
            An internal error occurred in the reduced eigenvalue solver.
            A possible cause is that the matrix B is not positive
            definite.

        (`errno` 8)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 9)
            On entry, `m0` = *<value>* and `n` = *<value>*.

            Constraint: 0 < `m0` <= `n`.

        (`errno` 13)
            On initial entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0.

        (`errno` 13)
            On intermediate entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 1, 2, 3 or 4.

        (`errno` 14)
            The option 'Subspace' = 'Yes' was set using the option
            setting function ``feast_option`` but no nonzero elements
            were found in the supplied subspace.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            No eigenvalues were found within the search contour.

        (`errno` 3)
            The function did not converge after the maximum number of
            iterations.

        (`errno` 4)
            The size of the eigenvector search subspace, `m0`, is too
            small.

        (`errno` 5)
            The option 'Execution Mode' = 'Subspace' was set using
            ``feast_option``. Columns 1:`m0` of `z` contain the search
            subspace after one contour integral and `d` contains an
            estimate of the eigenvalues.

        (`errno` 6)
            The option 'Execution Mode' = 'Estimate' was set using
            ``feast_option`` so only a stochastic estimate of the number
            of eigenvalues within the contour has been returned.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, and the corresponding eigenvectors, x, of a
    standard eigenvalue problem Ax = lambda x, or of a generalized
    eigenvalue problem Ax = lambda Bx, where the coefficient matrices A
    and B are sparse, real symmetric, and B is positive definite.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale, dense, real symmetric problems.

    ``feast_real_symm_solve`` is a **reverse communication** function,
    based on the FEAST eigensolver, described in Polizzi (2009), which
    finds eigenvalues using contour integration.
    Prior to calling ``feast_real_symm_solve``, the contour definition
    function ``feast_symm_contour`` is used to specify a search interval
    on the real line, [E_min,E_max], within which eigenvalues will be
    sought (note that the eigenvalues of real symmetric eigenproblems
    are themselves real). ``feast_symm_contour`` uses this interval to
    define nodes and weights for an elliptical contour to be used by
    ``feast_real_symm_solve``.

    The setup function ``feast_init`` and the contour definition
    function ``feast_symm_contour`` must be called before
    ``feast_real_symm_solve``.
    Between the calls to ``feast_init`` and ``feast_symm_contour``,
    options may be set by calls to the option setting function
    ``feast_option``.

    ``feast_real_symm_solve`` uses **reverse communication**, i.e., it
    returns repeatedly to the calling program with the argument `irevcm`
    (see [Parameters]) set to specified values which require the calling
    program to carry out one of the following tasks:

    -   compute a factorization of the matrix `ze`B-A, where `ze` is a
        point on the search contour;

    -   solve a linear system involving `ze`B-A, using the factorization
        above;

    -   compute the matrix product x = Az;

    -   compute the matrix product x = Bz;

    -   notify the completion of the computation.

    The number of contour points, the number of iterations, and other
    options can all be set using the option setting function
    ``feast_option`` (see [Other Parameters for feast_option] for
    details on setting options and of the default settings).
    The search contour itself is defined by a call to
    ``feast_symm_contour``.

    References
    ----------
    Polizzi, E, 2009, `Density-Matrix-Based Algorithms for Solving
    Eigenvalue Problems`, Phys. Rev. B. (79), 115112
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.feast_real_symm_solve'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12jjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_HANDLE, # Input, handle.
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._BYREF_COMPLEX128, # Input/Output, ze.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[ldx, *].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, y[ldy, *].
        utils._BYREF_INT, # Input, ldy.
        utils._BYREF_INT, # Input/Output, m0.
        utils._BYREF_INT, # Output, nconv.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._BYREF_FLOAT64, # Input/Output, eps.
        utils._BYREF_INT, # Input/Output, itera.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, resid[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (y, 'y'), (z, 'z')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _handle = utils._EngineCptrScalarType(
        handle, {'fun_name': fun_name, 'entity': 'handle'},
    ).to_ctypes()
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _ze = utils._EngineComplex128ScalarType(
        ze, {'fun_name': fun_name, 'entity': 'ze'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _m0_out = utils._f12jzyn(handle, irevcm, m0)
    _m0_out_py = utils._EngineIntScalarType.to_py(_m0_out)
    _vu_f12jzy_m0_out_py = _m0_out_py
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ldy_py = utils._EngineComplex128ArrayType.get_nd(y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldy = utils._EngineIntScalarType(
        _ldy_py, {'fun_name': fun_name, 'entity': '_ldy'},
    ).to_ctypes()
    _y_cls = utils._EngineComplex128ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py, _vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _y = _y_cls.to_ctypes()
    _m0 = utils._EngineIntScalarType(
        m0, {'fun_name': fun_name, 'entity': 'm0'},
    ).to_ctypes()
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_vu_f12jzy_m0_out_py,),
    )
    _d = _d_cls.to_ctypes()
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=(_n_py, _vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _eps = utils._EngineFloat64ScalarType(
        eps, {'fun_name': fun_name, 'entity': 'eps'},
    ).to_ctypes()
    _itera = utils._EngineIntScalarType(
        itera, {'fun_name': fun_name, 'entity': 'itera'},
    ).to_ctypes()
    _resid_cls = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vu_f12jzy_m0_out_py,),
    )
    _resid = _resid_cls.to_ctypes()
    _routine_name = utils._EngineRoutineName(*b'f12jj ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _handle,
        _irevcm,
        _ze,
        _n,
        _x,
        _ldx,
        _y,
        _ldy,
        _m0,
        _nconv,
        _d,
        _z,
        _ldz,
        _eps,
        _itera,
        _resid,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the contour setting function feast_symm_contour has not been called prior',
                    'to the first call of this function or',
                    'the supplied handle has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No eigenvalues were found within the search contour.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function did not converge after the maximum number of iterations.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The size of the eigenvector search subspace, m0, is too small.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Execution Mode\' = \'Subspace\' was set using feast_option.',
                    'Columns 1:m0 of z contain the search subspace after one contour integral',
                    'and d contains an estimate of the eigenvalues.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Execution Mode\' = \'Estimate\' was set using feast_option so',
                    'only a stochastic estimate of the number of eigenvalues within the',
                    'contour has been returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred in the reduced eigenvalue solver. A possible',
                    'cause is that the matrix B is not positive definite.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m0 = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 0 < m0 <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 1000:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On initial entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On intermediate entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 1, 2, 3 or 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 14:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Subspace\' = \'Yes\' was set using the option setting function feast_option',
                    'but no nonzero elements were found in the supplied subspace.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ze_py = utils._EngineComplex128ScalarType.to_py(_ze)
        _m0_out_py = utils._EngineIntScalarType.to_py(_m0_out)
        _vu_f12jzy_m0_out_py = _m0_out_py
        _x_cls.refresh_py(_x)
        _y_cls.refresh_py(_y)
        _m0_py = utils._EngineIntScalarType.to_py(_m0)
        _d_cls.refresh_py(_d)
        _z_cls.refresh_py(_z)
        _eps_py = utils._EngineFloat64ScalarType.to_py(_eps)
        _itera_py = utils._EngineIntScalarType.to_py(_itera)
        _resid_cls.refresh_py(_resid)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['irevcm', 'ze', 'm0', 'nconv', 'eps', 'itera'],
        )(
            _irevcm_py,
            _ze_py,
            _m0_py,
            _nconv_py,
            _eps_py,
            _itera_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def feast_real_gen_solve(handle, irevcm, ze, n, x, y, m0, d, z, eps, itera, resid, io_manager=None):
    u"""
    Selected eigenvalues and eigenvectors of a real nonsymmetric
    eigenproblem, reverse communication driver.

    ``feast_real_gen_solve`` is an iterative solver used to find some of
    the eigenvalues and the corresponding eigenvectors of a standard or
    generalized eigenvalue problem defined by real nonsymmetric
    matrices.
    This is part of a suite of functions that also includes
    ``feast_init``, ``feast_option``, ``feast_gen_contour`` and
    ``feast_custom_contour``.

    For full information please refer to the NAG Library document for
    f12jk

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12jkf.html

    Parameters
    ----------
    handle : Handle
        The handle to the internal data structure used by the NAG FEAST
        suite. It needs to be initialized by ``feast_init``. **It must
        not be changed between calls to the NAG FEAST suite.**

    irevcm : int
        `On initial entry`: `irevcm` = 0, otherwise an error condition
        will be raised.

        `On intermediate entry`: must be unchanged from its previous
        exit value. Changing `irevcm` to any other value between calls
        will result in an error.

        Note: the matrices x, y and z referred to in this section are
        all of size `n`*`m0` and are stored in the arrays `x`, `y` and
        `z`, respectively.

    ze : complex
        `On initial entry`: need not be set.

    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    x : float, shape (`n`, m0), modified in place
        `On initial entry`: need not be set.

        `On intermediate exit`:

            if `irevcm` = 5, the calling program must compute Az,
            storing the result in x prior to re-entry.

            If `irevcm` = 6, the calling program must compute A^Tz,
            storing the result in x prior to re-entry.

            If `irevcm` = 7, the calling program must compute Bz,
            storing the result in x prior to re-entry.

            If `irevcm` = 8, the calling program must compute B^Tz,
            storing the result in x prior to re-entry.

        Note: the matrices x and z are stored in the first `m0` columns
        of the arrays `x` and `z`, respectively.

    y : complex, shape (`n`, 2*m0), modified in place
        `On initial entry`: if the option 'Subspace' = 'Yes' was set
        using the option setting function ``feast_option``, then `y`
        should contain an initial guess at the eigenvector search
        subspace, otherwise `y` need not be set.

        Note: if the option 'Subspace' = 'Yes' was set but only right
        eigenvectors are to be computed, then only the first `m0`
        columns of `y` need to be set. However, if the option
        'Eigenvectors' = 'Two-sided' was set using ``feast_option``, so
        that both left and right eigenvectors are returned, then the
        first `m0` columns of `y` should contain the initial guess for
        the right eigenvector search subspace and the remaining `m0`
        columns should contain an initial guess for the left eigenvector
        search subspace.

        `On intermediate exit`:

            if `irevcm` = 2, the calling program must compute the
            solution to the linear system (`ze`B-A)w = y, overwriting y
            with the result w, prior to re-entry. The linear system has
            `m0` right-hand sides.

            If `irevcm` = 4, the calling program must compute the
            solution to the linear system (`ze`B-A)^Hw = y, overwriting
            y with the result w, prior to re-entry. The linear system
            has `m0` right-hand sides.

        `On final exit`: the first `nconv` columns of `y` contain the
        right eigenvectors corresponding to the eigenvalues found within
        the contour.

        Note: if the option 'Eigenvectors' = 'Two-sided' was set using
        ``feast_option``, the columns `m0`+1:`m0`+`nconv` of `y` contain
        the left eigenvectors corresponding to the eigenvalues found
        within the contour.

        Note: If the option 'Execution Mode' = 'Subspace' was set using
        the option setting function ``feast_option``, then on final exit
        columns 1:`m0` of `y` contain the current search subspace after
        one contour integral.

    m0 : int
        `On initial entry`: the size of the search subspace used to find
        the eigenvalues. This should exceed the number of eigenvalues
        within the search contour. See [Further Comments] for further
        details.

        `On intermediate entry`: `m0` must remain unchanged.

    d : complex, shape (m0), modified in place
        `On initial entry`: if the option 'Subspace' = 'Yes' was set
        using the option setting function ``feast_option``, then `d`
        should contain an initial guess at the eigenvalues lying within
        the eigenvector search subspace (this subspace should be
        specified by `y`), otherwise `d` need not be set.

        `On final exit`: the first `nconv` entries in `d` contain the
        eigenvalues.

    z : float, shape (`n`, m0), modified in place
        `On initial entry`: need not be set.

        `On intermediate exit`: must not be changed.

    eps : float
        `On initial entry`: need not be set.

    itera : int
        `On initial entry`: need not be set.

    resid : float, shape (2*m0), modified in place
        `On initial entry`: need not be set.

        `On final exit`: for i = 1,...,`nconv`, `resid`[i-1] contains
        the relative residual, in the 1-norm, of the ith eigenpair
        found, that is `resid`[i-1] = ||Ay_i-lambda_i
        By_i||/(||By_i||*(|E_mid|+r)), where E_mid is the centre of mass
        of the contour and r is the radius of the circle, centred at
        E_mid which just encloses the contour.

        If the option 'Eigenvectors' = 'Two-sided' was set in the option
        setting function ``feast_option``, the entries
        `m0`+1:`m0`+`nconv` of `resid` contain corresponding residuals
        for the left eigenvectors, `resid`[i-1] = ||A^Hy_i-lambda__i
        B^Hy_i||/(||B^Hy_i||*(|E_mid|+r)).

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    irevcm : int
        `On intermediate exit`: has the following meanings.

        `irevcm` = 1

            The calling program must compute a factorization of the
            matrix `ze`B-A suitable for solving a linear system, for
            example using ``sparse.complex_gen_precon_ilu`` which
            computes an incomplete LU factorization of a complex sparse
            matrix. All arguments to the routine must remain unchanged.

            **Note:** the factorization can be computed in single
            precision.

        `irevcm` = 2

            The calling program must compute the solution to the linear
            system (`ze`B-A)w = y, overwriting y with the result w. The
            matrix `ze`B-A has previously been factorized (when `irevcm`
            = 1 was returned) and this factorization can be reused here.

            **Note:** the solve can be performed in single precision.

        `irevcm` = 3

            Optionally, the calling program must compute a factorization
            of the matrix (`ze`B-A)^T. This need only be done if it is
            not possible to use the factorization computed when `irevcm`
            = 1 was returned to solve linear systems involving
            (`ze`B-A)^T. If this factorization is to be computed, then
            the factorization from `irevcm` = 1 must not be overwritten.

            **Note:** the factorization can be performed in single
            precision.

        `irevcm` = 4

            The calling program must compute the solution to the linear
            system (`ze`B-A)^Tw = y, overwriting y with the result w. If
            it is not possible to use the factorization of `ze`B-A
            (computed when `irevcm` = 1 was returned) then the
            factorization of (`ze`B-A)^T (computed when `irevcm` = 2 was
            returned) should be used here.

            **Note:** the solve can be performed in single precision.

        `irevcm` = 5

            The calling program must compute Az, storing the result in
            x.

        `irevcm` = 6

            The calling program must compute A^Tz, storing the result in
            x.

        `irevcm` = 7

            The calling program must compute Bz, storing the result in
            x. If a standard eigenproblem is being solved (so that B =
            I) then the calling program should set x = z.

        `irevcm` = 8

            The calling program must compute B^Tz, storing the result in
            x. If a standard eigenproblem is being solved (so that B =
            I) then the calling program should set x = z.

        `On final exit`: `irevcm` = 0: ``feast_real_gen_solve`` has
        completed its tasks. The value of errno determines whether the
        iteration has been successfully completed, or whether errors
        have been detected.

        Note: the matrices x, y and z referred to in this section are
        all of size `n`*`m0` and are stored in the arrays `x`, `y` and
        `z`, respectively.

    ze : complex
        `On intermediate exit`: contains the current point on the
        contour.

            If `irevcm` = 1, then this must be used by the calling
            program to form a factorization of the matrix `ze`B-A.

            If `irevcm` = 3, then, optionally, this can be used to form
            a factorization of (`ze`B-A)^H.

    m0 : int
        If the initial search subspace was found by
        ``feast_real_gen_solve`` to be too large, then a new smaller
        suitable choice is returned.

    nconv : int
        The number of eigenvalues found within the search contour.

        Note: if the option 'Execution Mode' = 'Estimate' was set in the
        option setting function ``feast_option``, then `nconv` contains
        a stochastic estimate of the number of eigenvalues within the
        search contour.

    eps : float
        The relative error on the trace. At iteration k, `eps` is given
        by the expression |trace_k-trace_{k-1}|/(|E_mid|+r), where
        trace_k is the sum of the eigenvalues found at the kth
        iteration, E_mid is the centre of mass of the contour and r is
        the radius of the circle, centred at E_mid which just encloses
        the contour.

    itera : int
        The number of subspace iterations performed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Either one of the contour setting functions
            ``feast_gen_contour`` or ``feast_custom_contour`` has not
            been called prior to the first call of this function or the
            supplied `handle` has become corrupted.

        (`errno` 7)
            An internal error occurred in the reduced eigenvalue solver.
            Please contact NAG.

        (`errno` 8)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 9)
            On entry, `m0` = *<value>* and `n` = *<value>*.

            Constraint: 0 < `m0` <= `n`.

        (`errno` 13)
            On initial entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0.

        (`errno` 13)
            On intermediate entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 1, 2, 3, 4, 5, 6, 7 or 8.

        (`errno` 14)
            The function converged but the left/right eigenvector
            subspaces are not bi-orthonormal.

        (`errno` 14)
            The function converged but the right eigenvector subspace is
            not orthonormal.

        (`errno` 15)
            The option 'Subspace' = 'Yes' was set using the option
            setting function ``feast_option`` but no nonzero elements
            were found in the supplied subspace.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            No eigenvalues were found within the search contour.

        (`errno` 3)
            The function did not converge after the maximum number of
            iterations.

        (`errno` 4)
            The size of the eigenvector search subspace, `m0`, is too
            small.

        (`errno` 5)
            The option 'Execution Mode' = 'Subspace' was set using
            ``feast_option``. Columns 1:`m0` of `z` contain the search
            subspace after one contour integral and `d` contains an
            estimate of the eigenvalues.

        (`errno` 6)
            The option 'Execution Mode' = 'Estimate' was set using
            ``feast_option`` so only a stochastic estimate of the number
            of eigenvalues within the contour has been returned.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, and the corresponding eigenvectors, x, of a
    standard eigenvalue problem Ax = lambda x, or of a generalized
    eigenvalue problem Ax = lambda Bx, where the coefficient matrices A
    and B are sparse, real and nonsymmetric.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale, dense problems.

    ``feast_real_gen_solve`` is a **reverse communication** function,
    based on the FEAST eigensolver, described in Polizzi (2009), which
    finds eigenvalues using contour integration.
    Prior to calling ``feast_real_gen_solve``, one of the contour
    definition functions ``feast_gen_contour`` or
    ``feast_custom_contour`` must be used to define nodes and weights
    for a contour around a region in the complex plane within which
    eigenvalues will be sought.

    The setup function ``feast_init`` and the contour definition
    function ``feast_gen_contour`` or ``feast_custom_contour`` must be
    called before ``feast_real_gen_solve``.
    Between the calls to ``feast_init`` and ``feast_gen_contour`` or
    ``feast_custom_contour``, options may be set by calls to the option
    setting function ``feast_option``.

    ``feast_real_gen_solve`` uses **reverse communication**, i.e., it
    returns repeatedly to the calling program with the argument `irevcm`
    (see [Parameters]) set to specified values which require the calling
    program to carry out one of the following tasks:

    -   compute a factorization of the complex matrix `ze`B-A, where
        `ze` is a point on the search contour;

    -   optionally, compute a factorization of the matrix (`ze`B-A)^H
        (this need only be done if the factorization `ze`B-A does not
        allow linear systems involving (`ze`B-A)^H to be solved);

    -   solve a linear system involving `ze`B-A or (`ze`B-A)^H, using
        the factorizations above;

    -   compute the matrix product x = Az;

    -   compute the matrix product x = A^Tz;

    -   compute the matrix product x = Bz;

    -   compute the matrix product x = B^Tz;

    -   notify the completion of the computation.

    The number of contour points, the number of iterations, and other
    options can all be set using the option setting function
    ``feast_option`` (see [Other Parameters for feast_option] for
    details on setting options and of the default settings).
    The search contour itself is defined by a call to
    ``feast_gen_contour`` or ``feast_custom_contour``.

    References
    ----------
    Polizzi, E, 2009, `Density-Matrix-Based Algorithms for Solving
    Eigenvalue Problems`, Phys. Rev. B. (79), 115112
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.feast_real_gen_solve'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12jkft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_HANDLE, # Input, handle.
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._BYREF_COMPLEX128, # Input/Output, ze.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[ldx, *].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, y[ldy, *].
        utils._BYREF_INT, # Input, ldy.
        utils._BYREF_INT, # Input/Output, m0.
        utils._BYREF_INT, # Output, nconv.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, d[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._BYREF_FLOAT64, # Input/Output, eps.
        utils._BYREF_INT, # Input/Output, itera.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, resid[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (y, 'y'), (z, 'z')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _handle = utils._EngineCptrScalarType(
        handle, {'fun_name': fun_name, 'entity': 'handle'},
    ).to_ctypes()
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _ze = utils._EngineComplex128ScalarType(
        ze, {'fun_name': fun_name, 'entity': 'ze'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _m0_out = utils._f12jzyn(handle, irevcm, m0)
    _m0_out_py = utils._EngineIntScalarType.to_py(_m0_out)
    _vu_f12jzy_m0_out_py = _m0_out_py
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ldy_py = utils._EngineComplex128ArrayType.get_nd(y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldy = utils._EngineIntScalarType(
        _ldy_py, {'fun_name': fun_name, 'entity': '_ldy'},
    ).to_ctypes()
    _y_cls = utils._EngineComplex128ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py, 2*_vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _y = _y_cls.to_ctypes()
    _m0 = utils._EngineIntScalarType(
        m0, {'fun_name': fun_name, 'entity': 'm0'},
    ).to_ctypes()
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    _d_cls = utils._EngineComplex128ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_vu_f12jzy_m0_out_py,),
    )
    _d = _d_cls.to_ctypes()
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=(_n_py, _vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _eps = utils._EngineFloat64ScalarType(
        eps, {'fun_name': fun_name, 'entity': 'eps'},
    ).to_ctypes()
    _itera = utils._EngineIntScalarType(
        itera, {'fun_name': fun_name, 'entity': 'itera'},
    ).to_ctypes()
    _resid_cls = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(2*_vu_f12jzy_m0_out_py,),
    )
    _resid = _resid_cls.to_ctypes()
    _routine_name = utils._EngineRoutineName(*b'f12jk ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _handle,
        _irevcm,
        _ze,
        _n,
        _x,
        _ldx,
        _y,
        _ldy,
        _m0,
        _nconv,
        _d,
        _z,
        _ldz,
        _eps,
        _itera,
        _resid,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either one of the contour setting functions feast_gen_contour or feast_custom_contour has not',
                    'been called prior to the first call of this function or',
                    'the supplied handle has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No eigenvalues were found within the search contour.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function did not converge after the maximum number of iterations.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The size of the eigenvector search subspace, m0, is too small.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Execution Mode\' = \'Subspace\' was set using feast_option.',
                    'Columns 1:m0 of z contain the search subspace after one contour integral',
                    'and d contains an estimate of the eigenvalues.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Execution Mode\' = \'Estimate\' was set using feast_option so',
                    'only a stochastic estimate of the number of eigenvalues within the',
                    'contour has been returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred in the reduced eigenvalue solver.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m0 = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 0 < m0 <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 1000:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On initial entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On intermediate entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 1, 2, 3, 4, 5, 6, 7 or 8.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 1000:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function converged but the left/right eigenvector subspaces are not',
                    'bi-orthonormal.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function converged but the right eigenvector subspace is not',
                    'orthonormal.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 15:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Subspace\' = \'Yes\' was set using the option setting function feast_option',
                    'but no nonzero elements were found in the supplied subspace.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ze_py = utils._EngineComplex128ScalarType.to_py(_ze)
        _m0_out_py = utils._EngineIntScalarType.to_py(_m0_out)
        _vu_f12jzy_m0_out_py = _m0_out_py
        _x_cls.refresh_py(_x)
        _y_cls.refresh_py(_y)
        _m0_py = utils._EngineIntScalarType.to_py(_m0)
        _d_cls.refresh_py(_d)
        _z_cls.refresh_py(_z)
        _eps_py = utils._EngineFloat64ScalarType.to_py(_eps)
        _itera_py = utils._EngineIntScalarType.to_py(_itera)
        _resid_cls.refresh_py(_resid)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['irevcm', 'ze', 'm0', 'nconv', 'eps', 'itera'],
        )(
            _irevcm_py,
            _ze_py,
            _m0_py,
            _nconv_py,
            _eps_py,
            _itera_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def feast_complex_herm_solve(handle, irevcm, ze, n, x, y, m0, d, z, eps, itera, resid, io_manager=None):
    u"""
    Selected eigenvalues and eigenvectors of a complex Hermitian
    eigenproblem, reverse communication driver.

    ``feast_complex_herm_solve`` is an iterative solver used to find
    some of the eigenvalues and the corresponding eigenvectors of a
    standard or generalized eigenvalue problem defined by complex
    Hermitian matrices.
    This is part of a suite of functions that also includes
    ``feast_init``, ``feast_option`` and ``feast_symm_contour``.

    For full information please refer to the NAG Library document for
    f12jr

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12jrf.html

    Parameters
    ----------
    handle : Handle
        The handle to the internal data structure used by the NAG FEAST
        suite. It needs to be initialized by ``feast_init``. **It must
        not be changed between calls to the NAG FEAST suite.**

    irevcm : int
        `On initial entry`: `irevcm` = 0, otherwise an error condition
        will be raised.

        `On intermediate entry`: must be unchanged from its previous
        exit value. Changing `irevcm` to any other value between calls
        will result in an error.

        Note: the matrices x, y and z referred to in this section are
        all of size `n`*`m0` and are stored in the arrays `x`, `y` and
        `z`, respectively.

    ze : complex
        `On initial entry`: need not be set.

    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    x : complex, shape (`n`, m0), modified in place
        `On initial entry`: need not be set.

        `On intermediate exit`:

            if `irevcm` = 5, the calling program must compute Az,
            storing the result in x prior to re-entry.

            If `irevcm` = 6, the calling program must compute Bz,
            storing the result in x prior to re-entry.

        Note: the matrices x and z are stored in the first `m0` columns
        of the arrays `x` and `z`, respectively.

    y : complex, shape (`n`, m0), modified in place
        `On initial entry`: need not be set.

        `On intermediate exit`:

            if `irevcm` = 2, the calling program must compute the
            solution to the linear system (`ze`B-A)w = y, overwriting y
            with the result w, prior to re-entry. The linear system has
            `m0` right-hand sides.

            If `irevcm` = 4, the calling program must compute the
            solution to the linear system (`ze`B-A)^Hw = y, overwriting
            y with the result w, prior to re-entry. The linear system
            has `m0` right-hand sides.

    m0 : int
        `On initial entry`: the size of the search subspace used to find
        the eigenvalues. This should exceed the number of eigenvalues
        within the search contour. See [Further Comments] for further
        details.

        `On intermediate entry`: `m0` must remain unchanged.

    d : float, shape (m0), modified in place
        `On initial entry`: if the option 'Subspace' = 'Yes' was set
        using the option setting function ``feast_option``, then `d`
        should contain an initial guess at the eigenvalues lying within
        the eigenvector search subspace (this subspace should be
        specified by `z`), otherwise `d` need not be set.

        `On final exit`: the first `nconv` entries in `d` contain the
        eigenvalues.

        Note: if the option 'Subspace' = 'Yes' was set using the option
        setting function ``feast_option``, then on final exit `d`
        contains an estimate of the eigenvalues after a single contour
        integral.

    z : complex, shape (`n`, m0), modified in place
        `On initial entry`: if the option 'Subspace' = 'Yes' was set
        using the option setting function ``feast_option``, then `z`
        should contain an initial guess at the eigenvector search
        subspace, otherwise `z` need not be set.

        `On intermediate exit`: must not be changed.

        `On final exit`: the first `nconv` columns of `z` contain the
        eigenvectors corresponding to the eigenvalues found within the
        contour.

        Note: if the option 'Execution Mode' = 'Subspace' was set using
        the option setting function ``feast_option``, then on final exit
        columns 1:`m0` of `z` contain the current search subspace after
        one contour integral.

    eps : float
        `On initial entry`: need not be set.

    itera : int
        `On initial entry`: need not be set.

    resid : float, shape (m0), modified in place
        `On initial entry`: need not be set.

        `On final exit`: for i = 1,...,`nconv`, `resid`[i-1] contains
        the relative residual, in the 1-norm, of the ith eigenpair
        found, that is `resid`[i-1] = ||Az_i-lambda_i
        Bz_i||/(||Bz_i||*max(|E_min|,|E_max|)), where E_min and E_max
        are the lower and upper bounds respectively of the eigenvalue
        search interval.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    irevcm : int
        `On intermediate exit`: has the following meanings.

        `irevcm` = 1

            The calling program must compute a factorization of the
            matrix `ze`B-A suitable for solving a linear system, for
            example using ``sparse.complex_gen_precon_ilu`` which
            computes an incomplete LU factorization of a complex sparse
            matrix. All arguments to the routine must remain unchanged.

            **Note:** the factorization can be computed in single
            precision.

        `irevcm` = 2

            The calling program must compute the solution to the linear
            system (`ze`B-A)w = y, overwriting y with the result w. The
            matrix `ze`B-A has previously been factorized (when `irevcm`
            = 1 was returned) and this factorization can be reused here.

            **Note:** the solve can be performed in single precision.

        `irevcm` = 3

            Optionally, the calling program must compute a factorization
            of the matrix (`ze`B-A)^H. This need only be done if it is
            not possible to use the factorization computed when `irevcm`
            = 1 was returned to solve linear systems involving
            (`ze`B-A)^H.

            **Note:** the solve can be performed in single precision.

        `irevcm` = 4

            The calling program must compute the solution to the linear
            system (`ze`B-A)^Hw = y, overwriting y with the result w. If
            it is not possible to use the factorization of `ze`B-A
            (computed when `irevcm` = 1 was returned) then the
            factorization of (`ze`B-A)^H (computed when `irevcm` = 2 was
            returned) should be used here.

        `irevcm` = 5

            The calling program must compute Az, storing the result in
            x.

        `irevcm` = 6

            The calling program must compute Bz, storing the result in
            x. If a standard eigenproblem is being solved (so that B =
            I) then the calling program should set x = z.

        `On final exit`: `irevcm` = 0: ``feast_complex_herm_solve`` has
        completed its tasks. The value of errno determines whether the
        iteration has been successfully completed, or whether errors
        have been detected.

        Note: the matrices x, y and z referred to in this section are
        all of size `n`*`m0` and are stored in the arrays `x`, `y` and
        `z`, respectively.

    ze : complex
        `On intermediate exit`: contains the current point on the
        contour.

            If `irevcm` = 1, then this must be used by the calling
            program to form a factorization of the matrix `ze`B-A.

            If `irevcm` = 3, then, optionally, this can be used to form
            a factorization of (`ze`B-A)^H.

    m0 : int
        If the initial search subspace was found by
        ``feast_complex_herm_solve`` to be too large, then a new smaller
        suitable choice is returned.

    nconv : int
        The number of eigenvalues found within the search contour.

        Note: if the option 'Execution Mode' = 'Estimate' was set in the
        option setting function ``feast_option``, then `nconv` contains
        a stochastic estimate of the number of eigenvalues within the
        search contour.

    eps : float
        The relative error on the trace. At iteration k, `eps` is given
        by the expression |trace_k-trace_{k-1}|/max(|E_min|,|E_max|),
        where trace_k is the sum of the eigenvalues found at the kth
        iteration, and E_min and E_max are the lower and upper bounds
        respectively of the eigenvalue search interval.

    itera : int
        The number of subspace iterations performed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Either the contour setting function ``feast_symm_contour``
            has not been called prior to the first call of this function
            or the supplied `handle` has become corrupted.

        (`errno` 7)
            An internal error occurred in the reduced eigenvalue solver.
            A possible cause is that the matrix B is not positive
            definite.

        (`errno` 8)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 9)
            On entry, `m0` = *<value>* and `n` = *<value>*.

            Constraint: 0 < `m0` <= `n`.

        (`errno` 13)
            On initial entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0.

        (`errno` 13)
            On intermediate entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 1, 2, 3, 4, 5 or 6.

        (`errno` 14)
            The option 'Subspace' = 'Yes' was set using the option
            setting function ``feast_option`` but no nonzero elements
            were found in the supplied subspace.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            No eigenvalues were found within the search contour.

        (`errno` 3)
            The function did not converge after the maximum number of
            iterations.

        (`errno` 4)
            The size of the eigenvector search subspace, `m0`, is too
            small.

        (`errno` 5)
            The option 'Execution Mode' = 'Subspace' was set using
            ``feast_option``. Columns 1:`m0` of `z` contain the search
            subspace after one contour integral and `d` contains an
            estimate of the eigenvalues.

        (`errno` 6)
            The option 'Execution Mode' = 'Estimate' was set using
            ``feast_option`` so only a stochastic estimate of the number
            of eigenvalues within the contour has been returned.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, and the corresponding eigenvectors, x, of a
    standard eigenvalue problem Ax = lambda x, or of a generalized
    eigenvalue problem Ax = lambda Bx, where the coefficient matrices A
    and B are sparse Hermitian, and B is positive definite.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale, dense, Hermitian problems.

    ``feast_complex_herm_solve`` is a **reverse communication**
    function, based on the FEAST eigensolver, described in Polizzi
    (2009), which finds eigenvalues using contour integration.
    Prior to calling ``feast_complex_herm_solve``, the contour
    definition function ``feast_symm_contour`` is used to specify a
    search interval on the real line, [E_min,E_max], within which
    eigenvalues will be sought (note that the eigenvalues of complex
    Hermitian eigenproblems are themselves real). ``feast_symm_contour``
    uses this interval to define nodes and weights for an elliptical
    contour to be used by ``feast_complex_herm_solve``.

    The setup function ``feast_init`` and the contour definition
    function ``feast_symm_contour`` must be called before
    ``feast_complex_herm_solve``.
    Between the calls to ``feast_init`` and ``feast_symm_contour``,
    options may be set by calls to the option setting function
    ``feast_option``.

    ``feast_complex_herm_solve`` uses **reverse communication**, i.e.,
    it returns repeatedly to the calling program with the argument
    `irevcm` (see [Parameters]) set to specified values which require
    the calling program to carry out one of the following tasks:

    -   compute a factorization of the matrix `ze`B-A, where `ze` is a
        point on the search contour;

    -   optionally, compute a factorization of the matrix (`ze`B-A)^H
        (this need only be done if the factorization `ze`B-A does not
        allow linear systems involving (`ze`B-A)^H to be solved);

    -   solve a linear system involving `ze`B-A or (`ze`B-A)^H, using
        the factorizations above;

    -   compute the matrix product x = Az;

    -   compute the matrix product x = Bz;

    -   notify the completion of the computation.

    The number of contour points, the number of iterations, and other
    options can all be set using the option setting function
    ``feast_option`` (see [Other Parameters for feast_option] for
    details on setting options and of the default settings).
    The search contour itself is defined by a call to
    ``feast_symm_contour``.

    References
    ----------
    Polizzi, E, 2009, `Density-Matrix-Based Algorithms for Solving
    Eigenvalue Problems`, Phys. Rev. B. (79), 115112
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.feast_complex_herm_solve'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12jrft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_HANDLE, # Input, handle.
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._BYREF_COMPLEX128, # Input/Output, ze.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x[ldx, *].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, y[ldy, *].
        utils._BYREF_INT, # Input, ldy.
        utils._BYREF_INT, # Input/Output, m0.
        utils._BYREF_INT, # Output, nconv.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[*].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._BYREF_FLOAT64, # Input/Output, eps.
        utils._BYREF_INT, # Input/Output, itera.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, resid[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (y, 'y'), (z, 'z')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _handle = utils._EngineCptrScalarType(
        handle, {'fun_name': fun_name, 'entity': 'handle'},
    ).to_ctypes()
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _ze = utils._EngineComplex128ScalarType(
        ze, {'fun_name': fun_name, 'entity': 'ze'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _ldx_py = utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _m0_out = utils._f12jzyn(handle, irevcm, m0)
    _m0_out_py = utils._EngineIntScalarType.to_py(_m0_out)
    _vu_f12jzy_m0_out_py = _m0_out_py
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ldy_py = utils._EngineComplex128ArrayType.get_nd(y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldy = utils._EngineIntScalarType(
        _ldy_py, {'fun_name': fun_name, 'entity': '_ldy'},
    ).to_ctypes()
    _y_cls = utils._EngineComplex128ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py, _vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _y = _y_cls.to_ctypes()
    _m0 = utils._EngineIntScalarType(
        m0, {'fun_name': fun_name, 'entity': 'm0'},
    ).to_ctypes()
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_vu_f12jzy_m0_out_py,),
    )
    _d = _d_cls.to_ctypes()
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=(_n_py, _vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _eps = utils._EngineFloat64ScalarType(
        eps, {'fun_name': fun_name, 'entity': 'eps'},
    ).to_ctypes()
    _itera = utils._EngineIntScalarType(
        itera, {'fun_name': fun_name, 'entity': 'itera'},
    ).to_ctypes()
    _resid_cls = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vu_f12jzy_m0_out_py,),
    )
    _resid = _resid_cls.to_ctypes()
    _routine_name = utils._EngineRoutineName(*b'f12jr ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _handle,
        _irevcm,
        _ze,
        _n,
        _x,
        _ldx,
        _y,
        _ldy,
        _m0,
        _nconv,
        _d,
        _z,
        _ldz,
        _eps,
        _itera,
        _resid,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the contour setting function feast_symm_contour has not been called prior',
                    'to the first call of this function or',
                    'the supplied handle has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No eigenvalues were found within the search contour.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function did not converge after the maximum number of iterations.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The size of the eigenvector search subspace, m0, is too small.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Execution Mode\' = \'Subspace\' was set using feast_option.',
                    'Columns 1:m0 of z contain the search subspace after one contour integral',
                    'and d contains an estimate of the eigenvalues.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Execution Mode\' = \'Estimate\' was set using feast_option so',
                    'only a stochastic estimate of the number of eigenvalues within the',
                    'contour has been returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred in the reduced eigenvalue solver. A possible',
                    'cause is that the matrix B is not positive definite.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m0 = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 0 < m0 <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 1000:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On initial entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On intermediate entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 1, 2, 3, 4, 5 or 6.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 14:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Subspace\' = \'Yes\' was set using the option setting function feast_option',
                    'but no nonzero elements were found in the supplied subspace.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ze_py = utils._EngineComplex128ScalarType.to_py(_ze)
        _m0_out_py = utils._EngineIntScalarType.to_py(_m0_out)
        _vu_f12jzy_m0_out_py = _m0_out_py
        _x_cls.refresh_py(_x)
        _y_cls.refresh_py(_y)
        _m0_py = utils._EngineIntScalarType.to_py(_m0)
        _d_cls.refresh_py(_d)
        _z_cls.refresh_py(_z)
        _eps_py = utils._EngineFloat64ScalarType.to_py(_eps)
        _itera_py = utils._EngineIntScalarType.to_py(_itera)
        _resid_cls.refresh_py(_resid)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['irevcm', 'ze', 'm0', 'nconv', 'eps', 'itera'],
        )(
            _irevcm_py,
            _ze_py,
            _m0_py,
            _nconv_py,
            _eps_py,
            _itera_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def feast_complex_symm_solve(handle, irevcm, ze, n, x, y, m0, d, z, eps, itera, resid, io_manager=None):
    u"""
    Selected eigenvalues and eigenvectors of a complex symmetric
    eigenproblem, reverse communication driver.

    ``feast_complex_symm_solve`` is an iterative solver used to find
    some of the eigenvalues and the corresponding eigenvectors of a
    standard or generalized eigenvalue problem defined by complex
    symmetric matrices.
    This is part of a suite of functions that also includes
    ``feast_init``, ``feast_option``, ``feast_gen_contour`` and
    ``feast_custom_contour``.

    For full information please refer to the NAG Library document for
    f12js

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12jsf.html

    Parameters
    ----------
    handle : Handle
        The handle to the internal data structure used by the NAG FEAST
        suite. It needs to be initialized by ``feast_init``. **It must
        not be changed between calls to the NAG FEAST suite.**

    irevcm : int
        `On initial entry`: `irevcm` = 0, otherwise an error condition
        will be raised.

        `On intermediate entry`: must be unchanged from its previous
        exit value. Changing `irevcm` to any other value between calls
        will result in an error.

        Note: the matrices x, y and z referred to in this section are
        all of size `n`*`m0` and are stored in the arrays `x`, `y` and
        `z`, respectively.

    ze : complex
        `On initial entry`: need not be set.

    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    x : complex, shape (`n`, m0), modified in place
        `On initial entry`: need not be set.

        `On intermediate exit`:

            if `irevcm` = 3, the calling program must compute Az,
            storing the result in x prior to re-entry.

            If `irevcm` = 4, the calling program must compute Bz,
            storing the result in x prior to re-entry.

        Note: the matrices x and z are stored in the first `m0` columns
        of the arrays `x` and `z`, respectively.

    y : complex, shape (`n`, m0), modified in place
        `On initial entry`: need not be set.

        `On intermediate exit`:

            if `irevcm` = 2, the calling program must compute the
            solution to the linear system (`ze`B-A)w = y, overwriting y
            with the result w, prior to re-entry. The linear system has
            `m0` right-hand sides.

    m0 : int
        `On initial entry`: the size of the search subspace used to find
        the eigenvalues. This should exceed the number of eigenvalues
        within the search contour. See [Further Comments] for further
        details.

        `On intermediate entry`: `m0` must remain unchanged.

    d : complex, shape (m0), modified in place
        `On initial entry`: if the option 'Subspace' = 'Yes' was set
        using the option setting function ``feast_option``, then `d`
        should contain an initial guess at the eigenvalues lying within
        the eigenvector search subspace (this subspace should be
        specified by `z`), otherwise `d` need not be set.

        `On final exit`: the first `nconv` entries in `d` contain the
        eigenvalues.

        Note: if the option 'Subspace' = 'Yes' was set using the option
        setting function ``feast_option``, then on final exit `d`
        contains an estimate of the eigenvalues after a single contour
        integral.

    z : complex, shape (`n`, m0), modified in place
        `On initial entry`: if the option 'Subspace' = 'Yes' was set
        using the option setting function ``feast_option``, then `z`
        should contain an initial guess at the eigenvector search
        subspace, otherwise `z` need not be set.

        `On intermediate exit`: must not be changed.

        `On final exit`: the first `nconv` columns of `z` contain the
        eigenvectors corresponding to the eigenvalues found within the
        contour.

        Note: if the option 'Execution Mode' = 'Subspace' was set using
        the option setting function ``feast_option``, then on final exit
        columns 1:`m0` of `z` contain the current search subspace after
        one contour integral.

    eps : float
        `On initial entry`: need not be set.

    itera : int
        `On initial entry`: need not be set.

    resid : float, shape (m0), modified in place
        `On initial entry`: need not be set.

        `On final exit`: for i = 1,...,`nconv`, `resid`[i-1] contains
        the relative residual, in the 1-norm, of the ith eigenpair
        found, that is `resid`[i-1] = ||Az_i-lambda_i
        Bz_i||/(||Bz_i||*(|E_mid|+r)), where E_mid is the centre of mass
        of the contour and r is the radius of the circle, centred at
        E_mid which just encloses the contour.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    irevcm : int
        `On intermediate exit`: has the following meanings.

        `irevcm` = 1

            The calling program must compute a factorization of the
            matrix `ze`B-A suitable for solving a linear system, for
            example using ``lapacklin.zsytrf`` which computes the
            Bunch--Kaufman factorization. All arguments to the routine
            must remain unchanged.

            **Note:** the factorization can be computed in single
            precision.

        `irevcm` = 2

            The calling program must compute the solution to the linear
            system (`ze`B-A)w = y, overwriting y with the result w. The
            matrix `ze`B-A has previously been factorized (when `irevcm`
            = 1 was returned) and this factorization can be reused here.

            **Note:** the solve can be performed in single precision.

        `irevcm` = 3

            The calling program must compute Az, storing the result in
            x.

        `irevcm` = 4

            The calling program must compute Bz, storing the result in
            x. If a standard eigenproblem is being solved (so that B =
            I) then the calling program should set x = z.

        `On final exit`: `irevcm` = 0: ``feast_complex_symm_solve`` has
        completed its tasks. The value of errno determines whether the
        iteration has been successfully completed, or whether errors
        have been detected.

        Note: the matrices x, y and z referred to in this section are
        all of size `n`*`m0` and are stored in the arrays `x`, `y` and
        `z`, respectively.

    ze : complex
        `On intermediate exit`: contains the current point on the
        contour.

            If `irevcm` = 1, then this must be used by the calling
            program to form a factorization of the matrix `ze`B-A.

    m0 : int
        If the initial search subspace was found by
        ``feast_complex_symm_solve`` to be too large, then a new smaller
        suitable choice is returned.

    nconv : int
        The number of eigenvalues found within the search contour.

        Note: if the option 'Execution Mode' = 'Estimate' was set in the
        option setting function ``feast_option``, then `nconv` contains
        a stochastic estimate of the number of eigenvalues within the
        search contour.

    eps : float
        The relative error on the trace. At iteration k, `eps` is given
        by the expression |trace_k-trace_{k-1}|/(|E_mid|+r), where
        trace_k is the sum of the eigenvalues found at the kth
        iteration, E_mid is the centre of mass of the contour and r is
        the radius of the circle, centred at E_mid which just encloses
        the contour.

    itera : int
        The number of subspace iterations performed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Either the contour setting function ``feast_symm_contour``
            has not been called prior to the first call of this function
            or the supplied `handle` has become corrupted.

        (`errno` 7)
            An internal error occurred in the reduced eigenvalue solver.
            A possible cause is that the matrix B is not positive
            definite.

        (`errno` 8)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 9)
            On entry, `m0` = *<value>* and `n` = *<value>*.

            Constraint: 0 < `m0` <= `n`.

        (`errno` 13)
            On initial entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0.

        (`errno` 13)
            On intermediate entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 1, 2, 3 or 4.

        (`errno` 14)
            The function converged but the eigenvector subspace is not
            orthonormal.

        (`errno` 15)
            The option 'Subspace' = 'Yes' was set using the option
            setting function ``feast_option`` but no nonzero elements
            were found in the supplied subspace.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            No eigenvalues were found within the search contour.

        (`errno` 3)
            The function did not converge after the maximum number of
            iterations.

        (`errno` 4)
            The size of the eigenvector search subspace, `m0`, is too
            small.

        (`errno` 5)
            The option 'Execution Mode' = 'Subspace' was set using
            ``feast_option``. Columns 1:`m0` of `z` contain the search
            subspace after one contour integral and `d` contains an
            estimate of the eigenvalues.

        (`errno` 6)
            The option 'Execution Mode' = 'Estimate' was set using
            ``feast_option`` so only a stochastic estimate of the number
            of eigenvalues within the contour has been returned.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, and the corresponding eigenvectors, x, of a
    standard eigenvalue problem Ax = lambda x, or of a generalized
    eigenvalue problem Ax = lambda Bx, where the coefficient matrices A
    and B are sparse, complex and symmetric.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale, dense, symmetric problems.

    ``feast_complex_symm_solve`` is a **reverse communication**
    function, based on the FEAST eigensolver, described in Polizzi
    (2009), which finds eigenvalues using contour integration.
    Prior to calling ``feast_complex_symm_solve``, one of the contour
    definition functions ``feast_gen_contour`` or
    ``feast_custom_contour`` must be used to define nodes and weights
    for a contour around a region in the complex plane within which
    eigenvalues will be sought.

    The setup function ``feast_init`` and the contour definition
    function ``feast_gen_contour`` or ``feast_custom_contour`` must be
    called before ``feast_complex_symm_solve``.
    Between the calls to ``feast_init`` and ``feast_gen_contour`` or
    ``feast_custom_contour``, options may be set by calls to the option
    setting function ``feast_option``.

    ``feast_complex_symm_solve`` uses **reverse communication**, i.e.,
    it returns repeatedly to the calling program with the argument
    `irevcm` (see [Parameters]) set to specified values which require
    the calling program to carry out one of the following tasks:

    -   compute a factorization of the matrix `ze`B-A, where `ze` is a
        point on the search contour;

    -   solve a linear system involving `ze`B-A, using the factorization
        above;

    -   compute the matrix product x = Az;

    -   compute the matrix product x = Bz;

    -   notify the completion of the computation.

    The number of contour points, the number of iterations, and other
    options can all be set using the option setting function
    ``feast_option`` (see [Other Parameters for feast_option] for
    details on setting options and of the default settings).
    The search contour itself is defined by a call to
    ``feast_gen_contour`` or ``feast_custom_contour``.

    References
    ----------
    Polizzi, E, 2009, `Density-Matrix-Based Algorithms for Solving
    Eigenvalue Problems`, Phys. Rev. B. (79), 115112
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.feast_complex_symm_solve'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12jsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_HANDLE, # Input, handle.
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._BYREF_COMPLEX128, # Input/Output, ze.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x[ldx, *].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, y[ldy, *].
        utils._BYREF_INT, # Input, ldy.
        utils._BYREF_INT, # Input/Output, m0.
        utils._BYREF_INT, # Output, nconv.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, d[*].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._BYREF_FLOAT64, # Input/Output, eps.
        utils._BYREF_INT, # Input/Output, itera.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, resid[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (y, 'y'), (z, 'z')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _handle = utils._EngineCptrScalarType(
        handle, {'fun_name': fun_name, 'entity': 'handle'},
    ).to_ctypes()
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _ze = utils._EngineComplex128ScalarType(
        ze, {'fun_name': fun_name, 'entity': 'ze'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _ldx_py = utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _m0_out = utils._f12jzyn(handle, irevcm, m0)
    _m0_out_py = utils._EngineIntScalarType.to_py(_m0_out)
    _vu_f12jzy_m0_out_py = _m0_out_py
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ldy_py = utils._EngineComplex128ArrayType.get_nd(y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldy = utils._EngineIntScalarType(
        _ldy_py, {'fun_name': fun_name, 'entity': '_ldy'},
    ).to_ctypes()
    _y_cls = utils._EngineComplex128ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py, _vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _y = _y_cls.to_ctypes()
    _m0 = utils._EngineIntScalarType(
        m0, {'fun_name': fun_name, 'entity': 'm0'},
    ).to_ctypes()
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    _d_cls = utils._EngineComplex128ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_vu_f12jzy_m0_out_py,),
    )
    _d = _d_cls.to_ctypes()
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=(_n_py, _vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _eps = utils._EngineFloat64ScalarType(
        eps, {'fun_name': fun_name, 'entity': 'eps'},
    ).to_ctypes()
    _itera = utils._EngineIntScalarType(
        itera, {'fun_name': fun_name, 'entity': 'itera'},
    ).to_ctypes()
    _resid_cls = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vu_f12jzy_m0_out_py,),
    )
    _resid = _resid_cls.to_ctypes()
    _routine_name = utils._EngineRoutineName(*b'f12js ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _handle,
        _irevcm,
        _ze,
        _n,
        _x,
        _ldx,
        _y,
        _ldy,
        _m0,
        _nconv,
        _d,
        _z,
        _ldz,
        _eps,
        _itera,
        _resid,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the contour setting function feast_symm_contour has not been called prior',
                    'to the first call of this function or',
                    'the supplied handle has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No eigenvalues were found within the search contour.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function did not converge after the maximum number of iterations.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The size of the eigenvector search subspace, m0, is too small.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Execution Mode\' = \'Subspace\' was set using feast_option.',
                    'Columns 1:m0 of z contain the search subspace after one contour integral',
                    'and d contains an estimate of the eigenvalues.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Execution Mode\' = \'Estimate\' was set using feast_option so',
                    'only a stochastic estimate of the number of eigenvalues within the',
                    'contour has been returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred in the reduced eigenvalue solver. A possible',
                    'cause is that the matrix B is not positive definite.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m0 = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 0 < m0 <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 1000:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On initial entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On intermediate entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 1, 2, 3 or 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function converged but the eigenvector subspace is not orthonormal.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 15:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Subspace\' = \'Yes\' was set using the option setting function feast_option',
                    'but no nonzero elements were found in the supplied subspace.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ze_py = utils._EngineComplex128ScalarType.to_py(_ze)
        _m0_out_py = utils._EngineIntScalarType.to_py(_m0_out)
        _vu_f12jzy_m0_out_py = _m0_out_py
        _x_cls.refresh_py(_x)
        _y_cls.refresh_py(_y)
        _m0_py = utils._EngineIntScalarType.to_py(_m0)
        _d_cls.refresh_py(_d)
        _z_cls.refresh_py(_z)
        _eps_py = utils._EngineFloat64ScalarType.to_py(_eps)
        _itera_py = utils._EngineIntScalarType.to_py(_itera)
        _resid_cls.refresh_py(_resid)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['irevcm', 'ze', 'm0', 'nconv', 'eps', 'itera'],
        )(
            _irevcm_py,
            _ze_py,
            _m0_py,
            _nconv_py,
            _eps_py,
            _itera_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def feast_complex_gen_solve(handle, irevcm, ze, n, x, y, m0, d, z, eps, itera, resid, io_manager=None):
    u"""
    Selected eigenvalues and eigenvectors of a complex non-Hermitian
    eigenproblem, reverse communication driver.

    ``feast_complex_gen_solve`` is an iterative solver used to find some
    of the eigenvalues and the corresponding eigenvectors of a standard
    or generalized eigenvalue problem defined by complex nonsymmetric
    matrices.
    This is part of a suite of functions that also includes
    ``feast_init``, ``feast_option``, ``feast_gen_contour`` and
    ``feast_custom_contour``.

    For full information please refer to the NAG Library document for
    f12jt

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12jtf.html

    Parameters
    ----------
    handle : Handle
        The handle to the internal data structure used by the NAG FEAST
        suite. It needs to be initialized by ``feast_init``. **It must
        not be changed between calls to the NAG FEAST suite.**

    irevcm : int
        `On initial entry`: `irevcm` = 0, otherwise an error condition
        will be raised.

        `On intermediate entry`: must be unchanged from its previous
        exit value. Changing `irevcm` to any other value between calls
        will result in an error.

        Note: the matrices x, y and z referred to in this section are
        all of size `n`*`m0` and are stored in the arrays `x`, `y` and
        `z`, respectively.

    ze : complex
        `On initial entry`: need not be set.

    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    x : complex, shape (`n`, 2*m0), modified in place
        `On initial entry`: need not be set.

        `On intermediate exit`:

            if `irevcm` = 5, the calling program must compute Az,
            storing the result in x prior to re-entry.

            If `irevcm` = 6, the calling program must compute A^Hz,
            storing the result in x prior to re-entry.

            If `irevcm` = 7, the calling program must compute Bz,
            storing the result in x prior to re-entry.

            If `irevcm` = 8, the calling program must compute B^Hz,
            storing the result in x prior to re-entry.

        Note: the matrices x and z are stored in the first `m0` columns
        of the arrays `x` and `z`, respectively.

    y : complex, shape (`n`, m0), modified in place
        `On initial entry`: need not be set.

        `On intermediate exit`:

            if `irevcm` = 2, the calling program must compute the
            solution to the linear system (`ze`B-A)w = y, overwriting y
            with the result w, prior to re-entry. The linear system has
            `m0` right-hand sides.

            If `irevcm` = 4, the calling program must compute the
            solution to the linear system (`ze`B-A)^Hw = y, overwriting
            y with the result w, prior to re-entry. The linear system
            has `m0` right-hand sides.

    m0 : int
        `On initial entry`: the size of the search subspace used to find
        the eigenvalues. This should exceed the number of eigenvalues
        within the search contour. See [Further Comments] for further
        details.

        `On intermediate entry`: `m0` must remain unchanged.

    d : complex, shape (m0), modified in place
        `On initial entry`: if the option 'Subspace' = 'Yes' was set
        using the option setting function ``feast_option``, then `d`
        should contain an initial guess at the eigenvalues lying within
        the eigenvector search subspace (this subspace should be
        specified by `z`), otherwise `d` need not be set.

        `On final exit`: the first `nconv` entries in `d` contain the
        eigenvalues.

        Note: if the option 'Subspace' = 'Yes' was set using the option
        setting function ``feast_option``, then on final exit `d`
        contains an estimate of the eigenvalues after a single contour
        integral.

    z : complex, shape (`n`, 2*m0), modified in place
        `On initial entry`: if the option 'Subspace' = 'Yes' was set
        using the option setting function ``feast_option``, then `z`
        should contain an initial guess at the eigenvector search
        subspace, otherwise `z` need not be set.

        Note: if the option 'Subspace' = 'Yes' was set but only right
        eigenvectors are to be computed, then only the first `m0`
        columns of `z` need to be set. However, if the option
        'Eigenvectors' = 'Two-sided' was set using ``feast_option``, so
        that both left and right eigenvectors are returned, then the
        first `m0` columns of `z` should contain the initial guess for
        the right eigenvector search subspace and the remaining `m0`
        columns should contain an initial guess for the left eigenvector
        search subspace.

        `On intermediate exit`: must not be changed.

        `On final exit`: the first `nconv` columns of `z` contain the
        right eigenvectors corresponding to the eigenvalues found within
        the contour. If the option 'Eigenvectors' = 'Two-sided' was set
        in the option setting function ``feast_option``, the columns
        `m0`+1:`m0`+`nconv` of `z` contain the left eigenvectors
        corresponding to the eigenvalues found within the contour.

        Note: if the option 'Execution Mode' = 'Subspace' was set using
        the option setting function ``feast_option``, then on final exit
        columns 1:`m0` of `z` contain the current search subspace after
        one contour integral.

    eps : float
        `On initial entry`: need not be set.

    itera : int
        `On initial entry`: need not be set.

    resid : float, shape (2*m0), modified in place
        `On initial entry`: need not be set.

        `On final exit`: for i = 1,...,`nconv`, `resid`[i-1] contains
        the relative residual, in the 1-norm, of the ith eigenpair
        found, that is `resid`[i-1] = ||Az_i-lambda_i
        Bz_i||/(||Bz_i||*(|E_mid|+r)), where E_mid is the centre of mass
        of the contour and r is the radius of the circle, centred at
        E_mid which just encloses the contour.

        If the option 'Eigenvectors' = 'Two-sided' was set in the option
        setting function ``feast_option``, the entries
        `m0`+1:`m0`+`nconv` of `resid` contain corresponding residuals
        for the left eigenvectors, `resid`[i-1] = ||A^Hz_i-lambda__i
        B^Hz_i||/(||B^Hz_i||*(|E_mid|+r)).

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    irevcm : int
        `On intermediate exit`: has the following meanings.

        `irevcm` = 1

            The calling program must compute a factorization of the
            matrix `ze`B-A suitable for solving a linear system, for
            example using ``sparse.complex_gen_precon_ilu`` which
            computes an incomplete LU factorization of a complex sparse
            matrix. All arguments to the routine must remain unchanged.

            **Note:** the factorization can be computed in single
            precision.

        `irevcm` = 2

            The calling program must compute the solution to the linear
            system (`ze`B-A)w = y, overwriting y with the result w. The
            matrix `ze`B-A has previously been factorized (when `irevcm`
            = 1 was returned) and this factorization can be reused here.

            **Note:** the solve can be performed in single precision.

        `irevcm` = 3

            Optionally, the calling program must compute a factorization
            of the matrix (`ze`B-A)^T. This need only be done if it is
            not possible to use the factorization computed when `irevcm`
            = 1 was returned to solve linear systems involving
            (`ze`B-A)^T. If this factorization is to be computed, then
            the factorization from `irevcm` = 1 must not be overwritten.

            **Note:** the factorization can be performed in single
            precision.

        `irevcm` = 4

            The calling program must compute the solution to the linear
            system (`ze`B-A)^Tw = y, overwriting y with the result w. If
            it is not possible to use the factorization of `ze`B-A
            (computed when `irevcm` = 1 was returned) then the
            factorization of (`ze`B-A)^T (computed when `irevcm` = 2 was
            returned) should be used here.

            **Note:** the solve can be performed in single precision.

        `irevcm` = 5

            The calling program must compute Az, storing the result in
            x.

        `irevcm` = 6

            The calling program must compute A^Tz, storing the result in
            x.

        `irevcm` = 7

            The calling program must compute Bz, storing the result in
            x. If a standard eigenproblem is being solved (so that B =
            I) then the calling program should set x = z.

        `irevcm` = 8

            The calling program must compute B^Tz, storing the result in
            x. If a standard eigenproblem is being solved (so that B =
            I) then the calling program should set x = z.

        `On final exit`: `irevcm` = 0: ``feast_complex_gen_solve`` has
        completed its tasks. The value of errno determines whether the
        iteration has been successfully completed, or whether errors
        have been detected.

        Note: the matrices x, y and z referred to in this section are
        all of size `n`*`m0` and are stored in the arrays `x`, `y` and
        `z`, respectively.

    ze : complex
        `On intermediate exit`: contains the current point on the
        contour.

            If `irevcm` = 1, then this must be used by the calling
            program to form a factorization of the matrix `ze`B-A.

            If `irevcm` = 3, then, optionally, this can be used to form
            a factorization of (`ze`B-A)^H.

    m0 : int
        If the initial search subspace was found by
        ``feast_complex_gen_solve`` to be too large, then a new smaller
        suitable choice is returned.

    nconv : int
        The number of eigenvalues found within the search contour.

        Note: if the option 'Execution Mode' = 'Estimate' was set in the
        option setting function ``feast_option``, then `nconv` contains
        a stochastic estimate of the number of eigenvalues within the
        search contour.

    eps : float
        The relative error on the trace. At iteration k, `eps` is given
        by the expression |trace_k-trace_{k-1}|/(|E_mid|+r), where
        trace_k is the sum of the eigenvalues found at the kth
        iteration, E_mid is the centre of mass of the contour and r is
        the radius of the circle, centred at E_mid which just encloses
        the contour.

    itera : int
        The number of subspace iterations performed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Either one of the contour setting functions
            ``feast_gen_contour`` or ``feast_custom_contour`` has not
            been called prior to the first call of this function or the
            supplied `handle` has become corrupted.

        (`errno` 7)
            An internal error occurred in the reduced eigenvalue solver.
            Please contact NAG.

        (`errno` 8)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 9)
            On entry, `m0` = *<value>* and `n` = *<value>*.

            Constraint: 0 < `m0` <= `n`.

        (`errno` 13)
            On initial entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0.

        (`errno` 13)
            On intermediate entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 1, 2, 3, 4, 5, 6, 7 or 8.

        (`errno` 14)
            The function converged but the left/right eigenvector
            subspaces are not bi-orthonormal.

        (`errno` 14)
            The function converged but the right eigenvector subspace is
            not orthonormal.

        (`errno` 15)
            The option 'Subspace' = 'Yes' was set using the option
            setting function ``feast_option`` but no nonzero elements
            were found in the supplied subspace.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            No eigenvalues were found within the search contour.

        (`errno` 3)
            The function did not converge after the maximum number of
            iterations.

        (`errno` 4)
            The size of the eigenvector search subspace, `m0`, is too
            small.

        (`errno` 5)
            The option 'Execution Mode' = 'Subspace' was set using
            ``feast_option``. Columns 1:`m0` of `z` contain the search
            subspace after one contour integral and `d` contains an
            estimate of the eigenvalues.

        (`errno` 6)
            The option 'Execution Mode' = 'Estimate' was set using
            ``feast_option`` so only a stochastic estimate of the number
            of eigenvalues within the contour has been returned.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, and the corresponding eigenvectors, x, of a
    standard eigenvalue problem Ax = lambda x, or of a generalized
    eigenvalue problem Ax = lambda Bx, where the coefficient matrices A
    and B are sparse, complex and nonsymmetric.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale, dense problems.

    ``feast_complex_gen_solve`` is a **reverse communication** function,
    based on the FEAST eigensolver, described in Polizzi (2009), which
    finds eigenvalues using contour integration.
    Prior to calling ``feast_complex_gen_solve``, one of the contour
    definition functions ``feast_gen_contour`` or
    ``feast_custom_contour`` must be used to define nodes and weights
    for a contour around a region in the complex plane within which
    eigenvalues will be sought.

    The setup function ``feast_init`` and one of the contour definition
    functions ``feast_gen_contour`` or ``feast_custom_contour`` must be
    called before ``feast_complex_gen_solve``.
    Between the calls to ``feast_init`` and ``feast_gen_contour`` or
    ``feast_custom_contour``, options may be set by calls to the option
    setting function ``feast_option``.

    ``feast_complex_gen_solve`` uses **reverse communication**, i.e., it
    returns repeatedly to the calling program with the argument `irevcm`
    (see [Parameters]) set to specified values which require the calling
    program to carry out one of the following tasks:

    -   compute a factorization of the matrix `ze`B-A, where `ze` is a
        point on the search contour;

    -   optionally, compute a factorization of the matrix (`ze`B-A)^H
        (this need only be done if the factorization `ze`B-A does not
        allow linear systems involving (`ze`B-A)^H to be solved);

    -   solve a linear system involving `ze`B-A or (`ze`B-A)^H, using
        the factorizations above;

    -   compute the matrix product x = Az;

    -   compute the matrix product x = Bz;

    -   compute the matrix product x = A^Hz;

    -   compute the matrix product x = B^Hz;

    -   notify the completion of the computation.

    The number of contour points, the number of iterations, and other
    options can all be set using the option setting function
    ``feast_option`` (see [Other Parameters for feast_option] for
    details on setting options and of the default settings).
    The search contour itself is defined by a call to either
    ``feast_gen_contour`` (for elliptical contours) or
    ``feast_custom_contour`` (for general contours).

    References
    ----------
    Polizzi, E, 2009, `Density-Matrix-Based Algorithms for Solving
    Eigenvalue Problems`, Phys. Rev. B. (79), 115112
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.feast_complex_gen_solve'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12jtft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_HANDLE, # Input, handle.
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._BYREF_COMPLEX128, # Input/Output, ze.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x[ldx, *].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, y[ldy, *].
        utils._BYREF_INT, # Input, ldy.
        utils._BYREF_INT, # Input/Output, m0.
        utils._BYREF_INT, # Output, nconv.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, d[*].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._BYREF_FLOAT64, # Input/Output, eps.
        utils._BYREF_INT, # Input/Output, itera.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, resid[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (y, 'y'), (z, 'z')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _handle = utils._EngineCptrScalarType(
        handle, {'fun_name': fun_name, 'entity': 'handle'},
    ).to_ctypes()
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _ze = utils._EngineComplex128ScalarType(
        ze, {'fun_name': fun_name, 'entity': 'ze'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _ldx_py = utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _m0_out = utils._f12jzyn(handle, irevcm, m0)
    _m0_out_py = utils._EngineIntScalarType.to_py(_m0_out)
    _vu_f12jzy_m0_out_py = _m0_out_py
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, 2*_vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ldy_py = utils._EngineComplex128ArrayType.get_nd(y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldy = utils._EngineIntScalarType(
        _ldy_py, {'fun_name': fun_name, 'entity': '_ldy'},
    ).to_ctypes()
    _y_cls = utils._EngineComplex128ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py, _vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _y = _y_cls.to_ctypes()
    _m0 = utils._EngineIntScalarType(
        m0, {'fun_name': fun_name, 'entity': 'm0'},
    ).to_ctypes()
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    _d_cls = utils._EngineComplex128ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_vu_f12jzy_m0_out_py,),
    )
    _d = _d_cls.to_ctypes()
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=(_n_py, 2*_vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _eps = utils._EngineFloat64ScalarType(
        eps, {'fun_name': fun_name, 'entity': 'eps'},
    ).to_ctypes()
    _itera = utils._EngineIntScalarType(
        itera, {'fun_name': fun_name, 'entity': 'itera'},
    ).to_ctypes()
    _resid_cls = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(2*_vu_f12jzy_m0_out_py,),
    )
    _resid = _resid_cls.to_ctypes()
    _routine_name = utils._EngineRoutineName(*b'f12jt ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _handle,
        _irevcm,
        _ze,
        _n,
        _x,
        _ldx,
        _y,
        _ldy,
        _m0,
        _nconv,
        _d,
        _z,
        _ldz,
        _eps,
        _itera,
        _resid,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either one of the contour setting functions feast_gen_contour or feast_custom_contour has not',
                    'been called prior to the first call of this function or',
                    'the supplied handle has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No eigenvalues were found within the search contour.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function did not converge after the maximum number of iterations.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The size of the eigenvector search subspace, m0, is too small.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Execution Mode\' = \'Subspace\' was set using feast_option.',
                    'Columns 1:m0 of z contain the search subspace after one contour integral',
                    'and d contains an estimate of the eigenvalues.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Execution Mode\' = \'Estimate\' was set using feast_option so',
                    'only a stochastic estimate of the number of eigenvalues within the',
                    'contour has been returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred in the reduced eigenvalue solver.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m0 = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 0 < m0 <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 1000:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On initial entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On intermediate entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 1, 2, 3, 4, 5, 6, 7 or 8.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 1000:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function converged but the left/right eigenvector subspaces are not',
                    'bi-orthonormal.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function converged but the right eigenvector subspace is not',
                    'orthonormal.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 15:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Subspace\' = \'Yes\' was set using the option setting function feast_option',
                    'but no nonzero elements were found in the supplied subspace.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ze_py = utils._EngineComplex128ScalarType.to_py(_ze)
        _m0_out_py = utils._EngineIntScalarType.to_py(_m0_out)
        _vu_f12jzy_m0_out_py = _m0_out_py
        _x_cls.refresh_py(_x)
        _y_cls.refresh_py(_y)
        _m0_py = utils._EngineIntScalarType.to_py(_m0)
        _d_cls.refresh_py(_d)
        _z_cls.refresh_py(_z)
        _eps_py = utils._EngineFloat64ScalarType.to_py(_eps)
        _itera_py = utils._EngineIntScalarType.to_py(_itera)
        _resid_cls.refresh_py(_resid)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['irevcm', 'ze', 'm0', 'nconv', 'eps', 'itera'],
        )(
            _irevcm_py,
            _ze_py,
            _m0_py,
            _nconv_py,
            _eps_py,
            _itera_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def feast_poly_symm_solve(handle, irevcm, deg, ze, n, x, y, m0, d, z, eps, itera, resid, io_manager=None):
    u"""
    Selected eigenvalues and eigenvectors of a symmetric polynomial
    eigenproblem, reverse communication driver.

    ``feast_poly_symm_solve`` is an iterative solver used to find some
    of the eigenvalues and the corresponding eigenvectors of a
    polynomial eigenvalue problem defined by complex symmetric matrices.
    This is part of a suite of functions that also includes
    ``feast_init``, ``feast_option``, ``feast_gen_contour`` and
    ``feast_custom_contour``.

    For full information please refer to the NAG Library document for
    f12ju

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12juf.html

    Parameters
    ----------
    handle : Handle
        The handle to the internal data structure used by the NAG FEAST
        suite. It needs to be initialized by ``feast_init``. **It must
        not be changed between calls to the NAG FEAST suite.**

    irevcm : int
        `On initial entry`: `irevcm` = 0, otherwise an error condition
        will be raised.

        `On intermediate entry`: must be unchanged from its previous
        exit value. Changing `irevcm` to any other value between calls
        will result in an error.

        Note: the matrices x, y and z referred to in this section are
        all of size `n`*`m0` and are stored in the arrays `x`, `y` and
        `z`, respectively.

    deg : int
        The degree, p, of the polynomial eigenvalue problem.

    ze : complex
        `On initial entry`: need not be set.

    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    x : complex, shape (`n`, m0), modified in place
        `On initial entry`: need not be set.

        `On intermediate exit`:

            if `irevcm` = 3, the calling program must compute A_k z,
            storing the result in x prior to re-entry.

        Note: the matrices x, y and z referred to in this section are
        all of size `n`*`m0` and are stored in the arrays `x`, `y` and
        `z`, respectively.

        Note: the matrices x and z are stored in the first `m0` columns
        of the arrays `x` and `z`, respectively.

    y : complex, shape (`n`, m0), modified in place
        `On initial entry`: need not be set.

        `On intermediate exit`:

            if `irevcm` = 2, the calling program must compute the
            solution to the linear system (∑_i = 0^p`ze`^iA_i)w = y,
            overwriting y with the result w, prior to re-entry. The
            linear system has `m0` right-hand sides.

    m0 : int
        `On initial entry`: the size of the search subspace used to find
        the eigenvalues. This should exceed the number of eigenvalues
        within the search contour. See [Further Comments] for further
        details.

        `On intermediate entry`: `m0` must remain unchanged.

    d : complex, shape (m0), modified in place
        `On initial entry`: if the option 'Subspace' = 'Yes' was set
        using the option setting function ``feast_option``, then `d`
        should contain an initial guess at the eigenvalues lying within
        the eigenvector search subspace (this subspace should be
        specified by `z`), otherwise `d` need not be set.

        `On final exit`: the first `nconv` entries in `d` contain the
        eigenvalues.

        Note: if the option 'Subspace' = 'Yes' was set using the option
        setting function ``feast_option``, then on final exit `d`
        contains an estimate of the eigenvalues after a single contour
        integral.

    z : complex, shape (`n`, m0), modified in place
        `On initial entry`: if the option 'Subspace' = 'Yes' was set
        using the option setting function ``feast_option``, then `z`
        should contain an initial guess at the eigenvector search
        subspace, otherwise `z` need not be set.

        `On intermediate exit`: must not be changed.

        `On final exit`: the first `nconv` columns of `z` contain the
        eigenvectors corresponding to the eigenvalues found within the
        contour.

        Note: if the option 'Execution Mode' = 'Subspace' was set using
        the option setting function ``feast_option``, then on final exit
        columns 1:`m0` of `z` contain the current search subspace after
        one contour integral.

    eps : float
        `On initial entry`: need not be set.

    itera : int
        `On initial entry`: need not be set.

    resid : float, shape (m0), modified in place
        `On initial entry`: need not be set.

        `On final exit`: for i = 1,...,`nconv`, `resid`[i-1] contains
        the relative residual, in the 1-norm, of the ith eigenpair
        found, that is `resid`[i-1] = ||(∑_j = 0^p
        lambda_i^jA_j)z_i||/(||lambda^pA_p z_i||*(|E_mid|+r)), where
        E_mid is the centre of mass of the contour and r is the radius
        of the circle, centred at E_mid which just encloses the contour.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    irevcm : int
        `On intermediate exit`: has the following meanings.

        `irevcm` = 1

            The calling program must compute a factorization of the
            matrix ∑_i = 0^p`ze`^iA_i suitable for solving a linear
            system, for example using ``sparse.complex_gen_precon_ilu``
            which computes an incomplete LU factorization of a complex
            sparse matrix. All arguments to the routine must remain
            unchanged.

            **Note:** the factorization can be computed in single
            precision.

        `irevcm` = 2

            The calling program must compute the solution to the linear
            system (∑_i = 0^p`ze`^iA_i)w = y, overwriting y with the
            result w. The matrix ∑_i = 0^p`ze`^iA_i has previously been
            factorized (when `irevcm` = 1 was returned) and this
            factorization can be reused here.

            **Note:** the solve can be performed in single precision.

        `irevcm` = 3

            The calling program must compute A_k z, storing the result
            in x.

        `On final exit`: `irevcm` = 0: ``feast_poly_symm_solve`` has
        completed its tasks. The value of errno determines whether the
        iteration has been successfully completed, or whether errors
        have been detected.

        Note: the matrices x, y and z referred to in this section are
        all of size `n`*`m0` and are stored in the arrays `x`, `y` and
        `z`, respectively.

    ze : complex
        `On intermediate exit`: contains the current point on the
        contour.

            If `irevcm` = 1, then this must be used by the calling
            program to form a factorization of the matrix ∑_i =
            0^p`ze`^iA_i.

    k : int
        `On intermediate exit`:

            if `irevcm` = 3, denotes which of the coefficient matrices
            A_k needs to be used to form A_k z.

    m0 : int
        If the initial search subspace was found by
        ``feast_poly_symm_solve`` to be too large, then a new smaller
        suitable choice is returned.

    nconv : int
        The number of eigenvalues found within the search contour.

    eps : float
        The relative error on the trace. At iteration k, `eps` is given
        by the expression |trace_k-trace_{k-1}|/(|E_mid|+r), where
        trace_k is the sum of the eigenvalues found at the kth
        iteration, E_mid is the centre of mass of the contour and r is
        the radius of the circle, centred at E_mid which just encloses
        the contour.

    itera : int
        The number of subspace iterations performed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Either the contour setting function ``feast_symm_contour``
            has not been called prior to the first call of this function
            or the supplied `handle` has become corrupted.

        (`errno` 7)
            An internal error occurred in the reduced eigenvalue solver.
            Please contact NAG.

        (`errno` 8)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 9)
            On entry, `m0` = *<value>* and `n` = *<value>*.

            Constraint: 0 < `m0` <= `n`.

        (`errno` 13)
            On initial entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0.

        (`errno` 13)
            On intermediate entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 1, 2, 3, 4, 5 or 6.

        (`errno` 14)
            On entry, `deg` = *<value>*.

            Constraint: `deg` > 0.

        (`errno` 15)
            The option 'Subspace' = 'Yes' was set using the option
            setting function ``feast_option`` but no nonzero elements
            were found in the supplied subspace.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            No eigenvalues were found within the search contour.

        (`errno` 3)
            The function did not converge after the maximum number of
            iterations.

        (`errno` 4)
            The size of the eigenvector search subspace, `m0`, is too
            small.

        (`errno` 5)
            The option 'Execution Mode' = 'Subspace' was set using
            ``feast_option``. Columns 1:`m0` of `z` contain the search
            subspace after one contour integral and `d` contains an
            estimate of the eigenvalues.

        (`errno` 6)
            The option 'Execution Mode' = 'Estimate' was set using
            ``feast_option``. `nconv` contains a stochastic estimate of
            the number of eigenvalues within the contour.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, and the corresponding eigenvectors, x, of a
    polynomial eigenvalue problem ∑_i = 0^p lambda^iA_i x = 0 where the
    coefficient matrices A_i are sparse, complex and symmetric.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale, dense problems.

    ``feast_poly_symm_solve`` is a **reverse communication** function,
    based on the FEAST eigensolver, described in Polizzi (2009), which
    finds eigenvalues using contour integration.
    Prior to calling ``feast_poly_symm_solve``, one of the contour
    definition functions ``feast_gen_contour`` or
    ``feast_custom_contour`` must be used to define nodes and weights
    for a contour around a region in the complex plane within which
    eigenvalues will be sought. ``feast_gen_contour`` or
    ``feast_custom_contour`` uses this interval to define nodes and
    weights for the contour to be used by ``feast_poly_symm_solve``.

    The setup function ``feast_init`` and one of the contour definition
    functions ``feast_gen_contour`` or ``feast_custom_contour`` must be
    called before ``feast_poly_symm_solve``.
    Between the calls to ``feast_init`` and ``feast_gen_contour`` or
    ``feast_custom_contour``, options may be set by calls to the option
    setting function ``feast_option``.

    ``feast_poly_symm_solve`` uses **reverse communication**, i.e., it
    returns repeatedly to the calling program with the argument `irevcm`
    (see [Parameters]) set to specified values which require the calling
    program to carry out one of the following tasks:

    -   compute a factorization of the matrix ∑_i = 0^p`ze`^iA_i, where
        `ze` is a point on the search contour;

    -   solve a linear system involving ∑_i = 0^p`ze`^iA_i using the
        factorizations above;

    -   compute the matrix product x = A_k z, for a given value of k;

    -   notify the completion of the computation.

    The number of contour points, the number of iterations, and other
    options can all be set using the option setting function
    ``feast_option`` (see [Other Parameters for feast_option] for
    details on setting options and of the default settings).
    The search contour itself is defined by a call to
    ``feast_gen_contour`` or ``feast_custom_contour``.

    References
    ----------
    Gavin, B, Międlar, A and Polizzi, E, 2018, `FEAST Eigensolver for
    Nonlinear Eigenvalue Problems`, J. Comput. Phys. (27), 107

    Polizzi, E, 2009, `Density-Matrix-Based Algorithms for Solving
    Eigenvalue Problems`, Phys. Rev. B. (79), 115112
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.feast_poly_symm_solve'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12juft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_HANDLE, # Input, handle.
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._BYREF_INT, # Input, deg.
        utils._BYREF_COMPLEX128, # Input/Output, ze.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x[ldx, *].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, y[ldy, *].
        utils._BYREF_INT, # Input, ldy.
        utils._BYREF_INT, # Output, k.
        utils._BYREF_INT, # Input/Output, m0.
        utils._BYREF_INT, # Output, nconv.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, d[*].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._BYREF_FLOAT64, # Input/Output, eps.
        utils._BYREF_INT, # Input/Output, itera.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, resid[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (y, 'y'), (z, 'z')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _handle = utils._EngineCptrScalarType(
        handle, {'fun_name': fun_name, 'entity': 'handle'},
    ).to_ctypes()
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _deg = utils._EngineIntScalarType(
        deg, {'fun_name': fun_name, 'entity': 'deg'},
    ).to_ctypes()
    _ze = utils._EngineComplex128ScalarType(
        ze, {'fun_name': fun_name, 'entity': 'ze'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _ldx_py = utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _m0_out = utils._f12jzyn(handle, irevcm, m0)
    _m0_out_py = utils._EngineIntScalarType.to_py(_m0_out)
    _vu_f12jzy_m0_out_py = _m0_out_py
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ldy_py = utils._EngineComplex128ArrayType.get_nd(y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldy = utils._EngineIntScalarType(
        _ldy_py, {'fun_name': fun_name, 'entity': '_ldy'},
    ).to_ctypes()
    _y_cls = utils._EngineComplex128ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py, _vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _y = _y_cls.to_ctypes()
    _k = utils._EngineIntScalarType.empty_ctypes()
    _m0 = utils._EngineIntScalarType(
        m0, {'fun_name': fun_name, 'entity': 'm0'},
    ).to_ctypes()
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    _d_cls = utils._EngineComplex128ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_vu_f12jzy_m0_out_py,),
    )
    _d = _d_cls.to_ctypes()
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=(_n_py, _vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _eps = utils._EngineFloat64ScalarType(
        eps, {'fun_name': fun_name, 'entity': 'eps'},
    ).to_ctypes()
    _itera = utils._EngineIntScalarType(
        itera, {'fun_name': fun_name, 'entity': 'itera'},
    ).to_ctypes()
    _resid_cls = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_vu_f12jzy_m0_out_py,),
    )
    _resid = _resid_cls.to_ctypes()
    _routine_name = utils._EngineRoutineName(*b'f12ju ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _handle,
        _irevcm,
        _deg,
        _ze,
        _n,
        _x,
        _ldx,
        _y,
        _ldy,
        _k,
        _m0,
        _nconv,
        _d,
        _z,
        _ldz,
        _eps,
        _itera,
        _resid,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the contour setting function feast_symm_contour has not been called prior',
                    'to the first call of this function or',
                    'the supplied handle has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No eigenvalues were found within the search contour.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function did not converge after the maximum number of iterations.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The size of the eigenvector search subspace, m0, is too small.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Execution Mode\' = \'Subspace\' was set using feast_option.',
                    'Columns 1:m0 of z contain the search subspace after one contour integral',
                    'and d contains an estimate of the eigenvalues.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Execution Mode\' = \'Estimate\' was set using feast_option.',
                    'nconv contains a stochastic estimate of the number of eigenvalues',
                    'within the contour.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred in the reduced eigenvalue solver.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m0 = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 0 < m0 <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 1000:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On initial entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On intermediate entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 1, 2, 3, 4, 5 or 6.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, deg = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: deg > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 15:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Subspace\' = \'Yes\' was set using the option setting function feast_option',
                    'but no nonzero elements were found in the supplied subspace.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ze_py = utils._EngineComplex128ScalarType.to_py(_ze)
        _m0_out_py = utils._EngineIntScalarType.to_py(_m0_out)
        _vu_f12jzy_m0_out_py = _m0_out_py
        _x_cls.refresh_py(_x)
        _y_cls.refresh_py(_y)
        _m0_py = utils._EngineIntScalarType.to_py(_m0)
        _d_cls.refresh_py(_d)
        _z_cls.refresh_py(_z)
        _eps_py = utils._EngineFloat64ScalarType.to_py(_eps)
        _itera_py = utils._EngineIntScalarType.to_py(_itera)
        _resid_cls.refresh_py(_resid)
        _k_py = utils._EngineIntScalarType.to_py(_k)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['irevcm', 'ze', 'k', 'm0', 'nconv', 'eps', 'itera'],
        )(
            _irevcm_py,
            _ze_py,
            _k_py,
            _m0_py,
            _nconv_py,
            _eps_py,
            _itera_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def feast_poly_gen_solve(handle, irevcm, deg, ze, n, x, y, m0, d, z, eps, itera, resid, io_manager=None):
    u"""
    Selected eigenvalues and eigenvectors of a nonsymmetric polynomial
    eigenproblem, reverse communication driver.

    ``feast_poly_gen_solve`` is an iterative solver used to find some of
    the eigenvalues and the corresponding eigenvectors of a polynomial
    eigenvalue problem defined by complex nonsymmetric matrices.
    This is part of a suite of functions that also includes
    ``feast_init``, ``feast_option``, ``feast_gen_contour`` and
    ``feast_custom_contour``.

    For full information please refer to the NAG Library document for
    f12jv

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12jvf.html

    Parameters
    ----------
    handle : Handle
        The handle to the internal data structure used by the NAG FEAST
        suite. It needs to be initialized by ``feast_init``. **It must
        not be changed between calls to the NAG FEAST suite.**

    irevcm : int
        `On initial entry`: `irevcm` = 0, otherwise an error condition
        will be raised.

        `On intermediate entry`: must be unchanged from its previous
        exit value. Changing `irevcm` to any other value between calls
        will result in an error.

        Note: the matrices x, y and z referred to in this section are
        all of size `n`*`m0` and are stored in the arrays `x`, `y` and
        `z`, respectively.

    deg : int
        The degree, p, of the polynomial eigenvalue problem.

    ze : complex
        `On initial entry`: need not be set.

    n : int
        The order of the matrices A_i that define the eigenvalue
        problem.

    x : complex, shape (`n`, 2*m0), modified in place
        `On initial entry`: need not be set.

        `On intermediate exit`:

            if `irevcm` = 5, the calling program must compute A_k z,
            storing the result in x prior to re-entry.

            If `irevcm` = 6, the calling program must compute A_k^Hz,
            storing the result in x prior to re-entry.

        Note: the matrices x and z are stored in the first `m0` columns
        of the arrays `x` and `z`, respectively.

    y : complex, shape (`n`, m0), modified in place
        `On initial entry`: need not be set.

        `On intermediate exit`:

            if `irevcm` = 2, the calling program must compute the
            solution to the linear system (∑_i = 0^p`ze`^iA_i)w = y,
            overwriting y with the result w, prior to re-entry. The
            linear system has `m0` right-hand sides.

            If `irevcm` = 4, the calling program must compute the
            solution to the linear system (∑_i = 0^p`ze`^iA_i)^Hw = y,
            overwriting y with the result w, prior to re-entry. The
            linear system has `m0` right-hand sides.

    m0 : int
        `On initial entry`: the size of the search subspace used to find
        the eigenvalues. This should exceed the number of eigenvalues
        within the search contour. See [Further Comments] for further
        details.

        `On intermediate entry`: `m0` must remain unchanged.

    d : complex, shape (m0), modified in place
        `On initial entry`: if the option 'Subspace' = 'Yes' was set
        using the option setting function ``feast_option``, then `d`
        should contain an initial guess at the eigenvalues lying within
        the eigenvector search subspace (this subspace should be
        specified by `z`), otherwise `d` need not be set.

        `On final exit`: the first `nconv` entries in `d` contain the
        eigenvalues.

        Note: if the option 'Subspace' = 'Yes' was set using the option
        setting function ``feast_option``, then on final exit `d`
        contains an estimate of the eigenvalues after a single contour
        integral.

    z : complex, shape (`n`, 2*m0), modified in place
        `On initial entry`: if the option 'Subspace' = 'Yes' was set
        using the option setting function ``feast_option``, then `z`
        should contain an initial guess at the eigenvector search
        subspace, otherwise `z` need not be set.

        Note: if the option 'Subspace' = 'Yes' was set but only right
        eigenvectors are to be computed, then only the first `m0`
        columns of `z` need to be set. However, if the option
        'Eigenvectors' = 'Two-sided' was set using ``feast_option``, so
        that both left and right eigenvectors are returned, then the
        first `m0` columns of `z` should contain the initial guess for
        the right eigenvector search subspace and the remaining `m0`
        columns should contain an initial guess for the left eigenvector
        search subspace.

        `On intermediate exit`: must not be changed.

        `On final exit`: the first `nconv` columns of `z` contain the
        right eigenvectors corresponding to the eigenvalues found within
        the contour. If the option 'Eigenvectors' = 'Two-sided' was set
        in the option setting function ``feast_option``, the columns
        `m0`+1:`m0`+`nconv` of `z` contain the left eigenvectors
        corresponding to the eigenvalues found within the contour.

        Note: if the option 'Execution Mode' = 'Subspace' was set using
        the option setting function ``feast_option``, then on final exit
        columns 1:`m0` of `z` contain the current search subspace after
        one contour integral.

    eps : float
        `On initial entry`: need not be set.

    itera : int
        `On initial entry`: need not be set.

    resid : float, shape (2*m0), modified in place
        `On initial entry`: need not be set.

        `On final exit`: for i = 1,...,`nconv`, `resid`[i-1] contains
        the relative residual, in the 1-norm, of the ith eigenpair
        found, that is `resid`[i-1] = ||(∑_j = 0^p
        lambda_i^jA_j)z_i||/(||lambda^pA_p z_i||*(|E_mid|+r)), where
        E_mid is the centre of mass of the contour and r is the radius
        of the circle, centred at E_mid which just encloses the contour.

        If the option 'Eigenvectors' = 'Two-sided' was set in the option
        setting function ``feast_option``, the entries
        `m0`+1:`m0`+`nconv` of `resid` contain corresponding residuals
        for the left eigenvectors, `resid`[i-1] = ||(∑_j = 0^p
        lambda_i^jA_j)^Hz_i||/(||lambda^pA_p^Hz_i||*(|E_mid|+r)).

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    irevcm : int
        `On intermediate exit`: has the following meanings.

        `irevcm` = 1

            The calling program must compute a factorization of the
            matrix ∑_i = 0^p`ze`^iA_i suitable for solving a linear
            system, for example using ``sparse.complex_gen_precon_ilu``
            which computes an incomplete LU factorization of a complex
            sparse matrix. All arguments to the routine must remain
            unchanged.

            **Note:** the factorization can be computed in single
            precision.

        `irevcm` = 2

            The calling program must compute the solution to the linear
            system (∑_i = 0^p`ze`^iA_i)w = y, overwriting y with the
            result w. The matrix ∑_i = 0^p`ze`^iA_i has previously been
            factorized (when `irevcm` = 1 was returned) and this
            factorization can be reused here.

            **Note:** the solve can be performed in single precision.

        `irevcm` = 3

            Optionally, the calling program must compute a factorization
            of the matrix (∑_i = 0^p`ze`^iA_i)^H. This need only be done
            if it is not possible to use the factorization computed when
            `irevcm` = 1 was returned to solve linear systems involving
            (∑_i = 0^p`ze`^iA_i)^H. If this factorization is to be
            computed, then the factorization from `irevcm` = 1 must not
            be overwritten.

            **Note:** the factorization can be performed in single
            precision.

        `irevcm` = 4

            The calling program must compute the solution to the linear
            system (∑_i = 0^p`ze`^iA_i)^Hw = y, overwriting y with the
            result w. If it is not possible to use the factorization of
            ∑_i = 0^p`ze`^iA_i (computed when `irevcm` = 1 was returned)
            then the factorization of (∑_i = 0^p`ze`^iA_i)^H (computed
            when `irevcm` = 2 was returned) should be used here.

            **Note:** the factorization can be performed in single
            precision.

        `irevcm` = 5

            The calling program must compute A_k z, storing the result
            in x.

        `irevcm` = 6

            The calling program must compute A_k^Hz, storing the result
            in x.

        `On final exit`: `irevcm` = 0: ``feast_poly_gen_solve`` has
        completed its tasks. The value of errno determines whether the
        iteration has been successfully completed, or whether errors
        have been detected.

        Note: the matrices x, y and z referred to in this section are
        all of size `n`*`m0` and are stored in the arrays `x`, `y` and
        `z`, respectively.

    ze : complex
        `On intermediate exit`: contains the current point on the
        contour.

            If `irevcm` = 1, then this must be used by the calling
            program to form a factorization of the matrix ∑_i =
            0^p`ze`^iA_i.

            If `irevcm` = 3, then, optionally, this can be used to form
            a factorization of (∑_i = 0^p`ze`^iA_i)^H.

    k : int
        `On intermediate exit`:

            if `irevcm` = 5, denotes which of the coefficient matrices
            A_k needs to be used to form A_k z.

            If `irevcm` = 6, denotes which of the coefficient matrices
            A_k needs to be used to form A_k^Hz.

    m0 : int
        If the initial search subspace was found by
        ``feast_poly_gen_solve`` to be too large, then a new smaller
        suitable choice is returned.

    nconv : int
        The number of eigenvalues found within the search contour.

    eps : float
        The relative error on the trace. At iteration k, `eps` is given
        by the expression |trace_k-trace_{k-1}|/(|E_mid|+r), where
        trace_k is the sum of the eigenvalues found at the kth
        iteration, E_mid is the centre of mass of the contour and r is
        the radius of the circle, centred at E_mid which just encloses
        the contour.

    itera : int
        The number of subspace iterations performed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Either one of the contour setting functions
            ``feast_gen_contour`` or ``feast_custom_contour`` has not
            been called prior to the first call of this function or the
            supplied `handle` has become corrupted.

        (`errno` 7)
            An internal error occurred in the reduced eigenvalue solver.
            Please contact NAG.

        (`errno` 8)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 9)
            On entry, `m0` = *<value>* and `n` = *<value>*.

            Constraint: 0 < `m0` <= `n`.

        (`errno` 13)
            On initial entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0.

        (`errno` 13)
            On intermediate entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 1, 2, 3, 4, 5 or 6.

        (`errno` 14)
            On entry, `deg` = *<value>*.

            Constraint: `deg` > 0.

        (`errno` 15)
            The option 'Subspace' = 'Yes' was set using the option
            setting function ``feast_option`` but no nonzero elements
            were found in the supplied subspace.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            No eigenvalues were found within the search contour.

        (`errno` 3)
            The function did not converge after the maximum number of
            iterations.

        (`errno` 4)
            The size of the eigenvector search subspace, `m0`, is too
            small.

        (`errno` 5)
            The option 'Execution Mode' = 'Subspace' was set using
            ``feast_option``. Columns 1:`m0` of `z` contain the search
            subspace after one contour integral and `d` contains an
            estimate of the eigenvalues.

        (`errno` 6)
            The option 'Execution Mode' = 'Estimate' was set using
            ``feast_option``. `nconv` contains a stochastic estimate of
            the number of eigenvalues within the contour.

    Notes
    -----
    The suite of functions is designed to calculate some of the
    eigenvalues, lambda, and the corresponding eigenvectors, x, of a
    polynomial eigenvalue problem ∑_i = 0^p lambda^iA_i x = 0 where the
    coefficient matrices A_i are sparse, complex and nonsymmetric.
    The suite can also be used to find selected eigenvalues/eigenvectors
    of smaller scale, dense problems.

    ``feast_poly_gen_solve`` is a **reverse communication** function,
    based on the FEAST eigensolver, described in Polizzi (2009), which
    finds eigenvalues using contour integration.
    Prior to calling ``feast_poly_gen_solve``, one of the contour
    definition functions ``feast_gen_contour`` or
    ``feast_custom_contour`` must be used to define nodes and weights
    for a contour around a region in the complex plane within which
    eigenvalues will be sought. ``feast_gen_contour`` or
    ``feast_custom_contour`` uses this interval to define nodes and
    weights for the contour to be used by ``feast_poly_gen_solve``.

    The setup function ``feast_init`` and one of the contour definition
    functions ``feast_gen_contour`` or ``feast_custom_contour`` must be
    called before ``feast_poly_gen_solve``.
    Between the calls to ``feast_init`` and ``feast_gen_contour`` or
    ``feast_custom_contour``, options may be set by calls to the option
    setting function ``feast_option``.

    ``feast_poly_gen_solve`` uses **reverse communication**, i.e., it
    returns repeatedly to the calling program with the argument `irevcm`
    (see [Parameters]) set to specified values which require the calling
    program to carry out one of the following tasks:

    -   compute a factorization of the matrix ∑_i = 0^p`ze`^iA_i, where
        `ze` is a point on the search contour;

    -   optionally, compute a factorization of the matrix (∑_i =
        0^p`ze`^iA_i)^H (this need only be done if the factorization ∑_i
        = 0^p`ze`^iA_i does not allow linear systems involving (∑_i =
        0^p`ze`^iA_i)^H to be solved);

    -   solve a linear system involving ∑_i = 0^p`ze`^iA_i or (∑_i =
        0^p`ze`^iA_i)^H, using the factorizations above;

    -   compute the matrix product x = A_k z, for a given value of k;

    -   compute the matrix product x = A^H_k z, for a given value of k;

    -   notify the completion of the computation.

    The number of contour points, the number of iterations, and other
    options can all be set using the option setting function
    ``feast_option`` (see [Other Parameters for feast_option] for
    details on setting options and of the default settings).
    The search contour itself is defined by a call to
    ``feast_gen_contour`` or ``feast_custom_contour``.

    References
    ----------
    Gavin, B, Międlar, A and Polizzi, E, 2018, `FEAST Eigensolver for
    Nonlinear Eigenvalue Problems`, J. Comput. Phys. (27), 107

    Polizzi, E, 2009, `Density-Matrix-Based Algorithms for Solving
    Eigenvalue Problems`, Phys. Rev. B. (79), 115112

    See Also
    --------
    :meth:`naginterfaces.library.examples.sparseig.feast_poly_gen_solve_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.feast_poly_gen_solve'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12jvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_HANDLE, # Input, handle.
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._BYREF_INT, # Input, deg.
        utils._BYREF_COMPLEX128, # Input/Output, ze.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x[ldx, *].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, y[ldy, *].
        utils._BYREF_INT, # Input, ldy.
        utils._BYREF_INT, # Output, k.
        utils._BYREF_INT, # Input/Output, m0.
        utils._BYREF_INT, # Output, nconv.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, d[*].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._BYREF_FLOAT64, # Input/Output, eps.
        utils._BYREF_INT, # Input/Output, itera.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, resid[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (y, 'y'), (z, 'z')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _handle = utils._EngineCptrScalarType(
        handle, {'fun_name': fun_name, 'entity': 'handle'},
    ).to_ctypes()
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _deg = utils._EngineIntScalarType(
        deg, {'fun_name': fun_name, 'entity': 'deg'},
    ).to_ctypes()
    _ze = utils._EngineComplex128ScalarType(
        ze, {'fun_name': fun_name, 'entity': 'ze'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _ldx_py = utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _m0_out = utils._f12jzyn(handle, irevcm, m0)
    _m0_out_py = utils._EngineIntScalarType.to_py(_m0_out)
    _vu_f12jzy_m0_out_py = _m0_out_py
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, 2*_vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ldy_py = utils._EngineComplex128ArrayType.get_nd(y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldy = utils._EngineIntScalarType(
        _ldy_py, {'fun_name': fun_name, 'entity': '_ldy'},
    ).to_ctypes()
    _y_cls = utils._EngineComplex128ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py, _vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _y = _y_cls.to_ctypes()
    _k = utils._EngineIntScalarType.empty_ctypes()
    _m0 = utils._EngineIntScalarType(
        m0, {'fun_name': fun_name, 'entity': 'm0'},
    ).to_ctypes()
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    _d_cls = utils._EngineComplex128ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_vu_f12jzy_m0_out_py,),
    )
    _d = _d_cls.to_ctypes()
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=(_n_py, 2*_vu_f12jzy_m0_out_py),
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _eps = utils._EngineFloat64ScalarType(
        eps, {'fun_name': fun_name, 'entity': 'eps'},
    ).to_ctypes()
    _itera = utils._EngineIntScalarType(
        itera, {'fun_name': fun_name, 'entity': 'itera'},
    ).to_ctypes()
    _resid_cls = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(2*_vu_f12jzy_m0_out_py,),
    )
    _resid = _resid_cls.to_ctypes()
    _routine_name = utils._EngineRoutineName(*b'f12jv ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _handle,
        _irevcm,
        _deg,
        _ze,
        _n,
        _x,
        _ldx,
        _y,
        _ldy,
        _k,
        _m0,
        _nconv,
        _d,
        _z,
        _ldz,
        _eps,
        _itera,
        _resid,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either one of the contour setting functions feast_gen_contour or feast_custom_contour has not',
                    'been called prior to the first call of this function or',
                    'the supplied handle has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No eigenvalues were found within the search contour.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function did not converge after the maximum number of iterations.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The size of the eigenvector search subspace, m0, is too small.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Execution Mode\' = \'Subspace\' was set using feast_option.',
                    'Columns 1:m0 of z contain the search subspace after one contour integral',
                    'and d contains an estimate of the eigenvalues.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Execution Mode\' = \'Estimate\' was set using feast_option.',
                    'nconv contains a stochastic estimate of the number of eigenvalues',
                    'within the contour.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error occurred in the reduced eigenvalue solver.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m0 = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 0 < m0 <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 1000:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On initial entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On intermediate entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 1, 2, 3, 4, 5 or 6.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, deg = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: deg > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 15:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Subspace\' = \'Yes\' was set using the option setting function feast_option',
                    'but no nonzero elements were found in the supplied subspace.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ze_py = utils._EngineComplex128ScalarType.to_py(_ze)
        _m0_out_py = utils._EngineIntScalarType.to_py(_m0_out)
        _vu_f12jzy_m0_out_py = _m0_out_py
        _x_cls.refresh_py(_x)
        _y_cls.refresh_py(_y)
        _m0_py = utils._EngineIntScalarType.to_py(_m0)
        _d_cls.refresh_py(_d)
        _z_cls.refresh_py(_z)
        _eps_py = utils._EngineFloat64ScalarType.to_py(_eps)
        _itera_py = utils._EngineIntScalarType.to_py(_itera)
        _resid_cls.refresh_py(_resid)
        _k_py = utils._EngineIntScalarType.to_py(_k)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['irevcm', 'ze', 'k', 'm0', 'nconv', 'eps', 'itera'],
        )(
            _irevcm_py,
            _ze_py,
            _k_py,
            _m0_py,
            _nconv_py,
            _eps_py,
            _itera_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def feast_free(handle):
    u"""
    Destroy the data handle initialized by ``feast_init`` and deallocate
    all the memory used.

    ``feast_free`` part of the NAG FEAST suite.
    It is used to deallocate the memory used within the problem handle
    and to destroy the handle itself.

    For full information please refer to the NAG Library document for
    f12jz

    https://www.nag.com/numeric/nl/nagdoc_28.4/flhtml/f12/f12jzf.html

    Parameters
    ----------
    handle : Handle, modified in place
        `On entry`: the handle to the problem. It needs to be
        initialized (e.g., by ``feast_init``) and to hold a problem
        formulation compatible with ``feast_free``. It **must not** be
        changed between calls to the NAG FEAST suite.

        `On exit`: the handle is destroyed and set to a null Handle.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The supplied `handle` was not initialized or has become
            corrupted.

    Notes
    -----
    Each initialized problem handle (e.g., by ``feast_init``) should be
    deallocated to avoid memory leaks.
    Therefore, ``feast_free`` should be called on all the handles which
    are no longer needed, typically after obtaining results from the
    solver.
    Please note that passing a handle which has not been properly
    initialized might cause unpredictable behaviour, including a crash
    of your program.
    See [the F12 Introduction] for more details about the NAG FEAST
    suite.

    References
    ----------
    Polizzi, E, 2009, `Density-Matrix-Based Algorithms for Solving
    Eigenvalue Problems`, Phys. Rev. B. (79), 115112
    """

    excs = []
    fun_name = 'naginterfaces.base.sparseig.feast_free'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=_logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f12jzft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_HANDLE, # Input/Output, handle.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _handle = utils._EngineCptrScalarType(
        handle, {'fun_name': fun_name, 'entity': 'handle'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _handle,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The supplied handle was not initialized or has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = None
    _err_class.manage()
    return return_data
